//--------------------------
// CW framework <ccontrol.h>
//
// (Released under the terms
// of the GNU General Public
// License.)
//
// author:  Michael Hansen
// initiated: February 1998
//--------------------------

#ifndef _CCONTROL_H_
#define _CCONTROL_H_

#ifndef LO_INT16
#define LO_INT16(dw) ((short) (dw & 0xffff))
#endif

#ifndef HI_INT16
#define HI_INT16(dw) ((short) (dw >> 16))
#endif



//////////////////////////////////////////////////////////////
//
// CComboBox
//
class CComboBox : public CWindow
{
private:
	void InitMsgHandlers(void);
	
protected:
	virtual bool ODADrawEntire(int nIdx, long lData, int nState, CDC &dc, RECT rcItem)
			{return false;}
	virtual bool ODASelect(int nIdx, long lData, int nState, CDC &dc, RECT rcItem)
			{return false;}
	virtual bool ODAFocus(int nIdx, long lData, int nState, CDC &dc, RECT rcItem)
			{return false;}
	virtual int CompareItem(int nIdx1, long lData1, int nIdx2, long lData2){return 0;}
	virtual bool DeleteItem(int nIdx, long lData){return false;}
	virtual bool MeasureItem(int nIdx, long lData, unsigned int &nWidth,
			unsigned int &nHeight){return false;}

	long OnCompareItem(WPARAM wParam, LPARAM lParam);
	long OnDeleteItem(WPARAM wParam, LPARAM lParam);
	long OnDrawItem(WPARAM wParam, LPARAM lParam);
	long OnMeasureItem(WPARAM wParam, LPARAM lParam);
	
public:
	CComboBox(){InitMsgHandlers();}
	CComboBox(HWND hwnd) : CWindow(hwnd){InitMsgHandlers();}
	bool Create(int x, int y, int nWidth, int nHeight, int nID=0,
			DWORD dwStyle=CBS_DROPDOWN | CBS_AUTOHSCROLL | WS_VSCROLL);
	int AddItem(long lData) // for owner-drawn
			{return (int)SendMessage(CB_ADDSTRING, 0, (LPARAM)lData);}
	int AddString(const TCHAR *pszString)
			{return (int)SendMessage(CB_ADDSTRING, 0, (LPARAM)pszString);}
	int DeleteItem(int nIndex) // for owner-drawn
			{return (int)SendMessage(CB_DELETESTRING, nIndex);}
	int DeleteString(int nIndex)
			{return (int)SendMessage(CB_DELETESTRING, nIndex);}
	int Dir(unsigned int nAttrib, const TCHAR *pszFileSpec)
			{return (int)SendMessage(CB_DIR, nAttrib, (LPARAM)pszFileSpec);}
	int FindString(const TCHAR *pszString, int nStart=0)
			{return (int)SendMessage(CB_FINDSTRING, nStart, (LPARAM)pszString);}
	int FindStringExact(const TCHAR *pszString, int nStart=0)
			{return (int)SendMessage(CB_FINDSTRINGEXACT, nStart, (LPARAM)pszString);}
	int GetCount(void){return (int)SendMessage(CB_GETCOUNT);}
	int GetCurSel(void){return (int)SendMessage(CB_GETCURSEL);}
	bool GetDroppedState(void)
			{return (SendMessage(CB_GETDROPPEDSTATE)!=0)?true:false;}
#ifdef __WIN16
	void GetEditSel(int &nStart, int &nEnd)
	{
		DWORD dwRetVal = SendMessage(CB_GETEDITSEL);
		nStart = LOWORD(dwRetVal);
		nEnd = HIWORD(dwRetVal);
	}
#else
	void GetEditSel(int &nStart, int &nEnd)
			{SendMessage(CB_GETEDITSEL, (WPARAM)&nStart, (LPARAM)&nEnd);}
#endif
	long GetItemData(int nIndex){return SendMessage(CB_GETITEMDATA, nIndex);}
	void GetLBText(int nIndex, TCHAR *pszText)
			{SendMessage(CB_GETLBTEXT, (WPARAM)nIndex, (LPARAM)pszText);}
	void GetLBTextLen(int nIndex){SendMessage(CB_GETLBTEXTLEN, (WPARAM)nIndex);}
	int InsertString(int nIndex, const TCHAR *pszString)
			{return (int)SendMessage(CB_INSERTSTRING, nIndex, (LPARAM)pszString);}
	void LimitText(int nSize){SendMessage(CB_LIMITTEXT, nSize);}
	void ResetContent(void){SendMessage(CB_RESETCONTENT);}
	int SelectString(const TCHAR *pszString, int nStart=0)
			{return (int)SendMessage(CB_SELECTSTRING, nStart, (LPARAM)pszString);}
	int SetCurSel(int nIndex){return (int)SendMessage(CB_SETCURSEL, nIndex);}
	void SetEditSel(int nStart, int nEnd=-1)
			{SendMessage(CB_SETEDITSEL, 0, MAKELPARAM(nStart, nEnd));}
	long SetItemData(int nIndex, DWORD dwData)
			{return SendMessage(CB_SETITEMDATA, nIndex, dwData);}
	void ShowDropDown(bool oShow=true){SendMessage(CB_SHOWDROPDOWN, oShow);}
};
//
//
//
//////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////
//
// CListBox
//
class CListBox : public CWindow
{
private:
	void InitMsgHandlers(void);

protected:
	virtual bool ODADrawEntire(int nIdx, long lData, int nState, CDC &dc, RECT rcItem)
			{return false;}
	virtual bool ODASelect(int nIdx, long lData, int nState, CDC &dc, RECT rcItem)
			{return false;}
	virtual bool ODAFocus(int nIdx, long lData, int nState, CDC &dc, RECT rcItem)
			{return false;}
	virtual int CompareItem(int nIdx1, long lData1, int nIdx2, long lData2){return 0;}
	virtual bool DeleteItem(int nIdx, long lData){return false;}
	virtual bool MeasureItem(int nIdx, long lData, unsigned int &nWidth,
			unsigned int &nHeight){return false;}

	long OnCompareItem(WPARAM wParam, LPARAM lParam);
	long OnDeleteItem(WPARAM wParam, LPARAM lParam);
	long OnDrawItem(WPARAM wParam, LPARAM lParam);
	long OnMeasureItem(WPARAM wParam, LPARAM lParam);
	
public:
	CListBox(){InitMsgHandlers();}
	CListBox(HWND hwnd) : CWindow(hwnd){InitMsgHandlers();}
	bool Create(int x, int y, int nWidth, int nHeight, int nID=0,
			DWORD dwStyle=LBS_STANDARD)
			{return CWindow::Create(_TEXT("listbox"), x, y, nWidth, nHeight, nID, dwStyle);}
	int AddItem(long lData) // for owner-drawn
			{return (int)SendMessage(LB_ADDSTRING, 0, (LPARAM)lData);}
	int AddString(const TCHAR *pszString)
			{return (int)SendMessage(LB_ADDSTRING, 0, (LPARAM)pszString);}
	int DeleteItem(int nIndex) // for owner-drawn
			{return (int)SendMessage(LB_DELETESTRING, nIndex);}
	int DeleteString(int nIndex)
			{return (int)SendMessage(LB_DELETESTRING, nIndex);}
	int Dir(unsigned int nAttrib, const TCHAR *pszFileSpec)
			{return (int)SendMessage(LB_DIR, nAttrib, (LPARAM)pszFileSpec);}
	int FindString(const TCHAR *pszString, int nStart=0)
			{return (int)SendMessage(LB_FINDSTRING, nStart, (LPARAM)pszString);}
	int FindStringExact(const TCHAR *pszString, int nStart=0)
			{return (int)SendMessage(LB_FINDSTRINGEXACT, nStart, (LPARAM)pszString);}
	int GetCount(void){return (int)SendMessage(LB_GETCOUNT);}
	int GetCurSel(void){return (int)SendMessage(LB_GETCURSEL);}
	long GetItemData(int nIndex){return SendMessage(LB_GETITEMDATA, nIndex);}
	int InsertString(int nIndex, const TCHAR *pszString)
			{return (int)SendMessage(LB_INSERTSTRING, (WPARAM)nIndex, (LPARAM)pszString);}
	void ResetContent(void){SendMessage(LB_RESETCONTENT);}
	int SelectString(const TCHAR *pszString, int nStart=0)
			{return (int)SendMessage(LB_SELECTSTRING, nStart, (LPARAM)pszString);}
	int SetCurSel(int nIndex){return (int)SendMessage(LB_SETCURSEL, nIndex);}
	long SetItemData(int nIndex, DWORD dwData)
			{return SendMessage(LB_SETITEMDATA, nIndex, dwData);}
	int SetSel(int aIndex, BOOL aShow)
			{return SendMessage(LB_SETSEL, aShow, aIndex);}
	int GetSel(int aIndex)
			{return SendMessage(LB_GETSEL, aIndex, 0);}
};
//
//
//
//////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////
//
// CEdit
//
class CEdit : public CWindow
{
public:
	CEdit(){}
	CEdit(HWND hwnd) : CWindow(hwnd){}
	bool Create(int x, int y, int nWidth, int nHeight, int nID=0,
			DWORD dwStyle=0)
			{return CWindow::Create(_TEXT("edit"), x, y, nWidth, nHeight, nID, dwStyle);}
	bool CanUndo(void){return (SendMessage(EM_CANUNDO)!=0)?true:false;}
	void EmptyUndoBuffer(void){SendMessage(EM_EMPTYUNDOBUFFER);}
	void GetLine(int nLine, TCHAR *pszLine)
			{pszLine[SendMessage(EM_GETLINE, nLine, (LPARAM)pszLine)] = 0;}
	int GetLineCount(void){return (int)SendMessage(EM_GETLINECOUNT);}
	bool GetModify(void){return (SendMessage(EM_GETMODIFY)!=0)?true:false;}
	TCHAR GetPasswordChar(void){return (TCHAR)SendMessage(EM_GETPASSWORDCHAR);}
	void GetRect(RECT *pRect){SendMessage(EM_GETRECT, 0, (LPARAM)pRect);}
#ifdef __WIN16
	void GetSel(int &nStart, int &nEnd)
	{
		DWORD dwRetVal = SendMessage(EM_GETSEL);
		nStart = LOWORD(dwRetVal);
		nEnd = HIWORD(dwRetVal);
	}
#else
	void GetSel(int &nStart, int &nEnd)
			{SendMessage(EM_GETSEL, (WPARAM)&nStart, (LPARAM)&nEnd);}
#endif
	void LimitText(int nMaxChar)
			{SendMessage(EM_LIMITTEXT, nMaxChar);}
	int LineFromChar(int nCharIdx)
			{return (int)SendMessage(EM_LINEFROMCHAR, nCharIdx);}
	int LineIndex(int nLine)
			{return (int)SendMessage(EM_LINEINDEX, nLine);}
	int LineLength(int nCharIdx=0)
			{return (int)SendMessage(EM_LINELENGTH, nCharIdx);}
#ifdef __WIN16
	void LineScroll(int nChars, int nLines)
			{SendMessage(EM_LINESCROLL, 0, MAKELPARAM(nLines, nChars));}
#else
	void LineScroll(int nChars, int nLines)
			{SendMessage(EM_LINESCROLL, nChars, nLines);}
#endif
	void ReplaceSel(const TCHAR *pszText)
			{SendMessage(EM_REPLACESEL, 0, (LPARAM)pszText);}
	void SetModify(bool oModified){SendMessage(EM_SETMODIFY, oModified);}
	void SetPasswordChar(TCHAR cChar){SendMessage(EM_SETPASSWORDCHAR, cChar);}
	void SetReadOnly(bool oReadOnly){SendMessage(EM_SETREADONLY, oReadOnly);}
	void SetRect(const RECT *pRect){SendMessage(EM_GETRECT, 0, (LPARAM)pRect);}
#ifdef __WIN16
	void SetSel(int nStart, int nEnd=-1, bool oShowCaret=false)
			{SendMessage(EM_SETSEL, oShowCaret, MAKELPARAM(nStart, nEnd));}
#else
	void SetSel(int nStart, int nEnd=-1, bool oShowCaret=false)
	{
		SendMessage(EM_SETSEL, nStart, nEnd);
		if(oShowCaret) SendMessage(EM_SCROLLCARET);
	}
#endif
	void SetTabStops(int nCount, const int *pnTabStops)
			{SendMessage(EM_SETTABSTOPS, nCount, (LPARAM)pnTabStops);}
	void Undo(void){SendMessage(EM_UNDO);}
};
//
//
//
//////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////
//
// CButton
//
class CButton : public CWindow
{
private:
	void InitMsgHandlers(void);
	
protected:
	virtual bool ODADrawEntire(int nIdx, long lData, int nState, CDC &dc, RECT rcItem)
			{return false;}
	virtual bool ODASelect(int nIdx, long lData, int nState, CDC &dc, RECT rcItem)
			{return false;}
	virtual bool ODAFocus(int nIdx, long lData, int nState, CDC &dc, RECT rcItem)
			{return false;}
	virtual bool MeasureItem(int nIdx, long lData, unsigned int &nWidth,
			unsigned int &nHeight){return false;}

	long OnDrawItem(WPARAM wParam, LPARAM lParam);
	long OnMeasureItem(WPARAM wParam, LPARAM lParam);

public:
	CButton(){InitMsgHandlers();}
	CButton(HWND hwnd) : CWindow(hwnd){InitMsgHandlers();}
	bool Create(int x, int y, int nWidth, int nHeight, int nID=0,
			DWORD dwStyle=BS_PUSHBUTTON)
			{return CWindow::Create(_TEXT("button"), x, y, nWidth, nHeight, nID, dwStyle);}
#ifdef __WIN16
	void Click(void){SendMessage(WM_LBUTTONDOWN); SendMessage(WM_LBUTTONUP);}
#else
	void Click(void){SendMessage(BM_CLICK);}
#endif
	int GetCheck(void){return (int)SendMessage(BM_GETCHECK);}
#ifdef __WINV40
	HANDLE GetImage(void){return (HANDLE)SendMessage(BM_GETIMAGE);}
#endif
	bool IsPushed(void){return (SendMessage(BM_GETSTATE)&0x04!=0)?true:false;}
	void SetCheck(int nCheck){SendMessage(BM_SETCHECK, nCheck);}
#ifdef __WINV40
	HANDLE SetImage(HANDLE hImage)
			{return (HANDLE)SendMessage(BM_SETIMAGE, 0, (LPARAM)hImage);}
#endif
	HICON SetIcon(HICON hIcon)
			{return (HICON)SendMessage(BM_SETIMAGE, IMAGE_ICON, (LPARAM)hIcon);}
	void SetPush(bool oPush){SendMessage(BM_SETSTATE, oPush);}
	void SetStyle(long lStyle, bool oRedraw=true)
			{SendMessage(BM_SETSTYLE, (WPARAM)LOWORD(lStyle), MAKELPARAM(oRedraw, 0));}
};
//
//
//
//////////////////////////////////////////////////////////////



//-------------------------------------------
// Extended common controls -- post Windows95
//-------------------------------------------

#ifdef __WINV40

#include <commctrl.h>
#include <richedit.h>

//////////////////////////////////////////////////////////////
//
// CTabControl
//
class CTabControl : public CWindow
{
public:
	CTabControl(){}
	CTabControl(HWND hwnd) : CWindow(hwnd){}
	bool Create(int x, int y, int nWidth, int nHeight, int nID=0, DWORD dwStyle=0)
			{return CWindow::Create(WC_TABCONTROL, x, y, nWidth, nHeight, nID, dwStyle);}
	void AdjustRect(BOOL oLarger, RECT *pRect)
			{TabCtrl_AdjustRect(HWindow, oLarger, pRect);}
	BOOL DeleteAllItems(void){return TabCtrl_DeleteAllItems(HWindow);}
	BOOL DeleteItem(int nItem){return TabCtrl_DeleteItem(HWindow, nItem);}
	int GetCurFocus(void){return TabCtrl_GetCurFocus(HWindow);}
	int GetCurSel(void){return TabCtrl_GetCurSel(HWindow);}
	HIMAGELIST GetImageList(void){return TabCtrl_GetImageList(HWindow);}
	BOOL GetItem(int nItem, TC_ITEM *pItem){return TabCtrl_GetItem(HWindow, nItem, pItem);}
	int GetItemCount(void){return TabCtrl_GetItemCount(HWindow);}
	BOOL GetItemRect(int nItem, RECT *pRect)
			{return TabCtrl_GetItemRect(HWindow, nItem, pRect);}
	int GetRowCount(void){return TabCtrl_GetRowCount(HWindow);}
	int HitTest(TC_HITTESTINFO *pInfo){return TabCtrl_HitTest(HWindow, pInfo);}
	int InsertItem(int nItem, const TC_ITEM *pItem)
			{return TabCtrl_InsertItem(HWindow, nItem, pItem);}
	void SetCurFocus(int nItem){TabCtrl_SetCurFocus(HWindow, nItem);}
	int SetCurSel(int nItem){return TabCtrl_SetCurSel(HWindow, nItem);}
	HIMAGELIST SetImageList(HIMAGELIST hList){return TabCtrl_SetImageList(HWindow, hList);}
	BOOL SetItem(int nItem, TC_ITEM *pItem){return TabCtrl_SetItem(HWindow, nItem, pItem);}
	BOOL SetItemExtra(int nBytes){return TabCtrl_SetItemExtra(HWindow, nBytes);}
	DWORD SetItemSize(int x, int y){return TabCtrl_SetItemSize(HWindow, x, y);}
	int SetMinTabWidth(int nWidth){return TabCtrl_SetMinTabWidth(HWindow, nWidth);}
	void SetPadding(int cx, int cy){TabCtrl_SetPadding(HWindow, cx, cy);}

	int InsertTextItem(TCHAR *pszText, int nItem, HWND hTemplate=0);
	int InsertImageItem(int nImage, int nItem, HWND hTemplate=0);
	int SelectTab(int nItem);
	void SizeTemplates(void);
	HWND GetTemplate(int nItem)
	{
		TC_ITEM Item;
		Item.mask = TCIF_PARAM;
		TabCtrl_GetItem(HWindow, nItem, &Item);
		return (HWND)Item.lParam;
	}
};
//
//
//
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
//
// CListView
//

#define WM_USER_ITEM_DBLCLICKED (WM_USER + 1)

class CListView : public CWindow
{
private:
	void InitMsgHandlers(void);
	long OnWmLButtonDblclk(WPARAM aWParam, LPARAM aLParam){OnLButtonDblclk(aWParam, LO_INT16(aLParam), HI_INT16(aLParam)); return 1;}
	void OnLButtonDblclk(int fwKeys, int aX, int aY);

public:
	CListView(){InitMsgHandlers();}
	CListView(HWND hwnd) : CWindow(hwnd){InitMsgHandlers();}
	bool Create(int x, int y, int nWidth, int nHeight, int nID=0,
			DWORD dwStyle=LVS_REPORT | LVS_SINGLESEL)
			{return CWindow::Create(WC_LISTVIEW, x, y, nWidth, nHeight, nID, dwStyle);}
	BOOL DeleteAllItems(void){return ListView_DeleteAllItems(HWindow);}
	BOOL DeleteColumn(int nCol){return ListView_DeleteColumn(HWindow, nCol);}
	BOOL DeleteItem(int nItem){return ListView_DeleteItem(HWindow, nItem);}
	BOOL GetColumn(int nCol, LV_COLUMN *pColumn)
			{return ListView_GetColumn(HWindow, nCol, pColumn);}
	HIMAGELIST GetImageList(int nType){return ListView_GetImageList(HWindow, nType);}
	BOOL GetItem(LV_ITEM *pItem){return ListView_GetItem(HWindow, pItem);}
	int GetItemCount(void){return ListView_GetItemCount(HWindow);}
	BOOL GetItemRect(int nItem, RECT *pRect, int nCode)
			{return ListView_GetItemRect(HWindow, nItem, pRect, nCode);}
	unsigned int GetItemState(int nItem, unsigned int nMask=LVIS_SELECTED)
			{return ListView_GetItemState(HWindow, nItem, nMask);}
	void GetItemText(int nItem, int nSubItem, TCHAR *pszText, int nSize)
			{ListView_GetItemText(HWindow, nItem, nSubItem, pszText, nSize);}
	int GetSelectedCount(void){return ListView_GetSelectedCount(HWindow);}
	int HitTest(LVHITTESTINFO *pInfo){return ListView_HitTest(HWindow, pInfo);}
	int InsertColumn(int nCol, const LV_COLUMN *pColumn)
			{return ListView_InsertColumn(HWindow, nCol, pColumn);}
	int InsertItem(const LV_ITEM *pItem)
			{return ListView_InsertItem(HWindow, pItem);}
	BOOL SetColumn(int nCol, LV_COLUMN *pColumn)
			{return ListView_SetColumn(HWindow, nCol, pColumn);}
	HIMAGELIST SetImageList(HIMAGELIST hList, int nType)
			{return ListView_SetImageList(HWindow, hList, nType);}
	BOOL SetItem(LV_ITEM *pItem){return ListView_SetItem(HWindow, pItem);}
	void SetItemCount(int nItems){ListView_SetItemCount(HWindow, nItems);}
	void SetItemCountEx(int nItems, DWORD dwFlags)
			{ListView_SetItemCountEx(HWindow, nItems, dwFlags);}
	void SetItemState(int nItem, unsigned int nState=LVIS_SELECTED,
			unsigned int nMask=LVIS_SELECTED)
			{ListView_SetItemState(HWindow, nItem, nState, nMask);}
	void SetItemText(int nItem, int nSubItem, TCHAR *pszText)
			{ListView_SetItemText(HWindow, nItem, nSubItem, pszText);}

	int InsertTextColumn(TCHAR *pszText, int nCol, int nWidth);
	int InsertTextItem(TCHAR *pszText, int nItem, LPARAM lParam=-1);
	long GetItemData(int nItem);
	void SetItemData(int nItem, long lData);

	int GetFirstSelectedItem();
	int SetImage(int nItem, int aImageIdex);

};
//
//
//
//////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////
//
// CTreeView
//
class CTreeView : public CWindow
{
public:
	CTreeView(){}
	CTreeView(HWND hwnd) : CWindow(hwnd){}
	bool Create(int x, int y, int nWidth, int nHeight, int nID=0,
			DWORD dwStyle=0)
			{return CWindow::Create(WC_TREEVIEW, x, y, nWidth, nHeight, nID, dwStyle);}
	BOOL DeleteAllItems(void){return TreeView_DeleteAllItems(HWindow);}
	BOOL DeleteItem(HTREEITEM hItem){return TreeView_DeleteItem(HWindow, hItem);}
	HWND EditLabel(HTREEITEM hItem){return TreeView_EditLabel(HWindow, hItem);}
	BOOL EndEditLabelNow(BOOL oCancel=true)
			{return TreeView_EndEditLabelNow(HWindow, oCancel);}
	BOOL EnsureVisible(HTREEITEM hItem){return TreeView_EnsureVisible(HWindow, hItem);}
	BOOL Expand(HTREEITEM hItem, unsigned int nFlag)
			{return TreeView_Expand(HWindow, hItem, nFlag);}
	HTREEITEM GetChild(HTREEITEM hItem){return TreeView_GetChild(HWindow, hItem);}
	int GetCount(void){return TreeView_GetCount(HWindow);}
	HWND GetEditControl(void){return TreeView_GetEditControl(HWindow);}
	HTREEITEM GetFirstVisible(void){return TreeView_GetFirstVisible(HWindow);}
	HIMAGELIST GetImageList(int nType){return TreeView_GetImageList(HWindow, nType);}
	int GetIndent(void){return TreeView_GetIndent(HWindow);}
	BOOL GetISearchString(TCHAR *pszString)
			{return TreeView_GetISearchString(HWindow, pszString);}
	BOOL GetItem(TVITEM *pItem){return TreeView_GetItem(HWindow, pItem);}
	BOOL GetItemRect(HTREEITEM hItem, RECT *pRect, BOOL oItemRect)
			{return TreeView_GetItemRect(HWindow, hItem, pRect, oItemRect);}
	HTREEITEM GetNextItem(HTREEITEM hItem, unsigned int nFlag)
			{return TreeView_GetNextItem(HWindow, hItem, nFlag);}
	HTREEITEM GetNextSibling(HTREEITEM hItem)
			{return TreeView_GetNextSibling(HWindow, hItem);}
	HTREEITEM GetNextVisible(HTREEITEM hItem)
			{return TreeView_GetNextVisible(HWindow, hItem);}
	HTREEITEM GetParent(HTREEITEM hItem){return TreeView_GetParent(HWindow, hItem);}
	HTREEITEM GetPrevSibling(HTREEITEM hItem)
			{return TreeView_GetPrevSibling(HWindow, hItem);}
	HTREEITEM GetPrevVisible(HTREEITEM hItem)
			{return TreeView_GetPrevVisible(HWindow, hItem);}
	HTREEITEM GetRoot(void){return TreeView_GetRoot(HWindow);}
	HTREEITEM GetSelection(void){return TreeView_GetSelection(HWindow);}
	int GetVisibleCount(void){return TreeView_GetVisibleCount(HWindow);}
	HTREEITEM HitTest(TVHITTESTINFO *pInfo){return TreeView_HitTest(HWindow, pInfo);}
	HTREEITEM InsertItem(TVINSERTSTRUCT *pItem)
			{return TreeView_InsertItem(HWindow, pItem);}
#if(_MSC_VER >= 1200) // specific to Visual C++ 6.0
	int Select(HTREEITEM hItem, unsigned int nFlag)
			{return TreeView_Select(HWindow, hItem, nFlag);}
	int SelectItem(HTREEITEM hItem){return TreeView_SelectItem(HWindow, hItem);}
	int SelectSetFirstVisible(HTREEITEM hItem)
			{return TreeView_SelectSetFirstVisible(HWindow, hItem);}
#else
	HTREEITEM Select(HTREEITEM hItem, unsigned int nFlag)
			{return TreeView_Select(HWindow, hItem, nFlag);}
	HTREEITEM SelectItem(HTREEITEM hItem){return TreeView_SelectItem(HWindow, hItem);}
	HTREEITEM SelectSetFirstVisible(HTREEITEM hItem)
			{return TreeView_SelectSetFirstVisible(HWindow, hItem);}
#endif
	HIMAGELIST SetImageList(HIMAGELIST hList, int nType)
			{return TreeView_SetImageList(HWindow, hList, nType);}
	BOOL SetIndent(int nIndent){return TreeView_SetIndent(HWindow, nIndent);}
	BOOL SetItem(TVITEM *pItem){return TreeView_SetItem(HWindow, pItem);}
	BOOL SortChildren(HTREEITEM hItem)
			{return TreeView_SortChildren(HWindow, hItem, 0);}
	BOOL SortChildrenCB(TVSORTCB *pSortFunc)
			{return TreeView_SortChildrenCB(HWindow, pSortFunc, 0);}

	BOOL GetItemText(HTREEITEM hItem, TCHAR *pszText, int nTextMax);
	HTREEITEM InsertTextItem(TCHAR *pszText, HTREEITEM hParent=0,
			HTREEITEM hAfter=TVI_SORT, LPARAM lParam=-1);
	long GetItemData(HTREEITEM hItem);
	void SetItemData(HTREEITEM hItem, long lData);
};
//
//
//
//////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////
//
// CRichEdit
//
class CRichEdit : public CEdit
{
private:
	const char *m_pText;
	int m_nTextCount;

	unsigned long Read(unsigned char *pbBuffer, long lCount, long *plBytesRead);
	unsigned long Write(unsigned char *pbBuffer, long lCount, long *plBytesRead);

public:
	CRichEdit(){LoadLibrary(_TEXT("riched32.dll"));} // load RICHED32.DLL to register class
	CRichEdit(HWND hwnd) : CEdit(hwnd){} // RICHED32.DLL must already be loaded here
	// Windows version 4.0 and above will free library when process terminates
	bool Create(int x, int y, int nWidth, int nHeight, int nID=0,
			DWORD dwStyle=0)
			{return CWindow::Create(_TEXT("richedit"), x, y, nWidth, nHeight, nID, dwStyle);}
	void SetText(const char *pszText);
	void SetRTFText(const char *pszText);

	friend DWORD CALLBACK EditReadStreamCallback(DWORD, LPBYTE, long, long *);
	friend DWORD CALLBACK EditWriteStreamCallback(DWORD, LPBYTE, long, long *);
};
//
//
//
//////////////////////////////////////////////////////////////

#endif //__WINV40

#endif
