/*******************************************************************
 *
 *	File:		CDeskwinEngine.h
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 1999, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/


#ifndef __CDESKWIN_ENGINE_H
#define __CDESKWIN_ENGINE_H

#include "TStickyWindow.h"
#include "MDesktopEngineObserver.h"

typedef list<TWindow>		TWindowList;
typedef vector<TWindowList>	TWindowLists;
typedef vector<TWindow>		TWindowArray;

typedef list<MDesktopEngineObserver*>		TDesktopEngineObservers;

const TCHAR	KExplorerName[] = "Program Manager";
const TCHAR	KTaskbarClassName[] = "Shell_TrayWnd";
const TCHAR	KStartMenuClassName[] = "DV2ControlHost";
const TCHAR	KToolTipClassName[] = "tooltips_class32";

const TCHAR KInternetExplorerClassName[] = "IFRAME";
const TCHAR KExplorerClassName[] = "CabinetWClass";


class CDeskwinEngine
{
public:
	virtual ~CDeskwinEngine();
	static CDeskwinEngine* Instance();

		/*
		 * static information
		 */
	static void		DefaultWindowRule(TWindow aWindow, TStickyWindow& aStickyWindow);
	static void		FormatStickyDesktopList(const TStickyWindow& aStickyWindow, string& aString);
	static void		GetWindowName(TWindow aWindow, string& aName);
	static void		GetClassName(TWindow aWindow, string& aName);
	static TWindow	GetExplorerWindow();
	static TWindow	GetTaskbarWindow();
	static TWindow	GetStartMenuWindow();


		/*
		 * Volume
		 */
	void VolumeToggleMute();
	void VolumeUp();
	void VolumeDown();
	
		/*
		 * Desktop management
		 */
	void	SetNoOfDesktops(int aNoOfDesktops);
	int		NoOfDesktops() { return iNoOfDesktops;}
	int		CurrentDesktop();
	void	SwitchTo(int aDesktop, TWindow aDragWindow = NULL);
	void	MinimiseAll(const TWindow aDeskwinWindow);

		/*
		 * window lists
		 */
	void	GetAllWindows(TWindowList& aWindowList);
	void	GetWindowList(int aDesktop, TWindowList& aWindowList);

		/*
		 * Sticky windows
		 */
	const TStickyWindowInfo&	StickyWindowInfo();
	void						SetStickyWindowInfo(const TStickyWindowInfo&);
	bool						IsStickyWindow(int aDesktop, TWindow aWindow);
	void						IsStickyWindow1(int aDesktop, TWindow aWindow, bool& aRuleMatch, bool& aIsSticky, int& aRuleNo);
	void						HandleStickyWindowInfoChanged();

	void						HideOrShowExplorerOrDesktop(int aDesktop);

	void AddObserver(MDesktopEngineObserver& aDesktopEngineObserver);
	void RemoveObserver(MDesktopEngineObserver& aDesktopEngineObserver);

	bool IsShellWorkerWindowVisible();
	bool IsShellToolTipVisible();
	bool IsStartMenuVisible();

	void PrepareForExit();

protected:
			CDeskwinEngine();
	void	ShowDesktop(TWindowList& aWindowList, TWindow& aForGroundWindow, TWindowList& aWindowListToShow);
	void	HideDesktop(TWindowList& aWindowList, int aDesktop, TWindow aDragWindow, TWindowList& aWindowListToHide);
	void	CaptureDesktop(TWindowList& aWindowList, TWindow& aForGroundWindow, bool aWorkAroundStupidWindows = false);
	void	GetRidOfWindowsWhenSwitching();

	void	WindowDied(TWindow aWindow);
	void	InitNewDesktops(int aOldNoODesktops, int aNewNoODesktops);

	void	HideOrShowWindow(int aDesktop, TWindow aWindow);
	void	MyShowWindow(TWindow aWindow, bool aShow);
	void	WaitUntilWindowsAreUpdated(TWindowList& aWindowListHidden, TWindowList& aWindowListShown);
	void	MonitorHungWindows(TWindowList& aWindowListHidden);

	static CDeskwinEngine* iDeskwinEngine;

	TWindowLists			iWindowLists;		// window list per desktop
	TWindowArray			iActiveWindows;		// active window per desktop

	int						iNoOfDesktops;
	int						iCurrentDesktop;

	TStickyWindowInfo		iStickyWindowInfo;
	TDesktopEngineObservers	iDesktopEngineObservers;


		/*
		 * Window persistency
		 */
	void RestoreWindowState();
	void StoreWindowState(TWindow aWindow, bool aShow);
	void DeleteStoredWindowState();


#ifdef _DEBUG

#define DUMPSTATE(me) me->DumpState();
#define DUMP_WINDOW_LIST(me, windowlist) me->DumpWindowList(windowlist);

public:
	void DumpState();
	void DumpWindowList(TWindowList& aWindowlist);
#else

#define DUMPSTATE(me)
#define DUMP_WINDOW_LIST(me, windowlist)

#endif



};


/*******************************************************************
 *
 * $Log: /Deskwin/Src/CDeskwinEngine.h $
 * 
 * 4     10/01/00 9:17 Pese
 * 
 * 3     5/01/00 20:26 Pese
 * 
 * 2     23/12/99 10:16 Pese
 *
 *******************************************************************/



#endif			/* __CDESKWIN_ENGINE_H */