/*******************************************************************
 *
 *	File:		CDeskwinWindow.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 1999, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#include "Deskwin.h"
#include "CDeskWinWindow.h"
#include "CDeskWinEngine.h"
#include "CWindowPositionTracker.h"
#include "COptionDialogs.h"

#include "dll.h"

#include "Resource.h"

#undef HDS_HORZ
#undef HDS_BUTTONS
#undef HDS_HIDDEN

#include <winresrc.h>				// for cursors

#include "regutil.h"

const KHotkeyToDesktop0			= 100;

const KHotkeyToggleExplorer		= 0;
const KHotkeyNextDesktop		= 1;
const KHotkeyPreviousDesktop	= 2;
const KHotkeyDesktopUp			= 3;
const KHotkeyDesktopDown		= 4;
const KHotkeyMinimiseAll		= 5;
const KHotkeyShowDeskwin		= 6;
const KHotkeyVolumeToggleMute	= 7;
const KHotkeyVolumeUp			= 8;
const KHotkeyVolumeDown			= 9;

#define MAX_TEXT 4096

#if 0
CDesktopNumberWindow::CDesktopNumberWindow() : CWindowEx(), iPen(ct_Black, 1), iBrush(ct_LightGray)
{
	AddMsg(SWinMsg(WM_TIMER, (CWMSGHANDLER)CDesktopNumberWindow::OnWmTimer));
}

void CDesktopNumberWindow::OnTimer()
{
	RECT	rect;
	DPRINT((HERE "CDesktopNumberWindow::OnTimer"));
	
	KillTimer(0);

	ShowWindow(SW_HIDE);
}


void CDesktopNumberWindow::Construct()
{
	// need our own DC, so register a new class
	GetApplication()->RegisterClass("CDesktopNumberWindow", NULL, NULL, -1, 0, CS_HREDRAW | CS_VREDRAW | CS_OWNDC | CS_SAVEBITS);

	bool result = Create("CDesktopNumberWindow", 0, 0, 1, 1, 0, WS_POPUP, NULL, WS_EX_TRANSPARENT | WS_EX_TOOLWINDOW);
	if (result)
	{
		HFONT font = CreateFont(
			80 * GetDeviceCaps(GetDC(), LOGPIXELSY) / 72,
			0,
			0,
			0,
			FW_NORMAL,
			1,
			0,
			0,
			DEFAULT_CHARSET,
			OUT_DEFAULT_PRECIS,
			CLIP_DEFAULT_PRECIS,
			PROOF_QUALITY,
			DEFAULT_PITCH | FF_DONTCARE,
			"Times New Roman"
			);
		::SelectObject(GetDC(), font);
		SetBkMode(GetDC(), TRANSPARENT);

		::SelectObject(GetDC(), iPen);
		::SelectObject(GetDC(), iBrush);
	}
}

void CDesktopNumberWindow::Paint(CDC& aDc)
{
	char	text[128];
	RECT	rect;

	DPRINT((HERE "CDesktopNumberWindow::Paint"));

	GetWindowText(text, 128);
	text[127] = 0;

	GetClientRect(&rect);
//	DPRINT((HERE "CDesktopNumberWindow::Paint, rect.w=%d, rect.h=%d", rect.right, rect.bottom));

	BeginPath(aDc);
	aDc.DrawText(text, strlen(text), &rect, DT_SINGLELINE);
	EndPath(aDc);

	StrokeAndFillPath(aDc);
}

void CDesktopNumberWindow::SetDesktop(int aDesktop)
{
	SIZE	size;
	char	text[128];
	long	screenX, screenY;
	long	windowX, windowY;
	
	KillTimer(0);
		
	sprintf(text, "Desktop %d", aDesktop + 1);
	::GetTextExtentPoint32(GetDC(), text, strlen(text), &size);
	DPRINT((HERE "CDesktopNumberWindow::SetDesktop, w=%d, h=%d", size.cx, size.cy));
	SetWindowText(text);

		/*
		 * Center window
		 */
	GetScreenSize(screenX, screenY);
	windowX = (screenX >> 1) - (size.cx >> 1);
	windowY = (screenY >> 1) - (size.cy >> 1);
	
	SetWindowPos(HWND_TOPMOST, windowX, windowY, size.cx, size.cy, 0);
	ShowWindow(SW_SHOW);
	
	SetTimer(0, 2500);
}
#endif


CDeskwinWindow::CDeskwinWindow() : CDeskwinWindowBase(), 
	iActiveDesktop(0), iIsDocked(false), iPreviousDockingPlace(EDockNowhere), 
	iState(EStateNormal), iMenu(IDR_MENU_DESKWIN), iMenuAlreadyRunning(false)
{
	// initialize message handlers
	AddMsg(SWinMsg(WM_LBUTTONDOWN, (CWMSGHANDLER)CDeskwinWindow::OnWmLButtonDown));
	AddMsg(SWinMsg(WM_RBUTTONDOWN, (CWMSGHANDLER)CDeskwinWindow::OnWmRButtonDown));
	AddMsg(SWinMsg(WM_RBUTTONUP, (CWMSGHANDLER)CDeskwinWindow::OnWmRButtonUp));
	AddMsg(SWinMsg(WM_LBUTTONUP, (CWMSGHANDLER)CDeskwinWindow::OnWmLButtonUp));
	AddMsg(SWinMsg(WM_MOUSEMOVE, (CWMSGHANDLER)CDeskwinWindow::OnWmMouseMove));
	AddMsg(SWinMsg(WM_CAPTURECHANGED, (CWMSGHANDLER)CDeskwinWindow::OnWmCaptureChanged));
	AddMsg(SWinMsg(WM_DISPLAYCHANGE, (CWMSGHANDLER)CDeskwinWindow::OnWmDisplayChanged));
	AddMsg(SWinMsg(WM_TIMER, (CWMSGHANDLER)CDeskwinWindow::OnWmTimer));
	AddMsg(SWinMsg(WM_HOTKEY, (CWMSGHANDLER)CDeskwinWindow::OnWmHotkey));
	AddMsg(SWinMsg(WM_WINDOW_POS_CHANGED, (CWMSGHANDLER)CDeskwinWindow::OnWmWindowPosChanged));
	AddMsg(SWinMsg(WM_WINDOW_CREATED, (CWMSGHANDLER)CDeskwinWindow::OnWmWindowCreated));
	AddMsg(SWinMsg(WM_SYSTRAY_EVENT, (CWMSGHANDLER)CDeskwinWindow::OnWmSysTrayEvent));

		/*
		 * Set up defaults. Each state calls NormalPaint for EEventPaint and WindowCreated
		 * for EEventWindowCreated; all other handlers are set to IgnoreEvent.
		 */
	DefaultEventHandlers();

		/*
		 * Selectively override handlers (per state)
		 */	
	SetEventHandler(EStateDragMoveMyself, EEventLMouseUp, DragMoveMyselfDone);
	SetEventHandler(EStateDragMoveMyself, EEventMouseMove, DoDragMoveMyself);

	SetEventHandler(EStateDragMoveWindow, EEventLMouseUp, DragMoveWindowDone);
	SetEventHandler(EStateDragMoveWindow, EEventMouseMove, DoDragMoveWindow);
	SetEventHandler(EStateDragMoveWindow, EEventWindowCreated, IgnoreEvent);

	SetEventHandler(EStateDragResizeIntention, EEventLMouseDown, StartDragging);
	SetEventHandler(EStateDragResizeIntention, EEventMouseMove, DoDragResizeIntention);

	SetEventHandler(EStateDragResize, EEventLMouseUp, DragResizeDone);
	SetEventHandler(EStateDragResize, EEventMouseMove, DoDragResize);

	SetEventHandler(EStateDragMoreDesktops, EEventLMouseUp, DragMoreDesktopsDone);
	SetEventHandler(EStateDragMoreDesktops, EEventMouseMove, DoDragMoreDesktops);
	SetEventHandler(EStateDragMoreDesktops, EEventPaint, ResizeNoOfDesktopsPaint);

	SetEventHandler(EStateTooltip, EEventRMouseUp, ShowPopupMenu);
	SetEventHandler(EStateTooltip, EEventLMouseDown, NormalMouseDown);
	SetEventHandler(EStateTooltip, EEventLMouseUp, NormalMouseUp);
	SetEventHandler(EStateTooltip, EEventMouseMove, NormalMouseMouse);
	SetEventHandler(EStateTooltip, EEventTimeout, StartFadingOut);

	SetEventHandler(EStateNormal, EEventRMouseUp, ShowPopupMenu);
	SetEventHandler(EStateNormal, EEventLMouseDown, NormalMouseDown);
	SetEventHandler(EStateNormal, EEventLMouseUp, NormalMouseUp);
	SetEventHandler(EStateNormal, EEventMouseMove, NormalMouseMouse);
	SetEventHandler(EStateNormal, EEventTimeout, StartFadingOut);

	SetEventHandler(EStateFadingOut, EEventSmoothMoveDone, FadingOutDone);

	SetEventHandler(EStateFadingIn, EEventSmoothMoveDone, FadingInDone);

	SetEventHandlerForHiddenState();

	iFadeMove.x = iFadeMove.y = 0;

	iCursorNeSw = ::LoadCursor(NULL, MAKEINTRESOURCE(OCR_SIZENESW)); 
	iCursorNwSe = ::LoadCursor(NULL, MAKEINTRESOURCE(OCR_SIZENWSE)); 
	iCursorMove = ::LoadCursor(NULL, IDC_SIZEALL); 
	iCursorMoreDesktops = ::LoadCursor(GetAppInstance(), MAKEINTRESOURCE(IDC_CURSOR_MOREDESKTOPS));

	iOptions.iHotkeyShowDeskwin			= 0;
	iOptions.iHotkeyNextDesktop			= 0;
	iOptions.iHotkeyPreviousDesktop		= 0;
	iOptions.iHotkeyDesktopUp			= 0;
	iOptions.iHotkeyDesktopDown			= 0;
	iOptions.iHotkeyToggleExplorer		= 0;
	iOptions.iHotkeyMinimiseAll			= 0;
	iOptions.iHotkeyVolumeToggleMute	= 0;
	iOptions.iHotkeyVolumeUp			= 0;
	iOptions.iHotkeyVolumeDown			= 0;
}


void CDeskwinWindow::Destroy()
{
	SaveOptions();
	SaveState();

	iDeskwinEngine->PrepareForExit();

	CWindowEx::Destroy();

	PostQuitMessage(0);
}


CDeskwinWindow::~CDeskwinWindow()
{
	delete iWindowPositionTracker;

	delete iSmoothWindowMover;
	iSmoothWindowMover = NULL;

	if (iDeskwinEngine && iSysTrayIconController)
		iDeskwinEngine->RemoveObserver(*iSysTrayIconController);

	delete iSysTrayIconController;
	iSysTrayIconController = NULL;

	iCursorNeSw = NULL;
	iCursorNwSe = NULL;
}


void CDeskwinWindow::DefaultEventHandlers()
{
	for (int s=0 ; s<EStateLastEntry ; s++)
	{
		for (int e=0; e<EEventLastEntry ; e++)
			iEventHandlers[s][e] = &CDeskwinWindow::IgnoreEvent;
		iEventHandlers[s][EEventPaint] = &CDeskwinWindow::NormalPaint;
		iEventHandlers[s][EEventWindowCreated] = &CDeskwinWindow::WindowCreated;
	}
}

void CDeskwinWindow::SetEventHandler(TState aState, TEvent aEvent, TEventHandler aEventHandler)
{
	iEventHandlers[aState][aEvent] = aEventHandler;
}


void CDeskwinWindow::SetEventHandlerForHiddenState()
{
	if (iOptions.iWantsShowByMouseClick)
	{
		SetEventHandler(EStateHidden, EEventRMouseUp, ShowPopupMenu);
		SetEventHandler(EStateHidden, EEventLMouseUp, StartFadingIn);
		SetEventHandler(EStateHidden, EEventMouseMove, IgnoreEvent);
	}
	else
	{
		SetEventHandler(EStateHidden, EEventRMouseUp, IgnoreEvent);
		SetEventHandler(EStateHidden, EEventLMouseUp, IgnoreEvent);
		SetEventHandler(EStateHidden, EEventMouseMove, StartFadingIn);
	}
}


void CDeskwinWindow::IgnoreEvent(TEvent aEvent)
{
}


void CDeskwinWindow::HandleEvent(TEvent aEvent)
{
	(this->*iEventHandlers[iState][aEvent])(aEvent);
}



void CDeskwinWindow::DragMoveMyselfDone(TEvent aEvent)
{
	StopDraggingMyself();

	SaveOptions();
	SaveState();
}


void CDeskwinWindow::DoDragMoveMyself(TEvent aEvent)
{
	RECT		screenrect;
	GetWindowRect(&screenrect);

//	ASSERT(IsInRange(iEvent.iMouse.x, 0, (int) iScreenSize.x));
//	ASSERT(IsInRange(iEvent.iMouse.y, 0, (int) iScreenSize.y));

	int diffX = screenrect.left + iEvent.iMouse.x - iDragStartX;
	int diffY = screenrect.top + iEvent.iMouse.y - iDragStartY;
	int newX = iDragRect.left + diffX;
	int newY = iDragRect.top + diffY;

	DPRINT((HERE "DoDragMoveMyself, %i, %i, %i, %i", iEvent.iMouse.x, iEvent.iMouse.y, newX, newY));

	MoveWindow(newX, newY, iWindowSize.x, iWindowSize.y);
	iIsDocked = false;		// no longer docked

		/*
		 * Let's see is we have to dock the window at its new postition
		 * Only dock if to a docking place, if the previous dock place was
		 * not the same!
		 */
	if (iOptions.iWantsDocking)
	{
		TDockingPlace newDockingPlace = GetDockingPlace();
		if (newDockingPlace != iPreviousDockingPlace)
		{
			if (newDockingPlace != EDockNowhere)
			{
					/*
					 * Dock the window
					 */
				DockWindow(newDockingPlace);
				StopDraggingMyself();
			}
			else
				iPreviousDockingPlace = newDockingPlace;
		}
	}
	::SetCursor((HICON)iCursorMove);
}



void CDeskwinWindow::NormalMouseDown(TEvent aEvent)
{
	ResetAutoHideTimer();

	int desktop, desktopX, desktopY;

	if (PointInDesktop(iEvent.iMouse.x, iEvent.iMouse.y, desktop, desktopX, desktopY))
	{
		if (desktop != iActiveDesktop)
		{
			SwitchDesktop(desktop);
		}
		else
		{
			StartDraggingWindow(desktopX, desktopY);
		}
	}
	else
	{
		StartDraggingMyself(iEvent.iMouse.x, iEvent.iMouse.y);
	}
}

void CDeskwinWindow::NormalMouseUp(TEvent aEvent)
{
	ResetAutoHideTimer();
}

void CDeskwinWindow::NormalMouseMouse(TEvent aEvent)
{
	ResetAutoHideTimer();

	iDragResizeArea = GetResizeArea(iEvent.iMouse.x , iEvent.iMouse.y);
	if (iDragResizeArea != EResizeAreaNone)
	{
		DPRINT((HERE "CDeskwinWindow::NormalMouseMouse, start dragging, outbound, (%i, %i)", iEvent.iMouse.x, iEvent.iMouse.y));
		iCurrentCursor = ((iDragResizeArea == EResizeAreaTopRight) || (iDragResizeArea == EResizeAreaBottomLeft)) ? iCursorNeSw : iCursorNwSe;
		iPreviousCursor = ::SetCursor((HICON)iCurrentCursor);
		iState = EStateDragResizeIntention;
	}
	else
	{
		int desktop, desktopX, desktopY;
		if (!PointInDesktop(iEvent.iMouse.x, iEvent.iMouse.y, desktop, desktopX, desktopY))
		{
			iCurrentCursor = iCursorMove;
			iPreviousCursor = ::SetCursor((HICON)iCurrentCursor);
		}
	}
}


void CDeskwinWindow::ShowOptionsMenu(int aX, int aY)
{
	if (iMenuAlreadyRunning)
		return;
	iMenuAlreadyRunning = true;
	CPopupMenu subMenu = iMenu.GetSubMenu(0);

//	SetForegroundWindow();	// make sure we have keyboard focus
//	int cmd = subMenu.Track(aX, aY, HWindow, TPM_LEFTALIGN | TPM_LEFTBUTTON | TPM_RETURNCMD);
		/*
		 * Use iSysTrayIconController's window to track the menu. We need to call
		 * SetForegroundWindow to get keyboard focus. Using the deskwin window will
		 * move it to front.
		 */
	iSysTrayIconController->SetForegroundWindow();
	int cmd = subMenu.Track(aX, aY, iSysTrayIconController->HWindow, TPM_LEFTALIGN | TPM_LEFTBUTTON | TPM_RETURNCMD);

	if (cmd == ID_POPUP_APPEARANCE_OPTIONS)
		DoCmdAppearanceOptions();
	else if (cmd == ID_POPUP_DOCK_OPTIONS)
		DoCmdDockOptions();
	else if (cmd == ID_POPUP_AUTOHIDE_OPTIONS)
		DoCmdAutoHideOptions();
	else if (cmd == ID_POPUP_HOTKEY_OPTIONS)
		DoCmdHotkeyOptions();
	else if (cmd == ID_POPUP_COLOR_OPTIONS)
		DoCmdColorOptions();
	else if (cmd == ID_POPUP_STICKY_WINDOWS_OPTIONS)
		DoCmdStickyWindows();
	else if (cmd == ID_POPUP_ABOUT)
		DoCmdAbout();
	else if (cmd == ID_POPUP_README)
		DoCmdReadme();
	else if (cmd == ID_POPUP_QUIT)
		DoCmdQuit();

	iMenuAlreadyRunning = false;
}

void CDeskwinWindow::ShowPopupMenu(TEvent aEvent)
{
	RECT	rect;
	GetWindowRect(&rect);
	ShowOptionsMenu(rect.left + iEvent.iMouse.x, rect.top + iEvent.iMouse.y);
}

void CDeskwinWindow::DoCmdQuit()
{
	PostMessage(WM_CLOSE);
}


//#define CRASH_TEST
void CDeskwinWindow::DoCmdReadme()
{
#ifdef CRASH_TEST
	int* i=0;
	*i=0;
#endif
	if (ShellExecute(GetDesktopWindow(), "open", "readme.html", NULL, NULL, SW_SHOWNORMAL) <= (HINSTANCE) 32)
	{
		char		errorTitle[MAX_TEXT];
		char		errorText[MAX_TEXT];

		errorTitle[0] = 0;
		LoadString(GetAppInstance(), IDS_STRING_README_LAUNCH_ERROR_CAPTION, errorTitle, MAX_TEXT); 
		errorTitle[MAX_TEXT - 1] = 0;

		errorText[0] = 0;
		LoadString(GetAppInstance(), IDS_STRING_README_LAUNCH_ERROR_TEXT, errorText, MAX_TEXT); 
		errorText[MAX_TEXT - 1] = 0;

		MessageBox(errorText, errorTitle, MB_ICONERROR);
	}
}

void CDeskwinWindow::DoCmdAbout()
{
	CAboutDialog  dialog;
	AddChild(&dialog);
//	GetApplication()->AddWindow(&dialog);

	dialog.Execute();
}


void CDeskwinWindow::DoCmdStickyWindows()
{
	TDeskwinOptions		newOptions;
	CopyOptions(iOptions, newOptions);

	CStickyWindowsDialog  dialog(newOptions, *iDeskwinEngine);
	AddChild(&dialog);

	if (dialog.Execute() == IDOK)
	{
		iOptions.iStickyWindowInfo = newOptions.iStickyWindowInfo;
		SaveOptions();
		iDeskwinEngine->SetStickyWindowInfo(iOptions.iStickyWindowInfo);
		RepaintDesktop(-1, TRUE);
	}
}


void CDeskwinWindow::DoCmdColorOptions()
{
	GetColorOptions();

	TDeskwinOptions		newOptions;
	CopyOptions(iOptions, newOptions);

	CColorOptionsDialog  dialog(newOptions);
	AddChild(&dialog);
	
	if (dialog.Execute() == IDOK)
	{
		iOptions.iOuterFrameColor		= newOptions.iOuterFrameColor;
		iOptions.iInnerFrameColor		= newOptions.iInnerFrameColor;
		iOptions.iBackgroundColor		= newOptions.iBackgroundColor;
		iOptions.iActiveDesktopColor	= newOptions.iActiveDesktopColor;
		iOptions.iWindowFrameColor		= newOptions.iWindowFrameColor;
		iOptions.iWindowColor			= newOptions.iWindowColor;

		iOptions.iSysTrayIconOptions.iFgColor = newOptions.iSysTrayIconOptions.iFgColor;
		iOptions.iSysTrayIconOptions.iBgColor = newOptions.iSysTrayIconOptions.iBgColor;
		iOptions.iSysTrayIconOptions.iTransparentBackground = newOptions.iSysTrayIconOptions.iTransparentBackground;

		iSysTrayIconController->OptionsChanged(iOptions.iSysTrayIconOptions);

		DPRINT((HERE "CDeskwinWindow::DoCmdColorOptions, color=0x%x", iOptions.iOuterFrameColor));

		SetColorOptions();
		SaveOptions();

		Redraw();
	}
}


void CDeskwinWindow::DoCmdDockOptions()
{
	TDeskwinOptions		newOptions;
	CopyOptions(iOptions, newOptions);

	CDockOptionsDialog  dialog(newOptions);
	AddChild(&dialog);
	
	if (dialog.Execute() == IDOK)
	{
		iOptions.iWantsDocking = newOptions.iWantsDocking;
		iOptions.iDockPercentFromEdge =  newOptions.iDockPercentFromEdge;
		SaveOptions();
	}
}

void CDeskwinWindow::SetZOrder()
{
	UINT	flags = SWP_NOSIZE | SWP_NOMOVE;
	SetWindowPos(
		iOptions.iAlwaysOnTop ? HWND_TOPMOST : HWND_NOTOPMOST,
		0,
		0,
		0,
		0,
		flags
		);
}

void CDeskwinWindow::DoCmdAppearanceOptions()
{
	TDeskwinOptions		newOptions;
	CopyOptions(iOptions, newOptions);

	CAppearanceOptionsDialog  dialog(newOptions);
	AddChild(&dialog);
	
	if (dialog.Execute() == IDOK)
	{
		iOptions.iSysTrayIconOptions.iIconSet = newOptions.iSysTrayIconOptions.iIconSet;
		iOptions.iAlwaysOnTop = newOptions.iAlwaysOnTop;
		SaveOptions();
		iSysTrayIconController->OptionsChanged(iOptions.iSysTrayIconOptions);
		SetZOrder();
	}
}

void CDeskwinWindow::DoCmdAutoHideOptions()
{
	TDeskwinOptions		newOptions;
	CopyOptions(iOptions, newOptions);

	CAutoHideOptionsDialog  dialog(newOptions);
	AddChild(&dialog);
	
	if (dialog.Execute() == IDOK)
	{
		iOptions.iWantsAutoHide = newOptions.iWantsAutoHide;
		iOptions.iWantsShowByMouseClick = newOptions.iWantsShowByMouseClick;
		iOptions.iPercentVisible =  newOptions.iPercentVisible;
		iOptions.iAutoHideTimeOut =  newOptions.iAutoHideTimeOut;
		SaveOptions();
	}
	SetEventHandlerForHiddenState();
}



void CDeskwinWindow::DoCmdHotkeyOptions()
{
	TDeskwinOptions		newOptions;
	CopyOptions(iOptions, newOptions);

	CHotkeyOptionsDialog  dialog(newOptions);
	AddChild(&dialog);

		/*
		 * by doing this, we get 
		 */
	UnregisterHotkeys(iOptions);

	if (dialog.Execute() == IDOK)
	{
		iOptions.iHotkeyShowDeskwin = newOptions.iHotkeyShowDeskwin;
		iOptions.iHotkeyMinimiseAll = newOptions.iHotkeyMinimiseAll;
		iOptions.iHotkeyToggleExplorer = newOptions.iHotkeyToggleExplorer;
		iOptions.iHotkeyNextDesktop = newOptions.iHotkeyNextDesktop;
		iOptions.iHotkeyPreviousDesktop = newOptions.iHotkeyPreviousDesktop;
		iOptions.iHotkeyDesktopUp = newOptions.iHotkeyDesktopUp;
		iOptions.iHotkeyDesktopDown = newOptions.iHotkeyDesktopDown;
		iOptions.iHotkeyVolumeToggleMute = newOptions.iHotkeyVolumeToggleMute;
		iOptions.iHotkeyVolumeUp = newOptions.iHotkeyVolumeUp;
		iOptions.iHotkeyVolumeDown = newOptions.iHotkeyVolumeDown;
		iOptions.iHotkeyList.assign(newOptions.iHotkeyList.begin(), newOptions.iHotkeyList.end());

		SaveOptions();
	}
	RegisterHotkeys(iOptions);
}

void CDeskwinWindow::StartFadingOut(TEvent aEvent)
{
	int		deltaX = iWindowSize.x - ((iWindowSize.x * iOptions.iPercentVisible) / 100);
	int		deltaY = iWindowSize.y - ((iWindowSize.y * iOptions.iPercentVisible) / 100);

//	DPRINT((HERE "CDeskwinWindow::StartFadingOut"));

	iFadeMove.x = iFadeMove.y = 0;
	if (iPreviousDockingPlace == EDockLeft)
		iFadeMove.x = -1 * deltaX;
	else if (iPreviousDockingPlace == EDockRight)
		iFadeMove.x = deltaX;
	else if (iPreviousDockingPlace == EDockTop)
		iFadeMove.y = -1 * deltaY;
	else
		iFadeMove.y = deltaY;

	iState = EStateFadingOut;
	iSmoothWindowMover->StartSmoothMoving(this, iFadeMove.x, iFadeMove.y, iOptions.iAnimationTime, iOptions.iAnimationStepTime, this);
	iWindowPositionTracker->PauseWindowPositionTracking();
}


void CDeskwinWindow::FadingOutDone(TEvent aEvent)
{
//	DPRINT((HERE "CDeskwinWindow::FadingOutDone"));
	iState = EStateHidden;

	SaveOptions();
	SaveState();
}

void CDeskwinWindow::FadingInDone(TEvent aEvent)
{
//	DPRINT((HERE "CDeskwinWindow::FadingInDone"));
	iState = EStateNormal;
	StartAutoHideTimer();
	iWindowPositionTracker->ResumeWindowPositionTracking();

	SaveOptions();
	SaveState();
}

void CDeskwinWindow::StartFadingIn(TEvent aEvent)
{
//	DPRINT((HERE "CDeskwinWindow::StartFadingIn"));

		/*
		 * reverse the operation of the previous fade out
		 */
	iFadeMove.x *= -1;
	iFadeMove.y *= -1;
	iState = EStateFadingIn;
	iSmoothWindowMover->StartSmoothMoving(this, iFadeMove.x, iFadeMove.y, iOptions.iAnimationTime, iOptions.iAnimationStepTime, this);
}



void CDeskwinWindow::SetOptions()
{
	iOptions.iNoOfDesktopsHor = 2;
	iOptions.iDesktopPixelWidth = 150;

	iOptions.iNoOfDesktopsVert = 2;
	iOptions.iDesktopPixelHeight = 75;

	iOptions.iDesktopOutFramePixelSize = 6;
	iOptions.iDesktopInFramePixelSize = 1;

	iOptions.iOuterFrameColor		= ct_LightPurple;
	iOptions.iInnerFrameColor		= ct_LightPurple;
	iOptions.iBackgroundColor		= ct_LightBlue;
	iOptions.iActiveDesktopColor	= ct_DarkPurple;
	iOptions.iWindowFrameColor		= ct_Black;
	iOptions.iWindowColor			= ct_White;

	iOptions.iWantsDocking = true;
	iOptions.iDockPercentFromEdge = 10;

	iOptions.iWantsAutoHide = true;
	iOptions.iWantsShowByMouseClick = true;
	iOptions.iPercentVisible = 5;
	iOptions.iAutoHideTimeOut = 2000;
	iOptions.iAnimationTime = 500;
	iOptions.iAnimationStepTime = 100;

	iOptions.iActivePositionTrackingTime = 100;
	iOptions.iBackgroundPositionTrackingTime = 500;

	iOptions.iSnapResizePercent = 80;

	iOptions.iAlwaysOnTop = true;
	iOptions.iSysTrayIconOptions.iIconSet = 2;
	iOptions.iSysTrayIconOptions.iTransparentBackground = true;
	iOptions.iSysTrayIconOptions.iFgColor = ct_LightRed;
	iOptions.iSysTrayIconOptions.iBgColor = ::GetSysColor(COLOR_3DFACE);


		/*
		 * After defaults, override from registry
		 */
	LoadOptions(iOptions);

		/*
		 * set member variables from options
		 */
	SetDeskwinBaseOptions();
	SetColorOptions();

		/*
		 * And some sanity checks...
		 */
	ForceToBeInRange(iNoOfDesktopsHor, MIN_NO_OF_DESKTOPS, MAX_NO_OF_DESKTOPS);
	ForceToBeInRange(iDesktopPixelWidth, MIN_DESKTOP_PIXEL_SIZE, MAX_DESKTOP_PIXEL_SIZE);

	ForceToBeInRange(iNoOfDesktopsVert, MIN_NO_OF_DESKTOPS, MAX_NO_OF_DESKTOPS);
	ForceToBeInRange(iDesktopPixelHeight, MIN_DESKTOP_PIXEL_SIZE, MAX_DESKTOP_PIXEL_SIZE);

	ForceToBeInRange(iDesktopOutFramePixelSize, MIN_OUT_FRAME_PIXEL_SIZE, MAX_OUT_FRAME_PIXEL_SIZE);
	ForceToBeInRange(iDesktopInFramePixelSize, MIN_IN_FRAME_PIXEL_SIZE, MAX_IN_FRAME_PIXEL_SIZE);

	ForceToBeInRange(iOptions.iDockPercentFromEdge, MIN_DOCK_PERCENT_FROM_EDGE, MAX_DOCK_PERCENT_FROM_EDGE);

	ForceToBeInRange(iOptions.iPercentVisible, MIN_PERCENT_VISIBLE, MAX_PERCENT_VISIBLE);
	ForceToBeInRange(iOptions.iAutoHideTimeOut, MIN_AUTO_HIDE_TIMEOUT, MAX_AUTO_HIDE_TIMEOUT);
	ForceToBeInRange(iOptions.iAnimationTime, MIN_ANIMATION_TIME, MAX_ANIMATION_TIME);
	ForceToBeInRange(iOptions.iAnimationStepTime, MIN_ANIMATION_STEP_TIME, MAX_ANIMATION_STEP_TIME);

	ForceToBeInRange(iOptions.iActivePositionTrackingTime, MIN_ACTIVE_POSITION_TRACKING_TIME, MAX_ACTIVE_POSITION_TRACKING_TIME);
	ForceToBeInRange(iOptions.iBackgroundPositionTrackingTime, MIN_BACKGROUND_POSITION_TRACKING_TIME, MAX_BACKGROUND_POSITION_TRACKING_TIME);

	iDeskwinEngine->SetNoOfDesktops(iNoOfDesktopsHor * iNoOfDesktopsVert);
	iDeskwinEngine->SetStickyWindowInfo(iOptions.iStickyWindowInfo);

	SetEventHandlerForHiddenState();
}



void CDeskwinWindow::CheckStickyWindow(TWindow aWindow)
{
	bool	ruleMatch;
	bool	isStickyIgnore;
	int		ruleNoIgnore;

	iDeskwinEngine->IsStickyWindow1(-1, aWindow, ruleMatch, isStickyIgnore, ruleNoIgnore);
	if (!ruleMatch)
	{
		TStickyWindow		stickyWindow;
		CDeskwinEngine::DefaultWindowRule(aWindow, stickyWindow);
		iOptions.iStickyWindowInfo.push_back(stickyWindow);
	}
}

void CDeskwinWindow::CheckStickyWindows()
{
	CheckStickyWindow(this->HWindow);
	CheckStickyWindow(CDeskwinEngine::GetExplorerWindow());
	CheckStickyWindow(CDeskwinEngine::GetTaskbarWindow());
	TWindow		startMenuWindow = CDeskwinEngine::GetStartMenuWindow();
	if (startMenuWindow)
		CheckStickyWindow(startMenuWindow);
	iDeskwinEngine->SetStickyWindowInfo(iOptions.iStickyWindowInfo);
}


void CDeskwinWindow::Construct(CDeskwinEngine* aDeskwinEngine)
{
	iDeskwinEngine = aDeskwinEngine;
	iWindowPositionTracker = new CWindowPositionTracker;
	iSmoothWindowMover = new CSmoothWindowMover;

	SetOptions();
	RestoreState();

		/*
		 * If we were auto-hiden last time, we start showing docked.
		 */
	ForceToBeInRange(iStatePosX, 0, (int) iScreenSize.x);
	ForceToBeInRange(iStatePosY, 0, (int) iScreenSize.y);

	DWORD	style	= WS_POPUP | WS_VISIBLE;
	DWORD	exStyle	= WS_EX_TOOLWINDOW;
	if (iOptions.iAlwaysOnTop)
		exStyle	|= WS_EX_TOPMOST;

	CDeskwinWindowBase::Construct(iStatePosX, iStatePosY, style, exStyle);

	iWindowIcon = ::LoadIcon(GetAppInstance(), MAKEINTRESOURCE(IDI_MAIN_ICON));
	SetClassLong(GCL_HICON, (DWORD) iWindowIcon);
	SetWindowText("Deskwin");

		// after we've been created, enforce the sticky window rules
	CheckStickyWindows();

	iWindowPositionTracker->StartWindowPositionTracking(HWindow, WM_WINDOW_POS_CHANGED, iOptions.iActivePositionTrackingTime, iOptions.iBackgroundPositionTrackingTime);
	iWindowPositionTracker->ResumeWindowPositionTracking();

	if (iStateDockingPlace != EDockNowhere)
		DockWindow(iStateDockingPlace);

	RegisterHotkeys(iOptions);

	StartBackground(HWindow, WM_WINDOW_CREATED);

	iSysTrayIconController = new CSysTrayIconController(this->HWindow, WM_SYSTRAY_EVENT);
	iSysTrayIconController->Construct();
	iDeskwinEngine->AddObserver(*iSysTrayIconController);
	iSysTrayIconController->OptionsChanged(iOptions.iSysTrayIconOptions);

//	AddNewCW(iDesktopNumberWindow, GetApplication());
//	iDesktopNumberWindow->Construct();
}


bool CDeskwinWindow::PointInWindowFrame(int aClientX, int aClientY)
{
	return ((aClientX < iDesktopOutFramePixelSize) || (aClientY < iDesktopOutFramePixelSize) || 
		(aClientX > iWindowSize.x - iDesktopOutFramePixelSize) || (aClientY > iWindowSize.y - iDesktopOutFramePixelSize));
}

bool CDeskwinWindow::PointInDesktop(int aClientX, int aClientY, int& aDesktop, int& aDesktopX, int& aDesktopY)
{
	if (PointInWindowFrame(aClientX, aClientY))
		return false;

	aClientX -= iDesktopOutFramePixelSize;
	aClientY -= iDesktopOutFramePixelSize;

	int h = aClientX / (iDesktopPixelWidth + iDesktopInFramePixelSize);
	int v = aClientY / (iDesktopPixelHeight + iDesktopInFramePixelSize);

	aDesktop = h + (v * iNoOfDesktopsHor);
	aDesktopX = aClientX - (h * iDesktopPixelWidth);
	aDesktopY = aClientY - (v * iDesktopPixelHeight);

	return true;
}



void CDeskwinWindow::GetDesktopRect(int aDesktop, RECT& aRect)
{
	int h = aDesktop % iNoOfDesktopsHor;
	int v = aDesktop / iNoOfDesktopsHor;

	aRect.left = iDesktopOutFramePixelSize + (h * (iDesktopPixelWidth + iDesktopInFramePixelSize));
	aRect.top = iDesktopOutFramePixelSize + (v * (iDesktopPixelHeight + iDesktopInFramePixelSize));

	aRect.right = aRect.left + iDesktopPixelWidth + iDesktopInFramePixelSize;
	aRect.bottom = aRect.top + iDesktopPixelHeight + iDesktopInFramePixelSize;
}

void CDeskwinWindow::PaintDesktop(CDC& aDc, int aDesktop, bool aIsActive)
{
	TWindow			explorerWindow = iDeskwinEngine->GetExplorerWindow();
	TWindow			taskbarWindow = iDeskwinEngine->GetTaskbarWindow();
	TWindowList		windowList;
	iDeskwinEngine->GetWindowList(aDesktop, windowList);

	CDeskwinWindowBase::PaintDesktop(aDc, aDesktop, aIsActive, windowList, explorerWindow, taskbarWindow, iDragWindow);
}



void CDeskwinWindow::DoPaint(CDC& aDc)
{
	iEvent.iPaintCDC = &aDc;
	HandleEvent(EEventPaint);
}

bool CDeskwinWindow::IsStickyWindow(int aDesktop, TWindow aWindow)
{
	return iDeskwinEngine->IsStickyWindow(aDesktop, aWindow);
}


void CDeskwinWindow::ResizeNoOfDesktopsPaint(TEvent aEvent)
{
	RECT	clientRect;
	
	GetClientRect(&clientRect);
	DoResizeNoOfDesktopsPaint(*iEvent.iPaintCDC, clientRect);
}


void CDeskwinWindow::DoResizeNoOfDesktopsPaint(CDC& aDc, RECT& aRect)
{
	CBrush	backgroundBrush(iOptions.iBackgroundColor);
	CPen	backgroundPen(iOptions.iBackgroundColor, 1);

	aDc.SelectObject(backgroundBrush);
	aDc.SelectObject(backgroundPen);
	aDc.Rectangle(aRect.left, aRect.top, aRect.right, aRect.bottom);

	CPen	outFramePen(iOptions.iOuterFrameColor, iDesktopOutFramePixelSize*2);
	CPen	inFramePen(iOptions.iInnerFrameColor, iDesktopInFramePixelSize);
	CPen	whitePen(ct_White, 1);
	CPen	blackPen(ct_Black, 1);
	CBrush	fillBrush(iOptions.iBackgroundColor);

	int		windowSize = aRect.right;
	aDc.SelectObject(outFramePen);
	aDc.SelectObject(fillBrush);
	aDc.Rectangle(aRect.left, aRect.top, aRect.right, aRect.bottom);

	aDc.SelectObject(whitePen);
	aDc.MoveTo(1, 1);
	aDc.LineTo(aRect.right - 1, 1);
	aDc.MoveTo(1, 1);
	aDc.LineTo(1, aRect.bottom - 1);

	aDc.SelectObject(blackPen);
	aDc.MoveTo(aRect.right - 1, 0);
	aDc.LineTo(aRect.right - 1, iWindowSize.y - 1);
	aDc.MoveTo(0, aRect.bottom - 1);
	aDc.LineTo(aRect.right - 1, aRect.bottom - 1);

		/*
		 * Draw vertical innerframes, if we are dragging from the left
		 * we start drawing on the right; left otherwise
		 */
	aDc.SelectObject(inFramePen);
	int VertLineSize = aRect.bottom - 2 * iDesktopOutFramePixelSize;
	int deltaX = iDesktopPixelWidth + iDesktopInFramePixelSize;
	int x=iDesktopOutFramePixelSize + iDesktopPixelWidth;
	if ((iDragResizeArea == EResizeAreaTopLeft) || (iDragResizeArea == EResizeAreaBottomLeft))
	{
		deltaX *= -1;
		x = aRect.right - (iDesktopOutFramePixelSize + iDesktopPixelWidth);
	}
	
	while ((x>0) && (x<aRect.right))
	{
		aDc.MoveTo(x, iDesktopOutFramePixelSize);
		aDc.LineTo(x, VertLineSize + iDesktopOutFramePixelSize);
		x += deltaX;
	}
	 
		/*
		 * Draw horizontal innerframes, if we are dragging from the top,
		 * we start drawing from the bottom
		 */
	int HorLineSize = aRect.right - 2 * iDesktopOutFramePixelSize;
	int y=iDesktopOutFramePixelSize + iDesktopPixelHeight;
	int deltaY = iDesktopPixelHeight + iDesktopInFramePixelSize;
	if ((iDragResizeArea == EResizeAreaTopLeft) || (iDragResizeArea == EResizeAreaTopRight))
	{
		deltaY *= -1;
		y=aRect.bottom - (iDesktopOutFramePixelSize + iDesktopPixelHeight);
	}
	while ((y>0) && (y<aRect.bottom))
	{
		aDc.MoveTo(iDesktopOutFramePixelSize, y);
		aDc.LineTo(iDesktopOutFramePixelSize + HorLineSize, y);
		y += deltaY;
	}

	aDc.RestoreObjects();
}


void CDeskwinWindow::NormalPaint(TEvent aEvent)
{
	CDC*	dc = iEvent.iPaintCDC;
	
	RECT	clientRect;
	GetClientRect(&clientRect);

	PaintFrames(*dc, clientRect);

		/*
		 * To facilitate dragging, we repaint the active desktop last
		 */
	for (int d=0 ; d<(iNoOfDesktopsHor * iNoOfDesktopsVert) ; d++)
	{
		if (d != iActiveDesktop)
			PaintDesktop(*dc, d, false);
	}
	PaintDesktop(*dc, iActiveDesktop, true);
}




void CDeskwinWindow::DockWindow(TDockingPlace aDockingPlace)
{
	RECT				rect;

	GetWindowRect(&rect);

	if (aDockingPlace == EDockLeft)
		rect.left = 0;
	else if (aDockingPlace == EDockRight)
		rect.left = iScreenSize.x - iWindowSize.x;
	else if (aDockingPlace == EDockTop)
		rect.top = 0;		
	else
		rect.top = iScreenSize.y - iWindowSize.y;

	MoveWindow(rect.left, rect.top, iWindowSize.x, iWindowSize.y);

	iPreviousDockingPlace = aDockingPlace;
	iIsDocked = true;

	StartAutoHideTimer();
}



void CDeskwinWindow::StartDraggingMyself(int aX, int aY)
{
	iState = EStateDragMoveMyself;

	GetWindowRect(&iDragRect);
	iDragStartX = iDragRect.left + aX;
	iDragStartY = iDragRect.top + aY;

	SetCapture();
}


void CDeskwinWindow::StopDraggingMyself()
{
	iState = EStateNormal;

//	DPRINT((HERE "CDeskwinWindow::StopDraggingMyself"));

	ReleaseCapture();

		/*
		 * repaint window
		 */
	RepaintDesktop(-1);
}





CDeskwinWindow::TDockingPlace CDeskwinWindow::GetDockingPlace()
{
	TDockingPlace		dockingPlace = EDockNowhere;
	RECT				rect;

	GetWindowRect(&rect);

	int dockX = (iScreenSize.x * iOptions.iDockPercentFromEdge) / 100;
	int dockY = (iScreenSize.y * iOptions.iDockPercentFromEdge) / 100;
	
	if (rect.left < dockX)
	{
		dockingPlace = EDockLeft;
	}
	else if (rect.top < dockY)
	{
		dockingPlace = EDockTop;
	}
	else if (rect.right > (iScreenSize.x - dockX))
	{
		dockingPlace = EDockRight;
	}
	else if (rect.bottom > (iScreenSize.y - dockY))
	{
		dockingPlace = EDockBottom;
	}

	return dockingPlace;
}
	
void CDeskwinWindow::ResetAutoHideTimer()
{
	KillTimer(0);
	if ((iIsDocked || (iState==EStateFadingIn)) && iOptions.iWantsAutoHide)
	{
		SetTimer(0, iOptions.iAutoHideTimeOut);
	}
}

void CDeskwinWindow::StartAutoHideTimer()
{
	if (iIsDocked && iOptions.iWantsAutoHide)
	{
		KillTimer(0);
		SetTimer(0, iOptions.iAutoHideTimeOut);
	}
}




void CDeskwinWindow::SwitchDesktop(int aNewDesktop, TWindow aDragWindow)
{
	if (aNewDesktop != iActiveDesktop)
	{
		iDeskwinEngine->SwitchTo(aNewDesktop, aDragWindow);
		iActiveDesktop = aNewDesktop;

			/*
			 * and redraw
			 */
		RepaintDesktop(-1, true);

//		iDesktopNumberWindow->SetDesktop(aNewDesktop);
	}
}



/*************************************
 *
 * External stimuli
 *
 *************************************/

void CDeskwinWindow::OnTimer()
{
	DPRINT((HERE "CDeskwinWindow::OnTimer"));

	KillTimer(0);
	HandleEvent(EEventTimeout);
}

void CDeskwinWindow::DoneSmoothWindowMoving(CWindow* aWindow)
{
	ASSERT(aWindow == this);

	HandleEvent(EEventSmoothMoveDone);
}

void CDeskwinWindow::OnLButtonUp(int aKey, int aX, int aY)
{
	iEvent.iMouse.key	= aKey;
	iEvent.iMouse.x	= aX;
	iEvent.iMouse.y	= aY;
	
	HandleEvent(EEventLMouseUp);
}

void CDeskwinWindow::OnLButtonDown(int aKey, int aX, int aY)
{
	if (!iOptions.iAlwaysOnTop)
	{
		SetForegroundWindow();	// make sure we have keyboard focus
		BringWindowToTop();
	}

	iEvent.iMouse.key	= aKey;
	iEvent.iMouse.x	= aX;
	iEvent.iMouse.y	= aY;
	
	HandleEvent(EEventLMouseDown);
}

void CDeskwinWindow::OnRButtonDown(int aKey, int aX, int aY)
{
	iEvent.iMouse.key	= aKey;
	iEvent.iMouse.x	= aX;
	iEvent.iMouse.y	= aY;
	
	HandleEvent(EEventRMouseDown);
}

void CDeskwinWindow::OnRButtonUp(int aKey, int aX, int aY)
{
	iEvent.iMouse.key	= aKey;
	iEvent.iMouse.x	= aX;
	iEvent.iMouse.y	= aY;
	
	HandleEvent(EEventRMouseUp);
}


void CDeskwinWindow::OnMouseMove(int aKey, int aX, int aY)
{
	iEvent.iMouse.key	= aKey;
	iEvent.iMouse.x	= aX;
	iEvent.iMouse.y	= aY;
	
	HandleEvent(EEventMouseMove);
}

void CDeskwinWindow::OnCaptureChanged()
{
	DPRINT((HERE "CDeskwinWindow::OnCaptureChanged"));
}

void CDeskwinWindow::OnDisplayChanged(int /*aBitsPerPixel*/, int aX, int aY)
{
}


void CDeskwinWindow::OnHotkey(int aHotkeyId, int aModifier, int aVirtKey)
{
//	DPRINT((HERE "CDeskwinWindow::OnHotkey"));

	if ((aHotkeyId >= KHotkeyToDesktop0) && (aHotkeyId < KHotkeyToDesktop0 + iNoOfDesktopsHor * iNoOfDesktopsVert))
	{
		int desktop = aHotkeyId - KHotkeyToDesktop0;
		SwitchDesktop(desktop);
	}
	else if (aHotkeyId == KHotkeyNextDesktop)
	{
		int delta = (((iActiveDesktop + 1) % iNoOfDesktopsHor) ==0) ? (-1 * (iNoOfDesktopsHor - 1)) : 1;
		SwitchDesktop(iActiveDesktop + delta);
	}
	else if (aHotkeyId == KHotkeyPreviousDesktop)
	{
		int delta = (((iActiveDesktop + 1) % iNoOfDesktopsHor) == 1) ? (iNoOfDesktopsHor - 1) : -1;
		SwitchDesktop(iActiveDesktop + delta);
	}
	else if (aHotkeyId == KHotkeyDesktopUp)
	{
		int delta = ((iActiveDesktop / iNoOfDesktopsHor) ==0) ? (iNoOfDesktopsHor * (iNoOfDesktopsHor - 1)) : iNoOfDesktopsHor * -1;
		SwitchDesktop(iActiveDesktop + delta);
	}
	else if (aHotkeyId == KHotkeyDesktopDown)
	{
		int delta = ((iActiveDesktop / iNoOfDesktopsHor) == iNoOfDesktopsVert - 1) ? -1 * (iNoOfDesktopsHor * (iNoOfDesktopsHor - 1)) : iNoOfDesktopsHor;
		SwitchDesktop(iActiveDesktop + delta);
	}
	else if (aHotkeyId == KHotkeyToggleExplorer)
	{

#if 0
		TWindow		explorerWindow = iDeskwinEngine->GetExplorerWindow();
		if (::IsWindowVisible(explorerWindow))
		{
			::ShowWindow(explorerWindow, SW_HIDE);
			::ShowWindow(iDeskwinEngine->GetTaskbarWindow(), SW_HIDE);
		}
		else
		{
			::ShowWindow(explorerWindow, SW_SHOW);
			::ShowWindow(iDeskwinEngine->GetTaskbarWindow(), SW_SHOW);
		}
#endif
		iDeskwinEngine->HideOrShowExplorerOrDesktop(iActiveDesktop);
		iOptions.iStickyWindowInfo = iDeskwinEngine->StickyWindowInfo();
		Redraw();

	}
	else if (aHotkeyId == KHotkeyMinimiseAll)
	{
		iDeskwinEngine->MinimiseAll(HWindow);
	}
	else if (aHotkeyId == KHotkeyShowDeskwin)
	{
		DPRINT((HERE "Hotkey show deskwin"));
		if (!iOptions.iAlwaysOnTop)
		{
			SetForegroundWindow();	// make sure we have keyboard focus
			BringWindowToTop();
			
				/*
				 * Yet another workaround.
				 * If you press the hide-desktop window, the shell activates some
				 * helper window which hides the currently set of windows.
				 * The helper window is deactivated the moment you click one of the
				 * task bar buttons to restore the window to its orignal size.
				 * Unfortunately, if we bring the deskwin window to the foreground, 
				 * this helper window doesn't get deactivated and clicking the destop
				 * makes deskwin invisible again. We get round this by creating and 
				 * destroying a dummy window; the adding and removing of the its taskbar
				 * button will successfully deactivate the help window.
				 */
			if (iDeskwinEngine->IsShellWorkerWindowVisible())
			{
				CWindow		someWindow;
				GetApplication()->AddWindow(&someWindow);
				GetApplication()->RegisterClass("CSomeWindow", NULL, NULL, -1, 0, CS_HREDRAW | CS_VREDRAW | CS_OWNDC);
				if (someWindow.Create("CSomeWindow", 0, 0, 0, 0, 0, WS_POPUP | WS_VISIBLE, NULL, WS_EX_APPWINDOW))
				{
					Sleep(250);
					someWindow.Destroy();
				}
			}
		}
		if (iState == EStateHidden)
			StartFadingIn(EEventLMouseDown);
	}
	else if (aHotkeyId == KHotkeyVolumeToggleMute)
	{
		iDeskwinEngine->VolumeToggleMute();
	}
	else if (aHotkeyId == KHotkeyVolumeUp)
	{
		iDeskwinEngine->VolumeUp();
	}
	else if (aHotkeyId == KHotkeyVolumeDown)
	{
		iDeskwinEngine->VolumeDown();
	}
}


void CDeskwinWindow::OnWindowCreated(TWindow aWindow)
{
	iEvent.iWindowCreated = aWindow;
	HandleEvent(EEventWindowCreated);
}


void CDeskwinWindow::WindowCreated(TEvent aEvent)
{
//	DPRINT((HERE "CDeskwinWindow::WindowCreated, window=0x%x", iEvent.iWindowCreated));

	iDeskwinEngine->HandleStickyWindowInfoChanged();
	Redraw();
}
 
void CDeskwinWindow::OnWindowPosChanged()
{
	Redraw();
}


void CDeskwinWindow::SaveState()
{
	int		docked;
	RECT	rect;

	if (!iIsDocked)
		docked = -1;
	else
		docked = (int) iPreviousDockingPlace;

	GetWindowRect(&rect);

	SetRegKeyIntegerValue(HKEY_CURRENT_USER, OPTION_STATE_ROOT, STATE_DOCKING_PLACE, docked);
	SetRegKeyIntegerValue(HKEY_CURRENT_USER, OPTION_STATE_ROOT, STATE_POSITION_X, rect.left);
	SetRegKeyIntegerValue(HKEY_CURRENT_USER, OPTION_STATE_ROOT, STATE_POSITION_Y, rect.top);
}

void CDeskwinWindow::RestoreState()
{
	int				dummy;

	GetRegKeyIntegerValue(HKEY_CURRENT_USER, OPTION_STATE_ROOT, STATE_DOCKING_PLACE, &dummy);
	GetRegKeyIntegerValue(HKEY_CURRENT_USER, OPTION_STATE_ROOT, STATE_POSITION_X, &iStatePosX);
	GetRegKeyIntegerValue(HKEY_CURRENT_USER, OPTION_STATE_ROOT, STATE_POSITION_Y, &iStatePosY);

	if (dummy < 0 || dummy >EDockLastEntry)
		iStateDockingPlace = EDockNowhere;
	else
		iStateDockingPlace = (TDockingPlace) dummy;
}





void CDeskwinWindow::DragMoveWindowDone(TEvent aEvent)
{
	RECT	rect;
	POINT	cursor;
	cursor.x = iEvent.iMouse.x;
	cursor.y = iEvent.iMouse.y;
	GetClientRect(&rect);
	bool cancelled = ::PtInRect(&rect, cursor) ? false : true;
	StopDraggingWindow(cancelled);

	SaveOptions();
	SaveState();
}


void CDeskwinWindow::InvalidateDragWindowBackground(RECT aScreenRect)
{
	RECT	desktopRect;
	GetDesktopRect(iActiveDesktop, desktopRect);

	ShrinkScreenRectToDesktopRect(aScreenRect, false);		// don't clip

	aScreenRect.left += desktopRect.left;
	aScreenRect.top += desktopRect.top;
	aScreenRect.right += desktopRect.left;
	aScreenRect.bottom += desktopRect.top;

	InvalidateRect(&aScreenRect, true);
}


void CDeskwinWindow::DoDragMoveWindow(TEvent aEvent)
{
	ASSERT(iDragWindow);
	
	RECT		screenrect;
	::GetWindowRect(iDragWindow, &screenrect);
	
	InvalidateDragWindowBackground(screenrect);

		/*
		 * don't temper with the window's size
		 */
	int windowSizeX = screenrect.right - screenrect.left;
	int windowSizeY = screenrect.bottom - screenrect.top;
	
		/*
		 * dragStartX and dragStartX are client coordinates
		 * calculate the mouse movement difference and inflate it to
		 * screen coordinates (ignore, .bottom and .right)
		 */
	RECT	clientRect;
	clientRect.left = iEvent.iMouse.x - iDragStartX;
	clientRect.top = iEvent.iMouse.y - iDragStartY;

	InflateDesktopRectToScreenRect(clientRect);

	int newX = screenrect.left + clientRect.left;
	int newY = screenrect.top + clientRect.top;

		/*
		 * Let's see if we are about to switch desktops, if so
		 */
	int			newDesktop = iActiveDesktop;
	int			ignoreX, ignoreY;
	PointInDesktop(iEvent.iMouse.x, iEvent.iMouse.y, newDesktop, ignoreX, ignoreY);
	if (newDesktop != iActiveDesktop)
	{
			/*
			 * Move the window for the new desktop
			 */
		UpdateWindowForDesktopSwitch(newDesktop, newX, newY);
		SwitchDesktop(newDesktop, iDragWindow);
		::BringWindowToTop(iDragWindow);
		SetCapture();		// recapture mouse which we lost because of desktop switching
 	}

	::MoveWindow(iDragWindow, newX, newY, windowSizeX, windowSizeY, true);

	iDragStartX = iEvent.iMouse.x;
	iDragStartY = iEvent.iMouse.y;

	RECT	newScreenRect;
	newScreenRect.left = newX;
	newScreenRect.top = newY;
	newScreenRect.right = newX + windowSizeX;
	newScreenRect.bottom = newY + windowSizeY;

	InvalidateDragWindowBackground(newScreenRect);

}


void CDeskwinWindow::StartDraggingWindow(int aDesktopX, int aDesktopY)
{
	iDragWindow = FindWindowInDesktop(iActiveDesktop, aDesktopX, aDesktopY);
	if (iDragWindow)
	{
		SetCapture();
//		DPRINT((HERE "FindWindowInDesktop, window=0x%x", iDragWindow));

		iWindowPositionTracker->PauseWindowPositionTracking();
		iDragOldAlwaysOnTop = iOptions.iAlwaysOnTop;
		iOptions.iAlwaysOnTop = true;
		SetZOrder();

			/*
			 * dragging start point in client coordinates
			 */
		iDragStartX = iEvent.iMouse.x;
		iDragStartY = iEvent.iMouse.y;

		RECT	myRect;
		GetWindowRect(&myRect);


		::GetWindowRect(iDragWindow, &iDragWindowOrigialRect);
		iDragWindowMaximized = IsZoomed(iDragWindow);
		// If Maximized, Make it so window's restored size is maximized size
		// Then restore the window. It's a little flicker but better than all other options.
		if(iDragWindowMaximized)
		{
			WINDOWPLACEMENT   wp;
			memset(&wp, 0, sizeof(WINDOWPLACEMENT));
			wp.length = sizeof(WINDOWPLACEMENT);

			::GetWindowPlacement(iDragWindow, &wp);

			wp.rcNormalPosition = iDragWindowOrigialRect;
			wp.length = sizeof(WINDOWPLACEMENT);

			::SetWindowPlacement(iDragWindow, &wp);

			::ShowWindow(iDragWindow, SW_RESTORE);
		}

		iState = EStateDragMoveWindow;
	}
}

void CDeskwinWindow::StopDraggingWindow(bool aCancelled)
{
	::ReleaseCapture();

	// If maximized, Max it maximized now that it's done dragging.
	if(iDragWindowMaximized)
	{
		::ShowWindow(iDragWindow, SW_MAXIMIZE);
	}
	else if ((iEvent.iMouse.key & MK_CONTROL) || aCancelled)
	{
		::SetWindowPos(
				iDragWindow, 
				HWND_TOP, 
				iDragWindowOrigialRect.left, 
				iDragWindowOrigialRect.top, 
				iDragWindowOrigialRect.right - iDragWindowOrigialRect.left, 
				iDragWindowOrigialRect.bottom - iDragWindowOrigialRect.top, 
				SWP_SHOWWINDOW
				);
	}
	iOptions.iAlwaysOnTop = iDragOldAlwaysOnTop;
	SetZOrder();

	iDragWindow = NULL;
	iDragWindowMaximized = FALSE;
	iWindowPositionTracker->ResumeWindowPositionTracking();
	RepaintDesktop(-1);
	iState = EStateNormal;
}


TWindow CDeskwinWindow::FindWindowInDesktop(int aDesktop, int aX, int aY)
{
	TWindow			explorerWindow = iDeskwinEngine->GetExplorerWindow();
	TWindow			taskbarWindow = iDeskwinEngine->GetTaskbarWindow();
	TWindowList		windowList;
	iDeskwinEngine->GetWindowList(aDesktop, windowList);

	POINT	point;
	point.x = aX;
	point.y = aY;
	for (TWindowList::iterator i=windowList.begin() ; i != windowList.end() ; i++)
	{
			/*
			 * ignore explorer, taskbar and ourself
			 */
		if (explorerWindow == *i || taskbarWindow == *i || HWindow == *i || ::IsIconic(*i))
			continue;

		RECT	windowRect;
		::GetWindowRect(*i, &windowRect);

		ShrinkScreenRectToDesktopRect(windowRect);

		if (::PtInRect(&windowRect, point))
			return *i;
	}

	return (TWindow) NULL;
}


void CDeskwinWindow::RepaintDesktop(int aDesktop, bool aRedrawBackground)
{
	RECT	rect;
	if (aDesktop == -1)
		GetClientRect(&rect);
	else
		GetDesktopRect(aDesktop, rect);
	
	InvalidateRect(&rect, aRedrawBackground);
}


CDeskwinWindow::TResizeArea CDeskwinWindow::GetResizeArea(int aX, int aY)
{
	TResizeArea resizeArea = EResizeAreaNone;

	if (IsInRange(aX, 0, iDesktopOutFramePixelSize))
	{
		if (IsInRange(aY, 0, iDesktopOutFramePixelSize))
			resizeArea = EResizeAreaTopLeft;
		else if (IsInRange(aY, (int) (iWindowSize.y - iDesktopOutFramePixelSize), (int) iWindowSize.y))
			resizeArea = EResizeAreaBottomLeft;
	}
	else if (IsInRange(aX, (int) iWindowSize.x - iDesktopOutFramePixelSize, (int) iWindowSize.x))
	{
		if (IsInRange(aY, 0, iDesktopOutFramePixelSize))
			resizeArea = EResizeAreaTopRight;
		else if (IsInRange(aY, (int) (iWindowSize.y - iDesktopOutFramePixelSize), (int) iWindowSize.y))
			resizeArea = EResizeAreaBottomRight;
	}

	return resizeArea;
}


void CDeskwinWindow::ExcludeFrameRect(RECT& aRect)
{
	ForceToBeInRange(aRect.left, (long) iDesktopOutFramePixelSize, (long) (iWindowSize.x - iDesktopOutFramePixelSize));
	ForceToBeInRange(aRect.right, (long) iDesktopOutFramePixelSize, (long) (iWindowSize.x - iDesktopOutFramePixelSize));
	ForceToBeInRange(aRect.top, (long) iDesktopOutFramePixelSize, (long) (iWindowSize.y - iDesktopOutFramePixelSize));
	ForceToBeInRange(aRect.bottom, (long) iDesktopOutFramePixelSize, (long) (iWindowSize.y - iDesktopOutFramePixelSize));
}

void CDeskwinWindow::UpdateWindowForDesktopSwitch(int aNewDesktop, int& aX, int& aY)
{
	int curH = iActiveDesktop % iNoOfDesktopsHor;
	int curV = iActiveDesktop / iNoOfDesktopsHor;
	int newH = aNewDesktop % iNoOfDesktopsHor;
	int newV =	aNewDesktop / iNoOfDesktopsHor;

	ASSERT(! ((curH == newH) && (curV == newV)))
	
	if (curV != newV)
	{
		if (curV < newV)
			aY -= iScreenSize.y;
		else
			aY += iScreenSize.y;
	}

	if (curH != newH)
	{
		if (curH < newH)
			aX -= iScreenSize.x;
		else
			aX += iScreenSize.x;
	}
}

void CDeskwinWindow::DoDragResizeIntention(TEvent aEvent)
{
	iDragResizeArea = GetResizeArea(iEvent.iMouse.x , iEvent.iMouse.y);
	if (iDragResizeArea == EResizeAreaNone)
	{
		::SetCursor((HICON)iPreviousCursor);
		iState = EStateNormal;
	}
	else
		::SetCursor((HICON)iCurrentCursor);
}



void CDeskwinWindow::DragResizeDone(TEvent aEvent)
{
	iState = EStateNormal;

	DPRINT((HERE "CDeskwinWindow::DragResizeDone"));

	ReleaseCapture();

	RepaintDesktop(-1, true);

	SaveOptions();
	SaveState();
}



void CDeskwinWindow::DoDragResize(TEvent aEvent)
{
	RECT		screenrect;
	GetWindowRect(&screenrect);

		/*
		 * calc diff since original drag point
		 */
	int diffX = screenrect.left + iEvent.iMouse.x - iDragStartX;
	int diffY = screenrect.top + iEvent.iMouse.y - iDragStartY;

	int newX, newY;

		/*
		 * Determine the new window size
		 */
	if (iDragResizeArea == EResizeAreaTopLeft)
	{
		DPRINT((HERE "DoDragResize --> topleft"));
		newX = iDragStartSizeX - diffX;
		newY = iDragStartSizeY - diffY;
	}
	else if (iDragResizeArea == EResizeAreaTopRight)
	{
		DPRINT((HERE "DoDragResize --> topright"));
		newX = iDragStartSizeX + diffX;
		newY = iDragStartSizeY - diffY;
	}
	else if (iDragResizeArea == EResizeAreaBottomLeft)
	{
		DPRINT((HERE "DoDragResize --> bottomleft"));
		newX = iDragStartSizeX - diffX;
		newY = iDragStartSizeY + diffY;
	}
	else /* EResizeAreaLBottomRight */
	{
		DPRINT((HERE "DoDragResize --> bottomright"));
		newX = iDragStartSizeX + diffX;
		newY = iDragStartSizeY + diffY;
	}

		/*
		 * Enforce minimun size
		 */
	int		minSizeY = 32;
	if (newY < minSizeY)
		newY = minSizeY;
	int		minSizeX = 32;
	if (newX < minSizeX)
		newX = minSizeX;

		/*
		 * ... however, round pixels, so that the desktops have equal size 
		 */
	int desktopsX = newX - 2 * iDesktopOutFramePixelSize;
	int desktopsY = newY - 2 * iDesktopOutFramePixelSize;
	desktopsX -= (iNoOfDesktopsHor - 1) * iDesktopInFramePixelSize;
	desktopsY -= (iNoOfDesktopsVert - 1) * iDesktopInFramePixelSize;

	iDesktopPixelWidth = desktopsX / iNoOfDesktopsHor;
	iDesktopPixelHeight = desktopsY / iNoOfDesktopsVert;

		/*
		 * Determine new rounded size
		 */
	newX	= iNoOfDesktopsHor * iDesktopPixelWidth + 2 * iDesktopOutFramePixelSize;
	newY	= iNoOfDesktopsVert * iDesktopPixelHeight + 2 * iDesktopOutFramePixelSize;
	newX	+= (iNoOfDesktopsHor - 1) * iDesktopInFramePixelSize;
	newY	+= (iNoOfDesktopsVert - 1) * iDesktopInFramePixelSize;

			
		/*
		 * So, how much do we need to grow shrink right now ?
		 */
	diffX = newX - iWindowSize.x;
	diffY = newY - iWindowSize.y;

	iWindowSize.x += diffX;
	iWindowSize.y += diffY;
	
	if (iDragResizeArea == EResizeAreaTopLeft)
	{
		screenrect.left -= diffX; 
		screenrect.top -= diffY; 
	}
	else if (iDragResizeArea == EResizeAreaTopRight)
	{
		screenrect.top -= diffY; 
	}
	else if (iDragResizeArea == EResizeAreaBottomLeft)
	{
		screenrect.left -= diffX; 
	}
	else /* EResizeAreaLBottomRight */
	{
	}

	if ((diffX != 0) || (diffY != 0))
		MoveWindow(screenrect.left, screenrect.top, iWindowSize.x, iWindowSize.y, true);

	::SetCursor((HICON)iCurrentCursor);
}



void CDeskwinWindow::StartDragging(TEvent aEvent)
{
	DPRINT((HERE "CDeskwinWindow::StartDragging"));

	if ((iEvent.iMouse.key & MK_CONTROL) && (iDragResizeArea == EResizeAreaBottomRight))
	{
		StartDragMoreDesktops();
		iCurrentCursor = iCursorMoreDesktops;
	}
	else
	{
		StartDragResize();
	}

	GetWindowRect(&iDragRect);
	
	iDragStartX = iDragRect.left + iEvent.iMouse.x;
	iDragStartY = iDragRect.top + iEvent.iMouse.y;
	iDragStartSizeX = iWindowSize.x;
	iDragStartSizeY = iWindowSize.y;

	::SetCursor((HICON)iCurrentCursor);

	SetCapture();
}


void CDeskwinWindow::StartDragResize()
{
	iState = EStateDragResize;
}


void CDeskwinWindow::RegisterHotkeys(TDeskwinOptions& aOptions)
{
	int			vk;
	int			mod;
	int			d = 0;
	for (THotkeyList::iterator i=aOptions.iHotkeyList.begin() ;  i != aOptions.iHotkeyList.end() ; i++, d++)
	{
		THotkey		hotkey = *i;

		MakeRegisterHotkeyParam(hotkey, vk, mod);
		if (vk)
			RegisterHotKey(KHotkeyToDesktop0 + d, mod, vk); 
	}
	MakeRegisterHotkeyParam(aOptions.iHotkeyShowDeskwin, vk, mod);
	if (vk)
		RegisterHotKey(KHotkeyShowDeskwin, mod, vk);

	MakeRegisterHotkeyParam(aOptions.iHotkeyMinimiseAll, vk, mod);
	if (vk)
		RegisterHotKey(KHotkeyMinimiseAll, mod, vk);

	MakeRegisterHotkeyParam(aOptions.iHotkeyToggleExplorer, vk, mod);
	if (vk)
		RegisterHotKey(KHotkeyToggleExplorer, mod, vk);

	MakeRegisterHotkeyParam(aOptions.iHotkeyPreviousDesktop, vk, mod);
	if (vk)
		RegisterHotKey(KHotkeyPreviousDesktop, mod, vk);

	MakeRegisterHotkeyParam(aOptions.iHotkeyDesktopUp, vk, mod);
	if (vk)
		RegisterHotKey(KHotkeyDesktopUp, mod, vk);

	MakeRegisterHotkeyParam(aOptions.iHotkeyDesktopDown, vk, mod);
	if (vk)
		RegisterHotKey(KHotkeyDesktopDown, mod, vk);

	MakeRegisterHotkeyParam(aOptions.iHotkeyNextDesktop, vk, mod);
	if (vk)
		RegisterHotKey(KHotkeyNextDesktop, mod, vk);

	MakeRegisterHotkeyParam(aOptions.iHotkeyVolumeToggleMute, vk, mod);
	if (vk)
		RegisterHotKey(KHotkeyVolumeToggleMute, mod, vk);

	MakeRegisterHotkeyParam(aOptions.iHotkeyVolumeUp, vk, mod);
	if (vk)
		RegisterHotKey(KHotkeyVolumeUp, mod, vk);

	MakeRegisterHotkeyParam(aOptions.iHotkeyVolumeDown, vk, mod);
	if (vk)
		RegisterHotKey(KHotkeyVolumeDown, mod, vk);
}


void CDeskwinWindow::UnregisterHotkeys(TDeskwinOptions& aOptions)
{
	for (int i=0 ;  i != aOptions.iHotkeyList.size() ; i++)
		UnregisterHotKey(KHotkeyToDesktop0 + i); 
	UnregisterHotKey(KHotkeyToggleExplorer);
	UnregisterHotKey(KHotkeyPreviousDesktop);
	UnregisterHotKey(KHotkeyNextDesktop);
	UnregisterHotKey(KHotkeyDesktopUp);
	UnregisterHotKey(KHotkeyDesktopDown);
	UnregisterHotKey(KHotkeyMinimiseAll);
	UnregisterHotKey(KHotkeyShowDeskwin);
	UnregisterHotKey(KHotkeyVolumeToggleMute);
	UnregisterHotKey(KHotkeyVolumeUp);
	UnregisterHotKey(KHotkeyVolumeDown);
}



void CDeskwinWindow::StartDragMoreDesktops()
{
	iWindowPositionTracker->PauseWindowPositionTracking();
	iDragVertDesktops = iNoOfDesktopsVert;
	iDragHorDesktops = iNoOfDesktopsHor;
	iState = EStateDragMoreDesktops;
}

void CDeskwinWindow::DragMoreDesktopsDone(TEvent aEvent)
{
	iState = EStateNormal;

	DPRINT((HERE "CDeskwinWindow::DragMoreDesktopsDone"));

		/*
		 * Set new number of desktops
		 */
	iNoOfDesktopsHor = iDragHorDesktops;
	iNoOfDesktopsVert = iDragVertDesktops;
	iDeskwinEngine->SetNoOfDesktops(iNoOfDesktopsHor * iNoOfDesktopsVert);
	iActiveDesktop = iDeskwinEngine->CurrentDesktop();

	int oldX = iWindowSize.x;
	int oldY = iWindowSize.y;

		/*
		 * Correct window size
		 */
	iWindowSize.x	= iNoOfDesktopsHor * iDesktopPixelWidth + 2 * iDesktopOutFramePixelSize;
	iWindowSize.x	+= (iNoOfDesktopsHor - 1) * iDesktopInFramePixelSize;
	iWindowSize.y	= iNoOfDesktopsVert * iDesktopPixelHeight + 2 * iDesktopOutFramePixelSize;
	iWindowSize.y	+= (iNoOfDesktopsVert - 1) * iDesktopInFramePixelSize;

		/*
		 * Diffs
		 */
	int diffX = oldX - iWindowSize.x;
	int diffY = oldY - iWindowSize.y;

		/*
		 * adjust final position according to how we drag
		 */
	RECT		screenrect;
	GetWindowRect(&screenrect);

	if (iDragResizeArea == EResizeAreaTopLeft)
	{
		screenrect.left -= diffX; 
		screenrect.top -= diffY; 
	}
	else if (iDragResizeArea == EResizeAreaTopRight)
	{
		screenrect.top -= diffY; 
	}
	else if (iDragResizeArea == EResizeAreaBottomLeft)
	{
		screenrect.left -= diffX; 
	}
	else /* EResizeAreaLBottomRight */
	{
	}
		/*
		 * Set final position + height and set for redraw
		 */
	MoveWindow(screenrect.left, screenrect.top, iWindowSize.x, iWindowSize.y, true);

	ReleaseCapture();
	iWindowPositionTracker->ResumeWindowPositionTracking();

	SaveOptions();
	SaveState();
}


void CDeskwinWindow::DoDragMoreDesktops(TEvent aEvent)
{
	RECT		screenrect;
	GetWindowRect(&screenrect);

	DPRINT((HERE "CDeskwinWindow::DoDragMoreDesktops"));

		/*
		 * calc diff since original drag point
		 */
	int diffX = screenrect.left + iEvent.iMouse.x - iDragStartX;
	int diffY = screenrect.top + iEvent.iMouse.y - iDragStartY;

	int newX, newY;

		/*
		 * Determine the new window size
		 */
	if (iDragResizeArea == EResizeAreaTopLeft)
	{
		DPRINT((HERE "DoDragResize --> topleft"));
		newX = iDragStartSizeX - diffX;
		newY = iDragStartSizeY - diffY;
	}
	else if (iDragResizeArea == EResizeAreaTopRight)
	{
		DPRINT((HERE "DoDragResize --> topright"));
		newX = iDragStartSizeX + diffX;
		newY = iDragStartSizeY - diffY;
	}
	else if (iDragResizeArea == EResizeAreaBottomLeft)
	{
		DPRINT((HERE "DoDragResize --> bottomleft"));
		newX = iDragStartSizeX - diffX;
		newY = iDragStartSizeY + diffY;
	}
	else /* EResizeAreaLBottomRight */
	{
		DPRINT((HERE "DoDragResize --> bottomright"));
		newX = iDragStartSizeX + diffX;
		newY = iDragStartSizeY + diffY;
	}

		/*
		 * So, how much do we need to grow shrink right now ?
		 */
	diffX = newX - iWindowSize.x;
	diffY = newY - iWindowSize.y;
	iWindowSize.x += diffX;
	iWindowSize.y += diffY;


		/*
		 * Should we "snap" to the next or previous horizontal full window ?
		 */
	int		diffSnapX = 0;
	int		diffSnapY = 0;

	int		sizeOneDesktopX = iDesktopInFramePixelSize + iDesktopPixelWidth;
	int		sizeX = iWindowSize.x - (2 * iDesktopOutFramePixelSize);
	int		sizeFullWindowX = (sizeX / sizeOneDesktopX) * sizeOneDesktopX;
	int		diffWindowSizeX = sizeX - sizeFullWindowX;

	if ((diffWindowSizeX < ((sizeOneDesktopX * iOptions.iSnapResizePercent) / 100)) &&  (diffX < 0)
//			((diffX < 0) && ((iDragResizeArea == EResizeAreaTopRight) || (iDragResizeArea == EResizeAreaBottomRight)))
//			||
//			((diffX > 0) && ((iDragResizeArea == EResizeAreaTopLeft) || (iDragResizeArea == EResizeAreaBottomLeft)))
			)
	{
		iDragHorDesktops = (sizeX / sizeOneDesktopX);
		if (iDragHorDesktops == 0)
			iDragHorDesktops = 1;

		int snapWindowSizeX	= iDragHorDesktops * iDesktopPixelWidth + 2 * iDesktopOutFramePixelSize;
		snapWindowSizeX	+= (iDragHorDesktops - 1) * iDesktopInFramePixelSize;

		diffSnapX = iWindowSize.x - snapWindowSizeX;
		diffX -= diffSnapX;
		iWindowSize.x = snapWindowSizeX;
	}
	else if ((diffWindowSizeX > ((sizeOneDesktopX * (100-iOptions.iSnapResizePercent)) / 100)) &&  (diffX > 0)
//			((diffX > 0) && ((iDragResizeArea == EResizeAreaTopRight) || (iDragResizeArea == EResizeAreaBottomRight)))
//			||
//			((diffX < 0) && ((iDragResizeArea == EResizeAreaTopLeft) || (iDragResizeArea == EResizeAreaBottomLeft)))
			)
	{
		iDragHorDesktops = (sizeX / sizeOneDesktopX) + 1;
		if (iDragHorDesktops == 0)
			iDragHorDesktops = 1;

		int snapWindowSizeX	= iDragHorDesktops * iDesktopPixelWidth + 2 * iDesktopOutFramePixelSize;
		snapWindowSizeX	+= (iDragHorDesktops - 1) * iDesktopInFramePixelSize;

		diffSnapX = iWindowSize.x - snapWindowSizeX;
		diffX -= diffSnapX;
		iWindowSize.x = snapWindowSizeX;
	}


	int		sizeOneDesktopY = iDesktopInFramePixelSize + iDesktopPixelHeight;
	int		sizeY = iWindowSize.y - (2 * iDesktopOutFramePixelSize);
	int		sizeFullWindowY = (sizeY / sizeOneDesktopY) * sizeOneDesktopY;
	int		diffWindowSizeY = sizeY - sizeFullWindowY;
	if ((diffWindowSizeY < ((sizeOneDesktopY * iOptions.iSnapResizePercent) / 100)) &&
			((diffY < 0) && ((iDragResizeArea == EResizeAreaBottomLeft) || (iDragResizeArea == EResizeAreaBottomRight)))
			||
			((diffY > 0) && ((iDragResizeArea == EResizeAreaTopLeft) || (iDragResizeArea == EResizeAreaTopRight)))
			)
	{
		iDragVertDesktops = (sizeY / sizeOneDesktopY);
		if (iDragVertDesktops == 0)
			iDragVertDesktops = 1;

		int snapWindowSizeY	= iDragVertDesktops * iDesktopPixelHeight + 2 * iDesktopOutFramePixelSize;
		snapWindowSizeY	+= (iDragVertDesktops - 1) * iDesktopInFramePixelSize;

		diffSnapY = iWindowSize.y - snapWindowSizeY;
		diffY -= diffSnapY;
		iWindowSize.y = snapWindowSizeY;
	}
	else if ((diffWindowSizeY > ((sizeOneDesktopY * (100-iOptions.iSnapResizePercent)) / 100)) &&
			((diffY > 0) && ((iDragResizeArea == EResizeAreaBottomLeft) || (iDragResizeArea == EResizeAreaBottomRight)))
			||
			((diffY < 0) && ((iDragResizeArea == EResizeAreaTopLeft) || (iDragResizeArea == EResizeAreaTopRight)))
			)
	{
		iDragVertDesktops = (sizeY / sizeOneDesktopY) + 1;
		if (iDragVertDesktops == 0)
			iDragVertDesktops = 1;

		int snapWindowSizeY	= iDragVertDesktops * iDesktopPixelHeight + 2 * iDesktopOutFramePixelSize;
		snapWindowSizeY	+= (iDragVertDesktops - 1) * iDesktopInFramePixelSize;

		diffSnapY = iWindowSize.y - snapWindowSizeY;
		diffY -= diffSnapY;
		iWindowSize.y = snapWindowSizeY;
	}


	if ((diffSnapX != 0) || (diffSnapY != 0))
	{
		POINT	mousePos;
		GetCursorPos(&mousePos);
		mousePos.x -= diffSnapX;
		mousePos.y -= diffSnapY;
		SetCursorPos(mousePos.x, mousePos.y);
	}
	
	if (iDragResizeArea == EResizeAreaTopLeft)
	{
		screenrect.left -= diffX; 
		screenrect.top -= diffY; 
	}
	else if (iDragResizeArea == EResizeAreaTopRight)
	{
		screenrect.top -= diffY; 
	}
	else if (iDragResizeArea == EResizeAreaBottomLeft)
	{
		screenrect.left -= diffX; 
	}
	else /* EResizeAreaLBottomRight */
	{
	}

	if ((diffX != 0) || (diffY != 0))
		MoveWindow(screenrect.left, screenrect.top, iWindowSize.x, iWindowSize.y, true);

	::SetCursor((HICON)iCurrentCursor);
}


void CDeskwinWindow::SetDeskwinBaseOptions()
{
	iNoOfDesktopsHor			= iOptions.iNoOfDesktopsHor;
	iDesktopPixelWidth			= iOptions.iDesktopPixelWidth;
	iNoOfDesktopsVert			= iOptions.iNoOfDesktopsVert;
	iDesktopPixelHeight			= iOptions.iDesktopPixelHeight;
	iDesktopOutFramePixelSize	= iOptions.iDesktopOutFramePixelSize;
	iDesktopInFramePixelSize	= iOptions.iDesktopInFramePixelSize;
}


void CDeskwinWindow::GetDeskwinBaseOptions()
{
	iOptions.iNoOfDesktopsHor			= iNoOfDesktopsHor;
	iOptions.iDesktopPixelWidth			= iDesktopPixelWidth;
	iOptions.iNoOfDesktopsVert			= iNoOfDesktopsVert;
	iOptions.iDesktopPixelHeight		= iDesktopPixelHeight;
	iOptions.iDesktopOutFramePixelSize	= iDesktopOutFramePixelSize;
	iOptions.iDesktopInFramePixelSize	= iDesktopInFramePixelSize;
}

void CDeskwinWindow::SetColorOptions()
{
	iOuterFrameColor	= iOptions.iOuterFrameColor;
	iInnerFrameColor	= iOptions.iInnerFrameColor;
	iBackgroundColor	= iOptions.iBackgroundColor;
	iActiveDesktopColor = iOptions.iActiveDesktopColor;
	iWindowFrameColor	= iOptions.iWindowFrameColor;
	iWindowColor		= iOptions.iWindowColor;
}

void CDeskwinWindow::GetColorOptions()
{
	iOptions.iOuterFrameColor		= iOuterFrameColor;
	iOptions.iInnerFrameColor		= iInnerFrameColor;
	iOptions.iBackgroundColor		= iBackgroundColor;
	iOptions.iActiveDesktopColor	= iActiveDesktopColor;
	iOptions.iWindowFrameColor		= iWindowFrameColor;
	iOptions.iWindowColor			= iWindowColor;
}


void CDeskwinWindow::SaveOptions()
{
	GetColorOptions();
	GetDeskwinBaseOptions();

	::SaveOptions(iOptions);

}

void CDeskwinWindow::OnQueryEndSession()
{
}


void CDeskwinWindow::OnSysTrayEvent(int aParam)
{
	switch (aParam)
	{
		case WM_RBUTTONDOWN:
			POINT	mousePos;
			if (::GetCursorPos(&mousePos))
				ShowOptionsMenu(mousePos.x, mousePos.y);
			break;
			
		case WM_LBUTTONDBLCLK:
			if (!iOptions.iAlwaysOnTop)
			{
				SetForegroundWindow();	// make sure we have keyboard focus
				BringWindowToTop();
			}
			break;
		default:
			break;
	}
}


/*******************************************************************
 *
 * $Log: /Deskwin/Src/CDeskwinWindow.cpp $
 * 
 * 4     10/01/00 9:17 Pese
 * 
 * 3     5/01/00 20:26 Pese
 * 
 * 2     23/12/99 10:16 Pese
 *
 *******************************************************************/
