/*******************************************************************
 *
 *	File:		CDeskwinWindowBase.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 1999, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#include "Deskwin.h"
#include "CDeskWinWindowBase.h"

#include "Resource.h"



CDeskwinWindowBase::CDeskwinWindowBase() : CWindowEx(), iPaintBitmap(NULL)
{
	GetScreenSize(iScreenSize.x, iScreenSize.y);

	iNoOfDesktopsHor = 2;
	iDesktopPixelWidth = 50;

	iNoOfDesktopsVert = 2;
	iDesktopPixelHeight = 50;

	iDesktopOutFramePixelSize = 6;
	iDesktopInFramePixelSize = 1;

	SetDefaultColors();
}


void CDeskwinWindowBase::SetDefaultColors()
{
	iOuterFrameColor		= ct_LightPurple;
	iInnerFrameColor		= ct_LightPurple;
	iBackgroundColor		= ct_LightBlue;
	iActiveDesktopColor		= ct_DarkPurple;
	iWindowFrameColor		= ct_Black;
	iWindowColor			= ct_White;
}


CDeskwinWindowBase::~CDeskwinWindowBase()
{
	DeleteObject(iPaintBitmap);
	iPaintBitmap = NULL;
}


void CDeskwinWindowBase::Redraw()
{
	RECT	clientRect;

	GetClientRect(&clientRect);
	InvalidateRect(&clientRect, false);
	UpdateWindow();
}

void CDeskwinWindowBase::SetSize(int aWidth, int aHeight)
{
	int desktopsX = aWidth - 2 * iDesktopOutFramePixelSize;
	int desktopsY = aHeight - 2 * iDesktopOutFramePixelSize;
	desktopsX -= (iNoOfDesktopsHor - 1) * iDesktopInFramePixelSize;
	desktopsY -= (iNoOfDesktopsVert - 1) * iDesktopInFramePixelSize;

	iDesktopPixelWidth = desktopsX / iNoOfDesktopsHor;
	iDesktopPixelHeight = desktopsY / iNoOfDesktopsVert;
}

void CDeskwinWindowBase::Construct(int aPosX, int aPosY, DWORD aStyle, DWORD aExStyle, int aWidth, int aHeight)
{
	if ((aWidth != -1) || (aHeight != -1))
	{
		SetSize(aWidth, aHeight);
	}

	iWindowSize.x	= iNoOfDesktopsHor * iDesktopPixelWidth + 2 * iDesktopOutFramePixelSize;
	iWindowSize.y	= iNoOfDesktopsVert * iDesktopPixelHeight + 2 * iDesktopOutFramePixelSize;

	iWindowSize.x	+= (iNoOfDesktopsHor - 1) * iDesktopInFramePixelSize;
	iWindowSize.y	+= (iNoOfDesktopsVert - 1) * iDesktopInFramePixelSize;
	
	DPRINT((HERE "Construct, width=%i, height=%i", iWindowSize.x, iWindowSize.y));

	bool result = Create(NULL, aPosX, aPosY, iWindowSize.x, iWindowSize.y, 0, aStyle, NULL, aExStyle);
}



void CDeskwinWindowBase::PaintFrames(CDC& aDc, RECT& aRect)
{
	CPen	outFramePen(iOuterFrameColor, iDesktopOutFramePixelSize*2);
	CPen	inFramePen(iInnerFrameColor, iDesktopInFramePixelSize);
	CPen	whitePen(ct_White, 1);
	CPen	blackPen(ct_Black, 1);
	CBrush	fillBrush(iBackgroundColor);


	int		windowSize = aRect.right;
	aDc.SelectObject(outFramePen);
	aDc.SelectObject(fillBrush);
	aDc.Rectangle(aRect.left, aRect.top, aRect.right, aRect.bottom);

	aDc.SelectObject(whitePen);
	aDc.MoveTo(1, 1);
	aDc.LineTo(aRect.right - 1, 1);
	aDc.MoveTo(1, 1);
	aDc.LineTo(1, aRect.bottom - 1);

	aDc.SelectObject(blackPen);
	aDc.MoveTo(aRect.right - 1, 1);
	aDc.LineTo(aRect.right - 1, iWindowSize.y - 1);
	aDc.MoveTo(1, aRect.bottom - 1);
	aDc.LineTo(aRect.right, aRect.bottom - 1);

		/*
		 * Draw vertical innerframes
		 */
	aDc.SelectObject(inFramePen);
	int x=iDesktopOutFramePixelSize + iDesktopPixelWidth;
	int VertLineSize = (iNoOfDesktopsVert * iDesktopPixelHeight) + ((iNoOfDesktopsVert - 1) * iDesktopInFramePixelSize);
	for (int h =0; h<iNoOfDesktopsHor-1; h++)
	{
		aDc.MoveTo(x, iDesktopOutFramePixelSize);
		aDc.LineTo(x, VertLineSize + iDesktopOutFramePixelSize);
		x += iDesktopPixelWidth + iDesktopInFramePixelSize;
	}
 
		/*
		 * Draw horizontal innerframes
		 */
	int y=iDesktopOutFramePixelSize + iDesktopPixelHeight;
	int HorLineSize = (iNoOfDesktopsHor * iDesktopPixelWidth) + ((iNoOfDesktopsHor - 1) * iDesktopInFramePixelSize);
	for (int v =0; v<iNoOfDesktopsVert-1; v++)
	{
		aDc.MoveTo(iDesktopOutFramePixelSize, y);
		aDc.LineTo(iDesktopOutFramePixelSize + HorLineSize, y);
		y += iDesktopPixelHeight + iDesktopInFramePixelSize;
	}

	aDc.RestoreObjects();
}





void CDeskwinWindowBase::InflateDesktopRectToScreenRect(RECT& rect)
{
	rect.left	= (rect.left * iScreenSize.x) / iDesktopPixelWidth;
	rect.right	= (rect.right * iScreenSize.x) / iDesktopPixelWidth;
	rect.top	= (rect.top * iScreenSize.y) / iDesktopPixelHeight;
	rect.bottom = (rect.bottom * iScreenSize.y) / iDesktopPixelHeight;

}

void CDeskwinWindowBase::ShrinkScreenRectToDesktopRect(RECT& rect, bool aClip)
{
		/*
		 * shrink to fit screen
		 */
	if (aClip)
	{
		if (rect.left < 0) rect.left =0;
		if (rect.right > iScreenSize.x) rect.right = iScreenSize.x;
		if (rect.top < 0) rect.top = 0;
		if (rect.bottom > iScreenSize.y) rect.bottom = iScreenSize.y;
	}

		/*
		 * ... and scale
		 */
	rect.left	= (rect.left * iDesktopPixelWidth) / iScreenSize.x;
	rect.right	= (rect.right * iDesktopPixelWidth) / iScreenSize.x;
	rect.top	= (rect.top * iDesktopPixelHeight) / iScreenSize.y;
	rect.bottom = (rect.bottom * iDesktopPixelHeight) / iScreenSize.y;
}

void CDeskwinWindowBase::GetDesktopRect(int aDesktop, RECT& aRect)
{
	int h = aDesktop % iNoOfDesktopsHor;
	int v = aDesktop / iNoOfDesktopsHor;

	aRect.left = iDesktopOutFramePixelSize + (h * (iDesktopPixelWidth + iDesktopInFramePixelSize));
	aRect.top = iDesktopOutFramePixelSize + (v * (iDesktopPixelHeight + iDesktopInFramePixelSize));

	aRect.right = aRect.left + iDesktopPixelWidth + iDesktopInFramePixelSize;
	aRect.bottom = aRect.top + iDesktopPixelHeight + iDesktopInFramePixelSize;
}


void CDeskwinWindowBase::PaintDesktop(CDC& dc, int aDesktop, bool aIsActive, TWindowList& aWindowList, TWindow aExplorerWindow, TWindow aTaskbarWindow, TWindow aDragWindow)
{
	RECT	myRect;

//	DPRINT((HERE "PaintDesktop, Desktop =%i", aDesktop));

	GetDesktopRect(aDesktop, myRect);

	CBrush	backgroundBrush(aIsActive ? iActiveDesktopColor : iBackgroundColor);
	CBrush	windowBrush(iWindowColor);
	CPen	backgroundPen(aIsActive ? iActiveDesktopColor : iBackgroundColor, 1);
	CPen	windowframePen(iWindowFrameColor, 1);

	dc.SelectObject(backgroundBrush);
	dc.SelectObject(backgroundPen);
	dc.Rectangle(myRect.left, myRect.top, myRect.left + iDesktopPixelWidth, myRect.top + iDesktopPixelHeight);

	dc.SelectObject(windowframePen);
	dc.SelectObject(windowBrush);
	dc.MoveTo(0, 0);

	for (TWindowList::reverse_iterator i=aWindowList.rbegin() ; i != aWindowList.rend() ; i++)
	{
		if (::IsIconic(*i) || !::IsWindow(*i))
			continue;

			/*
			 * don't paint explorer or taskbar on any desktop
			 */
		if (aExplorerWindow == *i || aTaskbarWindow == *i )
			continue;

			/*
			 * only paint ourself on the active desktop
			 */
		if ((HWindow == *i) && (!aIsActive))
			continue;

		RECT		rect;
		::GetWindowRect(*i, &rect);

			/*
			 * All but the window currently being dragged are clipped in their
			 * desktop
			 */
		bool shouldClip = !((*i == aDragWindow) && (aIsActive));
		ShrinkScreenRectToDesktopRect(rect, shouldClip);

		rect.left += myRect.left;
		rect.top += myRect.top;
		rect.right += myRect.left;
		rect.bottom += myRect.top;

		if (shouldClip)
		{
			ForceToBeInRange(rect.right, 0l, (long) myRect.right);
			ForceToBeInRange(rect.bottom, 0l, (long) myRect.bottom);
		}

//		DPRINT((HERE "CDeskwinWindowBase::PaintDesktop, desktop=%i, window=0x%x, (%i, %i) (%i, %i)", aDesktop, (int) *i, rect.left, rect.top, rect.right, rect.bottom));

		if (!IsStickyWindow(aDesktop, *i))
			dc.Rectangle(rect.left, rect.top, rect.right, rect.bottom);
		else
			dc.RoundRect(rect.left, rect.top, rect.right, rect.bottom, 5, 5);
	}

	dc.RestoreObjects();
}



void CDeskwinWindowBase::Paint(CDC& aDc)
{
	static int		sOldX = 0;
	static int		sOldY = 0;

		/*
		 * Usual method to prevent flicker....
		 * Create a Device Context in memory, use that to paint and after that
		 * BitBlit it to the screen's Device Context
		 *
		 * Only create a new bitmap if the window size has changed. Also, the
		 * bitmap creation may fail in which case we use the original DC.
		 */
	CMemoryDC	memDc;
	if ((sOldX != iWindowSize.x) || (sOldY != iWindowSize.y) || !iPaintBitmap)
	{
		if (iPaintBitmap)
		{
			DeleteObject(iPaintBitmap);
			iPaintBitmap = NULL;
		}
		iPaintBitmap = aDc.CreateCompatibleBitmap(iWindowSize.x, iWindowSize.y);
		sOldX = iWindowSize.x;
		sOldY = iWindowSize.y;
	}

	if (iPaintBitmap)
	{
		memDc.SelectBitmap(iPaintBitmap);
		DoPaint(memDc);
		aDc.BitBlt(
					m_PaintStruct.rcPaint.left, 
					m_PaintStruct.rcPaint.top, 
					m_PaintStruct.rcPaint.right - m_PaintStruct.rcPaint.left, 
					m_PaintStruct.rcPaint.bottom - m_PaintStruct.rcPaint.top, 
					memDc,
					m_PaintStruct.rcPaint.left, 
					m_PaintStruct.rcPaint.top
					);
	}
	else
	{
		DoPaint(aDc);
	}
}








/*******************************************************************
 *
 * $Log: /Deskwin/Src/CDeskwinWindowBase.cpp $
 * 
 * 3     10/01/00 9:17 Pese
 * 
 * 2     5/01/00 20:26 Pese
 * 
 * 1     23/12/99 10:17 Pese
 *
 *******************************************************************/
