/*******************************************************************
 *
 *	File:		CDeskwinWindowBase.h
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 1999, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#ifndef __CDESKWIN_WINDOW_BASE_H
#define __CDESKWIN_WINDOW_BASE_H

#include "CDeskwinEngine.h"

template<class T> void ForceToBeInRange(T& aT, T aLBound, T aUBound)
{
	if (aT < aLBound)
		aT = aLBound;
	else if (aT > aUBound)
		aT = aUBound;
}

template<class T> bool IsInRange(T& aT, T aLBound, T aUBound)
{
	return ((aT >= aLBound) && (aT<aUBound));
}




class CDeskwinWindowBase : public CWindowEx
{
public:
	CDeskwinWindowBase();
	~CDeskwinWindowBase();

	void Construct(int aPosX, int aPosY, DWORD aStyle, DWORD aExStyle, int aWidth = -1, int aHeight = -1);
	void Redraw();
	void SetDefaultColors();

	COLORREF	iOuterFrameColor;
	COLORREF	iInnerFrameColor;
	COLORREF	iBackgroundColor;
	COLORREF	iActiveDesktopColor;
	COLORREF	iWindowFrameColor;
	COLORREF	iWindowColor;

protected:
	void			Paint(CDC& dc);
	virtual void	DoPaint(CDC& dc) = 0;		// which makes us a pure virtual
	virtual bool	IsStickyWindow(int aDesktop, TWindow aWindow){return FALSE;}

	void SetSize(int aWidth, int aHeight);
	void ShrinkScreenRectToDesktopRect(RECT& rect, bool aClip = true);
	void InflateDesktopRectToScreenRect(RECT& rect);
	void GetDesktopRect(int aDesktop, RECT& aRect);

	void PaintDesktop(CDC& dc, int aDesktop, bool aIsActive, TWindowList& aWindowList, TWindow aExplorerWindow, TWindow aTaskbarWindow, TWindow aDragWindow);
	void PaintFrames(CDC& aDc, RECT& aRect);

protected:
	POINT				iWindowSize;		// our windowsize
	POINT				iScreenSize;

	int		iDesktopOutFramePixelSize;
	int		iDesktopInFramePixelSize;

	int		iNoOfDesktopsHor;
	int		iNoOfDesktopsVert;

	int		iDesktopPixelWidth;
	int		iDesktopPixelHeight;

private:
	HBITMAP				iPaintBitmap;
};


/*******************************************************************
 *
 * $Log: /Deskwin/Src/CDeskwinWindowBase.h $
 * 
 * 3     10/01/00 9:17 Pese
 * 
 * 2     5/01/00 20:26 Pese
 * 
 * 1     23/12/99 10:17 Pese
 *
 *******************************************************************/


#endif			/* __CDESKWIN_WINDOW_BASE_H */