/*******************************************************************
 *
 *	File:		CHyperLinkText.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 1999, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#include "Deskwin.h"
#include "CHyperLinkText.h"
#include "resource.h"				// for cursors

#define MAX_TEXT 4096


CHyperLinkText::CHyperLinkText(HWND aStaticWindow, COLORREF aFgColor, COLORREF aBgColor) : CWindowEx(aStaticWindow),
	iFont("Arial", 16), iSelectedFont("Arial", 16, 0, FW_NORMAL, 0, 1), iHorAlignment(EHorLeft), iVertAlignment(EVertTop), iFgColor(aFgColor), iSelected(false)
{
	iCursor = ::LoadCursor(GetAppInstance(), MAKEINTRESOURCE(IDC_CURSOR_HAND));
	InitMessageHandlers();
	SetFont(iFont);
	iRect.left = iRect.right = iRect.top = iRect.bottom = 0;
}

CHyperLinkText::~CHyperLinkText()
{
	CloseHandle(iCursor);
	iCursor = NULL;
}

LRESULT CHyperLinkText::Dispatch(unsigned int nMessage, WPARAM wParam, LPARAM lParam)
{
	return CWindow::Dispatch(nMessage, wParam, lParam);
}


void CHyperLinkText::InitMessageHandlers()
{
	AddMsg(SWinMsg(WM_LBUTTONDOWN, (CWMSGHANDLER)CHyperLinkText::OnWmLButtonDown));
	AddMsg(SWinMsg(WM_LBUTTONUP, (CWMSGHANDLER)CHyperLinkText::OnWmLButtonUp));
	AddMsg(SWinMsg(WM_MOUSEMOVE, (CWMSGHANDLER)CHyperLinkText::OnWmMouseMove));
}



void CHyperLinkText::OnLButtonDown(int fwKeys, int aX, int aY)
{
	POINT	mousePoint = {aX, aY};
	if (PtInRect(&iRect, mousePoint))
	{
 		::SetCursor(iCursor);
	}
}



void CHyperLinkText::DoHyperLink()
{
	char		text[MAX_TEXT];
	int			textLen = GetWindowTextLength();

	GetWindowText(text, MAX_TEXT);
	text[MAX_TEXT - 1] = 0;

	if (ShellExecute(GetDesktopWindow(), "open", text, NULL, NULL, SW_SHOWNORMAL) <= (HINSTANCE) 32)
	{
		char		errorTitle[MAX_TEXT];
		char		errorText[MAX_TEXT];
		char		errorText1[MAX_TEXT];

		errorTitle[0] = 0;
		LoadString(GetAppInstance(), IDS_STRING_HYPER_ERROR_TITLE, errorTitle, MAX_TEXT); 
		errorTitle[MAX_TEXT - 1] = 0;

		errorText[0] = 0;
		LoadString(GetAppInstance(), IDS_STRING_HYPER_ERROR_TEXT, errorText, MAX_TEXT); 
		errorText[MAX_TEXT - 1] = 0;

		sprintf(errorText1, errorText, text);

		MessageBox(errorText1, errorTitle, MB_ICONERROR);
	}
}


void CHyperLinkText::OnLButtonUp(int fwKeys, int aX, int aY)
{
	POINT	mousePoint = {aX, aY};
	if (PtInRect(&iRect, mousePoint))
	{
 		::SetCursor(iCursor);
		DoHyperLink();
	}
}

void CHyperLinkText::OnMouseMove(int fwKeys, int aX, int aY)
{
	bool oldSelected = iSelected;

	POINT	mousePoint = {aX, aY};
	if (PtInRect(&iRect, mousePoint))
	{
 		::SetCursor(iCursor);
		iSelected = true;

		SetCapture();
	}
	else
	{
		iSelected = false;
		ReleaseCapture();
	}
	if (oldSelected != iSelected)
	{
		RECT	clientRect;

		GetClientRect(&clientRect);
		InvalidateRect(&clientRect, true);
	}
}

void CHyperLinkText::SetWindowText(CDC& aDc, const char* aText, int aTextLen)
{
	RECT	clientRect;
	GetClientRect(&clientRect);

	SIZE	size;
	::GetTextExtentPoint32(aDc, aText, aTextLen, &size);

	POINT	point = {size.cx, size.cy};
	LPtoDP(aDc, &point, 1); 
	size.cx = point.x;
	size.cy = point.y;

	if (size.cx > clientRect.right)
		size.cx = clientRect.right;
	if (size.cy > clientRect.bottom)
		size.cy = clientRect.bottom;

	switch(iHorAlignment)
	{
		case EHorLeft:
			iRect.left = 0;
			iRect.right = size.cx;
			break;

		case EHorRight:
			iRect.left = clientRect.right - size.cx;
			iRect.right = clientRect.right;
			break;

		case EHorCenter:
			int		gap = (clientRect.right - size.cx) >> 1;
			iRect.left = gap;
			iRect.right = gap + size.cx;
			break;
	}
	
	switch(iVertAlignment)
	{
		case EVertTop:
			iRect.top = 0;
			iRect.bottom = size.cy;
			break;

		case EVertBottom:
			iRect.top = clientRect.bottom - size.cy;
			iRect.bottom = clientRect.bottom;
			break;

		case EVertCenter:
			int		gap = (clientRect.bottom - size.cy) >> 1;
			iRect.top = gap;
			iRect.bottom = size.cy + gap;
			break;
	}
}

void CHyperLinkText::Paint(CDC& aDc)
{
	COLORREF	bgColor = GetSysColor(COLOR_ACTIVEBORDER);
	char		text[MAX_TEXT];
	int			textLen = GetWindowTextLength();

	GetWindowText(text, MAX_TEXT);
	text[MAX_TEXT - 1] = 0;

	if (iSelected)
		aDc.SelectObject(iSelectedFont);
	else
		aDc.SelectObject(iFont);

	SetWindowText(aDc, text, textLen);

	aDc.SetTextColor(iFgColor);
	aDc.SetBkColor(bgColor);

	aDc.TextOut(iRect.left, iRect.top, text, textLen);

	aDc.RestoreObjects();
}
