/*******************************************************************
 *
 *	File:		CInfoWindow.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 1999, 2000, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#include "Deskwin.h"
#include "CInfoWindow.h"

CInfoWindow::CInfoWindow() : CWindowEx(), iPen(ct_Black, 1)
{
	AddMsg(SWinMsg(WM_TIMER, (CWMSGHANDLER)CInfoWindow::OnWmTimer));
}



void CInfoWindow::OnTimer()
{
	ShowWindow(SW_HIDE);
}

void CInfoWindow::SetInfoText(int aResourceId)
{
	char	text[4096];

	text[0] = 0;
	LoadString(GetAppInstance(), aResourceId, text, 4096); 
	text[4096 - 1] = 0;

	SetInfoText(text);	
}


void CInfoWindow::Construct(HWND aParent)
{
	// need our own DC, so register a new class
	GetApplication()->RegisterClass("CInfoWindow", NULL, NULL, -1, 0, CS_HREDRAW | CS_VREDRAW | CS_OWNDC);

	bool result = Create("CInfoWindow", 0, 0, 1, 1, 0, WS_POPUP, NULL, WS_EX_TOPMOST | WS_EX_TOOLWINDOW);
	if (result)
	{
#if 1
		HFONT font = CreateFont(
			10 * GetDeviceCaps(GetDC(), LOGPIXELSY) / 72,
			0,
			0,
			0,
			FW_NORMAL,
			0,
			0,
			0,
			DEFAULT_CHARSET,
			OUT_DEFAULT_PRECIS,
			CLIP_DEFAULT_PRECIS,
			PROOF_QUALITY,
			DEFAULT_PITCH | FF_DONTCARE,
			"Arial"
			);
		::SelectObject(GetDC(), font);
#endif
		::SelectObject(GetDC(), iPen);

		SetBkColor(GetDC(), 0xdff2f1);
	}
}

void CInfoWindow::Paint(CDC& aDc)
{
	RECT	textRect;
	CBrush	BlackBrush(ct_Black);
	CBrush	GrayBrush(ct_LightGray);
	CBrush	bkBrush(0xdff2f1);
	CBrush	crossBrush(ct_DarkGray);

	GetClientRect(&textRect);
	
	aDc.SelectObject(bkBrush);
	aDc.Rectangle(textRect.left, textRect.top, textRect.right, textRect.bottom);

	InflateRect(&textRect, -1, -1);
	aDc.FrameRect(&textRect, GrayBrush);

	InflateRect(&textRect, -4, -4);
	DPRINT((HERE "CInfoWindow::Paint, rect.w=%d, rect.h=%d", textRect.right, textRect.bottom));
	aDc.DrawText(iInfoString.c_str(), iInfoString.size(), &textRect, 0);
}


void CInfoWindow::Popup(int aMouseX, int aMouseY)
{
	DPRINT((HERE "CInfoWindow::Popup, x=%d, y=%d", aMouseX, aMouseY));

	KillTimer(0);

	RECT	rect;
	long	screenX, screenY;
	long	windowX, windowY;
	
	GetScreenSize(screenX, screenY);
	
	rect.top = rect.bottom = rect.left = 0;
	rect.right = 512;
	rect.bottom = ::DrawText(GetDC(), iInfoString.c_str(), iInfoString.size(), &rect, DT_CALCRECT);

	rect.right += 10;
	rect.bottom += 10;

	windowX = aMouseX - ((rect.right * 2 ) / 3);
	if (windowX + rect.right + 25> screenX)
		windowX = screenX - rect.right -25;
	windowY = aMouseY + 8;		// move down almost the size of the pointer
	if (windowY + rect.bottom + 25 > screenY)
		windowY = screenY - rect.bottom -25;
	
	DPRINT((HERE "CInfoWindow::Popup, x=%d, y=%d, w=%d, h=%d", windowX, windowY, rect.right, rect.bottom));
	SetWindowPos(HWND_TOPMOST, windowX, windowY, rect.right, rect.bottom, 0);
	ShowWindow(SW_SHOW);
	
	SetTimer(0, 2500);
}

