/*******************************************************************
 *
 *	File:		COptionDialogs.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 1999, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#include "Deskwin.h"
#include <COMMCTRL.H>
#include <COMMDLG.H>

#include "COptionDialogs.h"
#include "Resource.h"
#include "CDeskwinWindowBase.h"
#include "CHyperLinkText.h"
#include "CSysTrayIconUtil.h"

#define MAX_TEXT 1024
#define MAX_ABOUT_TEXT 4096

static void PositionDialog(CDialog& aDialog)
{
	long screenX, screenY;
	GetScreenSize(screenX, screenY);
	screenX -= 10;
	screenY -= 10;


	RECT	dialogRect;
	aDialog.GetWindowRect(&dialogRect);

	POINT	cursorPos;
	GetCursorPos(&cursorPos);

		// position dialog sothat mouse cursor is in middle
	RECT	desiredRect;
	desiredRect.left = cursorPos.x - ((dialogRect.right - dialogRect.left) / 2);
	desiredRect.right = desiredRect.left + (dialogRect.right - dialogRect.left);
	desiredRect.top = cursorPos.y - ((dialogRect.bottom - dialogRect.top) / 2);
	desiredRect.bottom = desiredRect.top + (dialogRect.bottom - dialogRect.top);

		// reposition to make dialog completely visible
	if (desiredRect.left < 0)
		desiredRect.left = 0;
	else if (desiredRect.right > screenX)
		desiredRect.left -=  (desiredRect.right - screenX);
	if (desiredRect.top <0)
		desiredRect.top = 0;
	else if (desiredRect.bottom > screenY)
		desiredRect.top -= (desiredRect.bottom - screenY);

	aDialog.SetWindowPos(HWND_TOPMOST, desiredRect.left, desiredRect.top, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
}


/************************************************
 *
 * Docking options
 *
 ************************************************/

CDockOptionsDialog::CDockOptionsDialog(TDeskwinOptions& aOptions) : CDialog((int) IDD_DIALOG_DOCK), iOptions(aOptions)
{
	AddCmd(SWinCmd(IDC_DOCK_CHECK, (CWCMDHANDLER) CDockOptionsDialog::OnToggleEnableDocking, BN_CLICKED));
	AddMsg(SWinMsg(WM_VSCROLL, (CWMSGHANDLER)CDockOptionsDialog::OnVScroll));
}


long CDockOptionsDialog::OnVScroll(WPARAM wParam, LPARAM lParam)
{
	int pos = (short int) HIWORD(wParam);
	
	iOptions.iDockPercentFromEdge = pos;
	SetDockingPercentage();

	return 1;
}



void CDockOptionsDialog::OnToggleEnableDocking()
{
	bool isChecked = (IsDlgButtonChecked(IDC_DOCK_CHECK) == BST_CHECKED);
	iOptions.iWantsDocking = isChecked;
	::EnableWindow(GetDlgItem(IDC_PERCENT_FROM_EDGE), isChecked);
	::EnableWindow(GetDlgItem(IDC_SPIN_DOCK_EDGE), isChecked);
}

void CDockOptionsDialog::SetupDialog()
{
	::SendMessage(GetDlgItem(IDC_SPIN_DOCK_EDGE), UDM_SETRANGE, 0, MAKELONG((short) 40, (short) 1));
	::SendMessage(GetDlgItem(IDC_SPIN_DOCK_EDGE), UDM_SETPOS, 0, MAKELONG((short) iOptions.iDockPercentFromEdge, (short) 0));
	CheckDlgButton(IDC_DOCK_CHECK, iOptions.iWantsDocking ? BST_CHECKED : BST_UNCHECKED);

	SetDockingPercentage();
	OnToggleEnableDocking();

	PositionDialog(*this);
}


void CDockOptionsDialog::SetDockingPercentage()
{
	char	Text1[MAX_TEXT];
	char	Text2[MAX_TEXT];

	Text1[0] = 0;
	LoadString(GetAppInstance(), IDS_DOCK_PERCENT_TEXT, Text1, MAX_TEXT); 
	Text1[MAX_TEXT - 1] = 0;
 
	sprintf(Text2, Text1, iOptions.iDockPercentFromEdge);

	SetDlgItemText(IDC_PERCENT_FROM_EDGE, Text2);
}


/************************************************
 *
 * Auto hide options
 *
 ************************************************/

CAutoHideOptionsDialog::CAutoHideOptionsDialog(TDeskwinOptions& aOptions) : CDialog((int) IDD_DIALOG_AUTO_HIDE), iOptions(aOptions)
{
	AddCmd(SWinCmd(IDC_CHECK_AUTO_HIDE, (CWCMDHANDLER) CAutoHideOptionsDialog::OnToggleEnableAutoHide, BN_CLICKED));
	AddMsg(SWinMsg(WM_VSCROLL, (CWMSGHANDLER)CAutoHideOptionsDialog::OnVScroll));
}


long CAutoHideOptionsDialog::OnVScroll(WPARAM wParam, LPARAM lParam)
{
	HWND window = (HWND) lParam;

	int pos = (short int) HIWORD(wParam);
	if (window == GetDlgItem(IDC_SPIN_PERCENT_VISIBLE))
	{
		iOptions.iPercentVisible = pos;
		SetVisiblePercentage();
	}
	else
	{
		iOptions.iAutoHideTimeOut = pos * 100;
		SetInactivityTimeout();
	}
	return 1;
}



void CAutoHideOptionsDialog::OnToggleEnableAutoHide()
{
	iOptions.iWantsAutoHide = (IsDlgButtonChecked(IDC_CHECK_AUTO_HIDE) == BST_CHECKED);
	::EnableWindow(GetDlgItem(IDC_SPIN_PERCENT_VISIBLE), iOptions.iWantsAutoHide);
	::EnableWindow(GetDlgItem(IDC_SPIN_INACTIVE_TIME), iOptions.iWantsAutoHide);
	::EnableWindow(GetDlgItem(IDC_STATIC_TIME_INACTIVE), iOptions.iWantsAutoHide);
	::EnableWindow(GetDlgItem(IDC_STATIC_PERCENT_VISIBLE), iOptions.iWantsAutoHide);
	::EnableWindow(GetDlgItem(IDC_RADIO_SHOW_BY_MOUSE_MOVE), iOptions.iWantsAutoHide);
	::EnableWindow(GetDlgItem(IDC_RADIO_SHOW_BY_MOUSE_CLICK), iOptions.iWantsAutoHide);
}


void CAutoHideOptionsDialog::SetupDialog()
{
	::SendMessage(GetDlgItem(IDC_SPIN_INACTIVE_TIME), UDM_SETRANGE, 0, MAKELONG((short) 100, (short) 1));
	::SendMessage(GetDlgItem(IDC_SPIN_PERCENT_VISIBLE), UDM_SETRANGE, 0, MAKELONG((short) 100, (short) 1));

	::SendMessage(GetDlgItem(IDC_SPIN_INACTIVE_TIME), UDM_SETPOS, 0, MAKELONG((short) (iOptions.iAutoHideTimeOut / 100), (short) 0));
	::SendMessage(GetDlgItem(IDC_SPIN_PERCENT_VISIBLE), UDM_SETPOS, 0, MAKELONG((short) iOptions.iPercentVisible, (short) 0));
	CheckDlgButton(iOptions.iWantsShowByMouseClick ? IDC_RADIO_SHOW_BY_MOUSE_CLICK : IDC_RADIO_SHOW_BY_MOUSE_MOVE, BST_CHECKED);
	CheckDlgButton(IDC_CHECK_AUTO_HIDE, iOptions.iWantsAutoHide ? BST_CHECKED : BST_UNCHECKED);

	SetVisiblePercentage();
	SetInactivityTimeout();
	OnToggleEnableAutoHide();
	PositionDialog(*this);
}


void CAutoHideOptionsDialog::SetVisiblePercentage()
{
	char	Text1[MAX_TEXT];
	char	Text2[MAX_TEXT];

	Text1[0] = 0;
	LoadString(GetAppInstance(), IDS_AUTO_HIDE_PERCENT_VISIBLE_TEXT, Text1, MAX_TEXT); 
	Text1[MAX_TEXT - 1] = 0;
 
	sprintf(Text2, Text1, iOptions.iPercentVisible);

	SetDlgItemText(IDC_STATIC_PERCENT_VISIBLE, Text2);
}


void CAutoHideOptionsDialog::SetInactivityTimeout()
{
	char	Text1[MAX_TEXT];
	char	Text2[MAX_TEXT];

	Text1[0] = 0;
	LoadString(GetAppInstance(), IDS_AUTO_HIDE_INACTIVE_TIMEOUT, Text1, MAX_TEXT); 
	Text1[MAX_TEXT - 1] = 0;
 
	sprintf(Text2, Text1, ((float) iOptions.iAutoHideTimeOut) / 1000.0);

	SetDlgItemText(IDC_STATIC_TIME_INACTIVE, Text2);
}

void CAutoHideOptionsDialog::CmOk()
{
	iOptions.iWantsShowByMouseClick = (IsDlgButtonChecked(IDC_RADIO_SHOW_BY_MOUSE_CLICK) == BST_CHECKED);

	CDialog::CmOk();
}


/************************************************
 *
 * Hotkey options
 *
 ************************************************/


CHotkeyOptionsDialog::CHotkeyOptionsDialog(TDeskwinOptions& aOptions) : CDialog((int) IDD_DIALOG_HOTKEYS), iOptions(aOptions), iCurDesktop(0)
{
	AddCmd(SWinCmd(IDC_RADIO_INDIVUAL, (CWCMDHANDLER) CHotkeyOptionsDialog::OnDoIndividual, BN_CLICKED));
	AddCmd(SWinCmd(IDC_RADIO_GROUP, (CWCMDHANDLER) CHotkeyOptionsDialog::OnDoGroup, BN_CLICKED));
	AddCmd(SWinCmd(IDC_COMBO_DESKTOP, (CWCMDHANDLER) CHotkeyOptionsDialog::OnSelectNewDesktop, CBN_SELCHANGE));
}


void CHotkeyOptionsDialog::OnDoIndividual()
{
	DPRINT((HERE "OnDoIndividual"));
	::ShowWindow(GetDlgItem(IDC_STATIC_DESKTOP_1), SW_HIDE);
	::ShowWindow(GetDlgItem(IDC_COMBO_DESKTOP), SW_SHOW);

	iCurDesktop = 0;
	CHotkeyOptionsDialog::SetDesktopHotkey(iCurDesktop);
	::SendMessage(GetDlgItem(IDC_COMBO_DESKTOP), CB_SETCURSEL, 0, 0);
}


void CHotkeyOptionsDialog::OnDoGroup()
{
	DPRINT((HERE "OnDoGroup"));
	::ShowWindow(GetDlgItem(IDC_STATIC_DESKTOP_1), SW_SHOW);
	::ShowWindow(GetDlgItem(IDC_COMBO_DESKTOP), SW_HIDE);

	CHotkeyOptionsDialog::SetDesktopHotkey(0);
	iCurDesktop = -1;
}


void CHotkeyOptionsDialog::OnSelectNewDesktop()
{
	int newDesktop = ::SendMessage(GetDlgItem(IDC_COMBO_DESKTOP), CB_GETCURSEL, 0, 0);

	GetDesktopHotkey(iCurDesktop);
	iCurDesktop = newDesktop;
	SetDesktopHotkey(iCurDesktop);
}


void CHotkeyOptionsDialog::SetupDialog()
{
	WORD	wParam;

	MakeWParmHotkeyMessage(iOptions.iHotkeyShowDeskwin, wParam);
	::SendMessage(GetDlgItem(IDC_HOTKEY_SHOW_DESKWIN), HKM_SETHOTKEY, wParam, 0);

	MakeWParmHotkeyMessage(iOptions.iHotkeyMinimiseAll, wParam);
	::SendMessage(GetDlgItem(IDC_HOTKEY_MINIMISE_ALL), HKM_SETHOTKEY, wParam, 0);

	MakeWParmHotkeyMessage(iOptions.iHotkeyToggleExplorer, wParam);
	::SendMessage(GetDlgItem(IDC_HOTKEY_TOGGLE_DESKTOP), HKM_SETHOTKEY, wParam, 0);

	MakeWParmHotkeyMessage(iOptions.iHotkeyPreviousDesktop, wParam);
	::SendMessage(GetDlgItem(IDC_HOTKEY_PREVIOUS_DESKTOP), HKM_SETHOTKEY, wParam, 0);

	MakeWParmHotkeyMessage(iOptions.iHotkeyNextDesktop, wParam);
	::SendMessage(GetDlgItem(IDC_HOTKEY_NEXT_DESKTOP), HKM_SETHOTKEY, wParam, 0);

	MakeWParmHotkeyMessage(iOptions.iHotkeyDesktopUp, wParam);
	::SendMessage(GetDlgItem(IDC_HOTKEY_DESKTOP_UP), HKM_SETHOTKEY, wParam, 0);

	MakeWParmHotkeyMessage(iOptions.iHotkeyDesktopDown, wParam);
	::SendMessage(GetDlgItem(IDC_HOTKEY_DESKTOP_DOWN), HKM_SETHOTKEY, wParam, 0);

	MakeWParmHotkeyMessage(iOptions.iHotkeyVolumeToggleMute, wParam);
	::SendMessage(GetDlgItem(IDC_HOTKEY_VOLUME_TOGGLE_MUTE), HKM_SETHOTKEY, wParam, 0);

	MakeWParmHotkeyMessage(iOptions.iHotkeyVolumeUp, wParam);
	::SendMessage(GetDlgItem(IDC_HOTKEY_VOLUME_UP), HKM_SETHOTKEY, wParam, 0);
	
	MakeWParmHotkeyMessage(iOptions.iHotkeyVolumeDown, wParam);
	::SendMessage(GetDlgItem(IDC_HOTKEY_VOLUME_DOWN), HKM_SETHOTKEY, wParam, 0);
	
	for (int i=0 ; i< iOptions.iNoOfDesktopsHor * iOptions.iNoOfDesktopsVert ; i++)
	{
		char	Text1[MAX_TEXT];
		char	Text2[MAX_TEXT];

		Text1[0] = 0;
		LoadString(GetAppInstance(), IDS_HOTKEYS_DESKTOP_STRING, Text1, MAX_TEXT); 
		Text1[MAX_TEXT - 1] = 0;
 
		sprintf(Text2, Text1, i + 1);

		::SendMessage(GetDlgItem(IDC_COMBO_DESKTOP), CB_ADDSTRING, 0, (long) Text2);
	}
//	::SendMessage(GetDlgItem(IDC_RADIO_INDIVUAL), CB_SETCURSEL, 0, 0);

	CheckDlgButton(IDC_RADIO_INDIVUAL, BST_CHECKED);
	OnDoIndividual();

	PositionDialog(*this);
}


void CHotkeyOptionsDialog::SetDesktopHotkey(int aDesktop)
{
	WORD	wParam;

	THotkey hotkey = iOptions.iHotkeyList[aDesktop];
	MakeWParmHotkeyMessage(hotkey, wParam);
	::SendMessage(GetDlgItem(IDC_HOTKEY_DESKTOP), HKM_SETHOTKEY, wParam, 0);
}


void CHotkeyOptionsDialog::GetDesktopHotkey(int aDesktop)
{
	THotkey		hotkey;
	WORD		wParam;
	
	wParam = ::SendMessage(GetDlgItem(IDC_HOTKEY_DESKTOP), HKM_GETHOTKEY, 0, 0);
	MakeHotkeyFromWParamMessage(hotkey, wParam);
	iOptions.iHotkeyList[aDesktop] = hotkey;
}


bool CHotkeyOptionsDialog::IsOkToExit()
{
	bool okToExit = true;
	if (!iOptions.iAlwaysOnTop && (iOptions.iSysTrayIconOptions.iIconSet == 0) 
			&& (iOptions.iHotkeyShowDeskwin == 0))
	{
		char	text[MAX_TEXT];
		text[0] = 0;

		LoadString(GetAppInstance(), IDS_STRING_APPEARANCE_WARNING, text, MAX_TEXT); 
		okToExit = MessageBox(text,"Hotkey options", MB_ICONSTOP | MB_YESNO) == IDYES;
	}
	return okToExit;
}

void CHotkeyOptionsDialog::CmOk()
{
	WORD wParam = ::SendMessage(GetDlgItem(IDC_HOTKEY_SHOW_DESKWIN), HKM_GETHOTKEY, 0, 0);
	MakeHotkeyFromWParamMessage(iOptions.iHotkeyShowDeskwin, wParam);

	wParam = ::SendMessage(GetDlgItem(IDC_HOTKEY_TOGGLE_DESKTOP), HKM_GETHOTKEY, 0, 0);
	MakeHotkeyFromWParamMessage(iOptions.iHotkeyToggleExplorer, wParam);

	wParam = ::SendMessage(GetDlgItem(IDC_HOTKEY_MINIMISE_ALL), HKM_GETHOTKEY, 0, 0);
	MakeHotkeyFromWParamMessage(iOptions.iHotkeyMinimiseAll, wParam);

	wParam = ::SendMessage(GetDlgItem(IDC_HOTKEY_PREVIOUS_DESKTOP), HKM_GETHOTKEY, 0, 0);
	MakeHotkeyFromWParamMessage(iOptions.iHotkeyPreviousDesktop, wParam);

	wParam = ::SendMessage(GetDlgItem(IDC_HOTKEY_NEXT_DESKTOP), HKM_GETHOTKEY, 0, 0);
	MakeHotkeyFromWParamMessage(iOptions.iHotkeyNextDesktop, wParam);

	wParam = ::SendMessage(GetDlgItem(IDC_HOTKEY_DESKTOP_UP), HKM_GETHOTKEY, 0, 0);
	MakeHotkeyFromWParamMessage(iOptions.iHotkeyDesktopUp, wParam);

	wParam = ::SendMessage(GetDlgItem(IDC_HOTKEY_DESKTOP_DOWN), HKM_GETHOTKEY, 0, 0);
	MakeHotkeyFromWParamMessage(iOptions.iHotkeyDesktopDown, wParam);

	wParam = ::SendMessage(GetDlgItem(IDC_HOTKEY_VOLUME_TOGGLE_MUTE), HKM_GETHOTKEY, 0, 0);
	MakeHotkeyFromWParamMessage(iOptions.iHotkeyVolumeToggleMute, wParam);

	wParam = ::SendMessage(GetDlgItem(IDC_HOTKEY_VOLUME_UP), HKM_GETHOTKEY, 0, 0);
	MakeHotkeyFromWParamMessage(iOptions.iHotkeyVolumeUp, wParam);

	wParam = ::SendMessage(GetDlgItem(IDC_HOTKEY_VOLUME_DOWN), HKM_GETHOTKEY, 0, 0);
	MakeHotkeyFromWParamMessage(iOptions.iHotkeyVolumeDown, wParam);

	if (iCurDesktop == -1)
	{
		GetDesktopHotkey(0);
		THotkey		hotKeyDesk0 = iOptions.iHotkeyList[0];
		for (int i=1 ; i< iOptions.iNoOfDesktopsHor * iOptions.iNoOfDesktopsVert && hotKeyDesk0; i++)
		{
			int hotKeyInc = (hotKeyDesk0 >> 16) + i;
			iOptions.iHotkeyList[i] = (hotKeyDesk0 & 0xffff) | (hotKeyInc << 16);
		}
	}
	else
	{
		GetDesktopHotkey(iCurDesktop);
	}

	if (!IsOkToExit())
		return;

	CDialog::CmOk();
}





/************************************************
 *
 * Color dialog
 *
 ************************************************/

class CPreviewDeskwinWindow : public CDeskwinWindowBase
{
protected:
	virtual void	DoPaint(CDC& dc);
};

void CPreviewDeskwinWindow::DoPaint(CDC& aDc)
{
	TWindowList		emptyWindowList;
	TWindowList		activeDesktopWindowList;

	activeDesktopWindowList.push_back(GetParent());

	RECT	clientRect;
	GetClientRect(&clientRect);

	PaintFrames(aDc, clientRect);

	for (int d=0 ; d<(iNoOfDesktopsHor * iNoOfDesktopsVert) ; d++)
	{
		if (d == 0)
			CDeskwinWindowBase::PaintDesktop(aDc, d, true, activeDesktopWindowList, NULL, NULL, NULL);
		else
			CDeskwinWindowBase::PaintDesktop(aDc, d, false, emptyWindowList, NULL, NULL, NULL);
	}
}




CColorOptionsDialog::CColorOptionsDialog(TDeskwinOptions& aOptions) : CDialog((int) IDD_DIALOG_COLOR), iOptions(aOptions), iBlinkColor(NULL)
{
	AddCmd(SWinCmd(IDC_COMBO_COLOR_OPTIONS, (CWCMDHANDLER) CColorOptionsDialog::OnSelectNewColor, CBN_SELCHANGE));
	AddMsg(SWinMsg(WM_TIMER, (CWMSGHANDLER)CColorOptionsDialog::OnWmTimer));
	AddCmd(SWinCmd(ID_CHANGE_COLOR, (CWCMDHANDLER) CColorOptionsDialog::OnChangeColor, BN_CLICKED));
	AddCmd(SWinCmd(ID_CHANGE_BG_ICON_COLOR, (CWCMDHANDLER) CColorOptionsDialog::OnChangeBgIconColor, BN_CLICKED));
	AddCmd(SWinCmd(ID_CHANGE_FG_ICON_COLOR, (CWCMDHANDLER) CColorOptionsDialog::OnChangeFgIconColor, BN_CLICKED));
	AddCmd(SWinCmd(ID_DEFAULT_COLORS, (CWCMDHANDLER) CColorOptionsDialog::OnDefaultColors, BN_CLICKED));
	AddCmd(SWinCmd(IDC_RADIO_ICON_TRANSPARENT, (CWCMDHANDLER) CColorOptionsDialog::OnToggleTransparentBackground, BN_CLICKED));
	AddCmd(SWinCmd(IDC_RADIO_ICON_CUSTOM_BG_COLOR, (CWCMDHANDLER) CColorOptionsDialog::OnToggleCustomBackground, BN_CLICKED));
}


void CColorOptionsDialog::GetChildRect(TWindow aChild, RECT& aRect)
{
	RECT	childRect;
	::GetWindowRect(aChild, &childRect);

	POINT	point = {childRect.left, childRect.top};
	ScreenToClient(&point); 
	DPRINT((HERE "CColorOptionsDialog::SetupDialog point: left=%i, top=%i", point.x, point.y));

	aRect.left = point.x;
	aRect.top = point.y;
	aRect.right = point.x + (childRect.right - childRect.left);
	aRect.bottom = point.y + (childRect.bottom - childRect.top);
}


void CColorOptionsDialog::SetupColorInfo()
{
	iColorInfo[0].iStringId = IDS_STRING_COLOR_OUT_FRAME;
	iColorInfo[1].iStringId = IDS_STRING_COLOR_IN_FRAME;
	iColorInfo[2].iStringId = IDS_STRING_COLOR_BACKGROUND;
	iColorInfo[3].iStringId = IDS_STRING_COLOR_ACTIVE_DESKTOP;
	iColorInfo[4].iStringId = IDS_STRING_COLOR_WINDOW_FRAME;
	iColorInfo[5].iStringId = IDS_STRING_COLOR_WINDOW;

	iColorInfo[0].iColor = &iDeskwinWindow->iOuterFrameColor;
	iColorInfo[1].iColor = &iDeskwinWindow->iInnerFrameColor;
	iColorInfo[2].iColor = &iDeskwinWindow->iBackgroundColor;
	iColorInfo[3].iColor = &iDeskwinWindow->iActiveDesktopColor;
	iColorInfo[4].iColor = &iDeskwinWindow->iWindowFrameColor;
	iColorInfo[5].iColor = &iDeskwinWindow->iWindowColor;

	for (int i=0 ; i<MAX_COLOR_INFOS ; i++)
	{
		iColorInfo[i].Name[0] = 0;
		LoadString(GetAppInstance(), iColorInfo[i].iStringId, iColorInfo[i].Name, MAX_COLOR_NAME);
		iColorInfo[i].Name[MAX_COLOR_NAME - 1] = 0;
	}
}


void CColorOptionsDialog::SetupPreviewPane()
{
	RECT	previewPaneRect;
	RECT	deskwinRect;
	GetChildRect(GetDlgItem(IDC_STATIC_PREVIEW_PANE), previewPaneRect);

	CopyRect(&deskwinRect, &previewPaneRect);
	DPRINT((HERE "CColorOptionsDialog::SetupDialog child: left=%i, top=%i, width=%i, height=%i", previewPaneRect.left, previewPaneRect.top,  previewPaneRect.right - previewPaneRect.left, previewPaneRect.bottom - previewPaneRect.top));

//	deskwinRect.left += 10;
//	deskwinRect.top += 20;
//	deskwinRect.right -= 10;
//	deskwinRect.bottom -= 10;

	iDeskwinWindow = new CPreviewDeskwinWindow;
	AddNewCW(iDeskwinWindow, this);
	DWORD	style	= WS_VISIBLE | WS_CHILD;
	DWORD	exStyle	= 0;

	int		width = deskwinRect.right - deskwinRect.left;
	int		height = deskwinRect.bottom - deskwinRect.top;

	DPRINT((HERE "CColorOptionsDialog::SetupDialog left=%i, top=%i, width=%i, height=%i", deskwinRect.left, deskwinRect.top, width, height));
	iDeskwinWindow->Construct(deskwinRect.left, deskwinRect.top, style, exStyle, width, height);

	iDeskwinWindow->iOuterFrameColor	= iOptions.iOuterFrameColor;
	iDeskwinWindow->iInnerFrameColor	= iOptions.iInnerFrameColor;
	iDeskwinWindow->iBackgroundColor	= iOptions.iBackgroundColor;
	iDeskwinWindow->iActiveDesktopColor	= iOptions.iActiveDesktopColor;
	iDeskwinWindow->iWindowFrameColor	= iOptions.iWindowFrameColor;
	iDeskwinWindow->iWindowColor		= iOptions.iWindowColor;

	SetSysTrayMiniIcon();
}

void CColorOptionsDialog::OnToggleCustomBackground()
{
	iOptions.iSysTrayIconOptions.iTransparentBackground = false;
	SetSysTrayMiniIcon();
}

void CColorOptionsDialog::OnToggleTransparentBackground()
{
	iOptions.iSysTrayIconOptions.iTransparentBackground = true;
	SetSysTrayMiniIcon();
}

void CColorOptionsDialog::SetSysTrayMiniIcon()
{
	COLORREF	bgColor = iOptions.iSysTrayIconOptions.iBgColor;
	if (iOptions.iSysTrayIconOptions.iTransparentBackground)
		bgColor =  ::GetSysColor(COLOR_3DFACE);
	HICON icon = CSysTrayIconUtil::Icon(1 + 1, 0, iOptions.iSysTrayIconOptions);
	::SendMessage(GetDlgItem(IDC_STATIC_SYSTRAY_ICON_COLOR), STM_SETICON, (WPARAM) icon, 0);

	CheckRadioButton(IDC_RADIO_ICON_TRANSPARENT, IDC_RADIO_ICON_CUSTOM_BG_COLOR, iOptions.iSysTrayIconOptions.iTransparentBackground ?
		IDC_RADIO_ICON_TRANSPARENT : IDC_RADIO_ICON_CUSTOM_BG_COLOR);
	::EnableWindow(GetDlgItem(ID_CHANGE_BG_ICON_COLOR), iOptions.iSysTrayIconOptions.iTransparentBackground ? false : true);
}


void CColorOptionsDialog::OnChangeBgIconColor()
{
	if (CColorOptionsDialog::GetColor(iOptions.iSysTrayIconOptions.iBgColor))
	{
		SetSysTrayMiniIcon();
	}
}


void CColorOptionsDialog::OnChangeFgIconColor()
{
	if (CColorOptionsDialog::GetColor(iOptions.iSysTrayIconOptions.iFgColor))
	{
		SetSysTrayMiniIcon();
	}
}


void CColorOptionsDialog::OnTimer()
{
	BlinkColor();
}

void CColorOptionsDialog::StopBlinkColor()
{
	if (iBlinkColor)
	{
			/*
			 * do a final redraw to paint in the correct color
			 */
		iBlinkCount = 1;
		BlinkColor();
		iBlinkColor = NULL;
	}
}

void CColorOptionsDialog::StartBlinkColor(COLORREF* aColor)
{
	StopBlinkColor();		// stop if needed

	iBlinkCount = 8;
	iBlinkColor = aColor;
	iBlinkColors[0] = (*aColor == ct_White) ? ct_Black : ct_White;
	iBlinkColors[1] = *aColor;
	BlinkColor();
	SetTimer(0, 200);
}

void CColorOptionsDialog::BlinkColor()
{
	*iBlinkColor = iBlinkColors[iBlinkCount % 2];
	iDeskwinWindow->Redraw();

	if (--iBlinkCount <= 0)
	{
		KillTimer(0);
		iBlinkColor = NULL;
	}
}

void CColorOptionsDialog::SetupColorList()
{
	TWindow		colorOptionsCombo = GetDlgItem(IDC_COMBO_COLOR_OPTIONS);

	for (int i=0 ; i<MAX_COLOR_INFOS ; i++)
	{
		::SendMessage(colorOptionsCombo, CB_ADDSTRING, 0, (LPARAM) iColorInfo[i].Name);
	}
}

void CColorOptionsDialog::SetupDialog()
{
		/*
		 * don't change the order
		 */
	SetupPreviewPane();
	SetupColorInfo();
	SetupColorList();

	PositionDialog(*this);
}


bool CColorOptionsDialog::GetColor(COLORREF& aColor)
{
	CHOOSECOLOR				ChooseColor;
	static	COLORREF		UsersChoice[16];

	ChooseColor.lStructSize = sizeof(CHOOSECOLOR);
	ChooseColor.hwndOwner = HWindow; 
    ChooseColor.hInstance = NULL;
	ChooseColor.rgbResult = aColor;
    ChooseColor.lpCustColors = UsersChoice;
	ChooseColor.Flags = CC_ANYCOLOR | CC_RGBINIT; 
    ChooseColor.lCustData = NULL;
	ChooseColor.lpfnHook = NULL;
    ChooseColor.lpTemplateName = NULL;
	
	bool colorSelected  = ::ChooseColor(&ChooseColor) ? true : false;
	if (colorSelected)
	{
		aColor = ChooseColor.rgbResult;
	}

	return colorSelected;
}

void CColorOptionsDialog::ToggleChangeColorButton(bool aEnable)
{
	::EnableWindow(GetDlgItem(ID_CHANGE_COLOR), aEnable);
}

void CColorOptionsDialog::OnSelectNewColor()
{
	TWindow		colorOptionsCombo = GetDlgItem(IDC_COMBO_COLOR_OPTIONS);

	int index = ::SendMessage(colorOptionsCombo, CB_GETCURSEL, 0, 0);
	ASSERT(index>=-1 && index<MAX_COLOR_INFOS);

	ToggleChangeColorButton((index != -1));

	if (index != -1)
		StartBlinkColor(iColorInfo[index].iColor);
}


void CColorOptionsDialog::OnChangeColor()
{
	TWindow		colorOptionsCombo = GetDlgItem(IDC_COMBO_COLOR_OPTIONS);

	int index = ::SendMessage(colorOptionsCombo, CB_GETCURSEL, 0, 0);
	ASSERT(index>=0 && index<MAX_COLOR_INFOS);
	
	StopBlinkColor();

	GetColor(*iColorInfo[index].iColor);
	iDeskwinWindow->Redraw();
}

void CColorOptionsDialog::OnDefaultColors()
{
	iDeskwinWindow->SetDefaultColors();
	iDeskwinWindow->Redraw();

	iOptions.iSysTrayIconOptions.iFgColor = ct_Black;
	iOptions.iSysTrayIconOptions.iBgColor = ::GetSysColor(COLOR_3DFACE);
	iOptions.iSysTrayIconOptions.iTransparentBackground = true;
	SetSysTrayMiniIcon();
}

void CColorOptionsDialog::CmOk()
{
	iOptions.iOuterFrameColor		= iDeskwinWindow->iOuterFrameColor;
	iOptions.iInnerFrameColor		= iDeskwinWindow->iInnerFrameColor;
	iOptions.iBackgroundColor		= iDeskwinWindow->iBackgroundColor;
	iOptions.iActiveDesktopColor	= iDeskwinWindow->iActiveDesktopColor;
	iOptions.iWindowFrameColor		= iDeskwinWindow->iWindowFrameColor;
	iOptions.iWindowColor			= iDeskwinWindow->iWindowColor;

	DPRINT((HERE "CColorOptionsDialog::CmOk, color=0x%x", iDeskwinWindow->iOuterFrameColor));

	CDialog::CmOk();
}


/************************************************
 *
 * Sticky Window dialog
 *
 ************************************************/

CStickyWindowsDialog::CStickyWindowsDialog(TDeskwinOptions& aOptions, CDeskwinEngine& aDeskwinEngine)
	: CDialog((int) IDD_DIALOG_STICKY_WINDOWS), iOptions(aOptions), iDeskwinEngine(aDeskwinEngine)
{
	AddCmd(SWinCmd(ID_ADD_STICKY_WINDOW_RULE, (CWCMDHANDLER) CStickyWindowsDialog::OnAddStickyWindowRule, BN_CLICKED));
	AddCmd(SWinCmd(ID_DELETE_STICKY_WINDOW_RULE, (CWCMDHANDLER) CStickyWindowsDialog::OnDeleteStickyWindowRule, BN_CLICKED));
	AddCmd(SWinCmd(IDC_RULE_UP_BUTTON, (CWCMDHANDLER) CStickyWindowsDialog::OnUpStickyWindowRule, BN_CLICKED));
	AddCmd(SWinCmd(IDC_RULE_DOWN_BUTTON, (CWCMDHANDLER) CStickyWindowsDialog::OnDownStickyWindowRule, BN_CLICKED));
	AddCmd(SWinCmd(ID_WINDOW_DEFAULTS, (CWCMDHANDLER) CStickyWindowsDialog::OnWindowDefaults, BN_CLICKED));
	
	AddMsg(SWinMsg(WM_USER_ITEM_DBLCLICKED , (CWMSGHANDLER)CStickyWindowsDialog::OnWmUserItemDblClicked));

	AddToolTipInfo(SToolTipInfo(IDC_RULE_UP_BUTTON, IDS_HELP_RULE_UP_BUTTON));
}


CStickyWindowsDialog::~CStickyWindowsDialog()
{
	delete iStickyListView;
}


void CStickyWindowsDialog::SetupDialog()
{
	DPRINT((HERE "CStickyWindowsDialog::SetupDialog"));

	iStickyListView = new CListView(GetDlgItem(IDC_STICKY_LISTVIEW));
	AddChild(iStickyListView);

	iStickyListView->InsertTextColumn("Rule name", 0, 100);
	iStickyListView->InsertTextColumn("Show on desktops", 1, 150);
	iStickyListView->InsertTextColumn("Title", 2, 150);
	iStickyListView->InsertTextColumn("Or/And", 3, 75);
	iStickyListView->InsertTextColumn("Class", 4, 150);


	SetData();

	PositionDialog(*this);
}


void CStickyWindowsDialog::SetData()
{
	int			noOfRules = iOptions.iStickyWindowInfo.size();
	char		text[1024];
	string		desktopList;

	iStickyListView->DeleteAllItems();

	for (int i=0; i<noOfRules; i++)
	{
		TStickyWindow& stickyWindow = iOptions.iStickyWindowInfo[i];

		iStickyListView->InsertTextItem((char *)stickyWindow.iRuleName.c_str(), i);

		CDeskwinEngine::FormatStickyDesktopList(stickyWindow, desktopList);
		iStickyListView->SetItemText(i, 1, (TCHAR*)desktopList.c_str());
		
		iStickyListView->SetItemText(i, 2, (TCHAR*) stickyWindow.iTitleName.c_str());

		sprintf(text, (stickyWindow.iStickyWindowMatch == EStickyWindowMatchAnd) ? "And" : "Or");
		iStickyListView->SetItemText(i, 3, text);

		iStickyListView->SetItemText(i, 4, (TCHAR*) stickyWindow.iClassName.c_str());
	}
}

void CStickyWindowsDialog::OnUserItemDblClicked(int aItem)
{
	EditStickyWindowRule(aItem);
}


void CStickyWindowsDialog::AddWindowRule(TWindow aWindow)
{
	TStickyWindow	stickyWindow;

	CDeskwinEngine::GetWindowName(aWindow, stickyWindow.iRuleName);
	CDeskwinEngine::GetWindowName(aWindow, stickyWindow.iTitleName);
	CDeskwinEngine::GetClassName(aWindow, stickyWindow.iClassName);

	iOptions.iStickyWindowInfo.push_back(stickyWindow);
}


void CStickyWindowsDialog::OnWindowDefaults()
{
	iOptions.iStickyWindowInfo.clear();

	AddWindowRule(GetParent());
	AddWindowRule(CDeskwinEngine::GetExplorerWindow());
	AddWindowRule(CDeskwinEngine::GetTaskbarWindow());

	SetData();
}


void CStickyWindowsDialog::EditStickyWindowRule(int aItem)
{
	CEditStickyWindowRuleDialog	dialog(iOptions.iStickyWindowInfo.at(aItem), iDeskwinEngine, IsSystemRule(iOptions.iStickyWindowInfo.at(aItem)));
	AddChild(&dialog);

	if (dialog.Execute() == IDOK)
	{
		iOptions.iStickyWindowInfo[aItem] = dialog.GetRule();
		SetData();
	}
}

void CStickyWindowsDialog::OnAddStickyWindowRule()
{
	TStickyWindow	dummy;
	CEditStickyWindowRuleDialog	dialog(dummy, iDeskwinEngine, false);
	AddChild(&dialog);

	if (dialog.Execute() == IDOK)
	{
		iOptions.iStickyWindowInfo.push_back(dialog.GetRule());
		iStickyListView->DeleteAllItems();
		SetData();
	}
}


bool CStickyWindowsDialog::IsSystemRule(const TStickyWindow& aStickyWindow)
{
	string		string;
	
	CDeskwinEngine::GetWindowName(GetParent(), string);
	if (aStickyWindow.iRuleName == string)
		return true;
	CDeskwinEngine::GetWindowName(CDeskwinEngine::GetExplorerWindow(), string);
	if (aStickyWindow.iRuleName == string)
		return true;
	CDeskwinEngine::GetWindowName(CDeskwinEngine::GetTaskbarWindow(), string);
	if (aStickyWindow.iRuleName == string)
		return true;
	return false;
}


void CStickyWindowsDialog::OnDeleteStickyWindowRule()
{
	TCHAR		text[1024];

	int item = iStickyListView->GetFirstSelectedItem();
	if (item >= 0)
	{
		if (IsSystemRule(iOptions.iStickyWindowInfo[item]))
		{
			sprintf(text, "You are not allowed to delete rule:%s.\nDeskwin will not operate correctly without it.", iOptions.iStickyWindowInfo[item].iRuleName.c_str());
			MessageBox(text, "Delete sticky window rule", MB_ICONSTOP);
		}
		else
		{
			sprintf(text, "Are you sure you want to delete rule:\n%s", iOptions.iStickyWindowInfo[item].iRuleName.c_str());
			if (MessageBox(text, "Delete sticky window rule", MB_ICONQUESTION | MB_YESNO) == IDYES)
			{
				iOptions.iStickyWindowInfo.erase(iOptions.iStickyWindowInfo.begin() + item);
				iStickyListView->DeleteAllItems();
				SetData();
			}
		}
	}
}

void CStickyWindowsDialog::OnUpStickyWindowRule()
{
	OnMoveStickyWindowRule(-1);
}

void CStickyWindowsDialog::OnDownStickyWindowRule()
{
	OnMoveStickyWindowRule(1);
}


void CStickyWindowsDialog::OnMoveStickyWindowRule(int aDelta)
{
	int item = iStickyListView->GetFirstSelectedItem();
	int	count = iStickyListView->GetItemCount();
	if ((item + aDelta >= 0) && (item + aDelta <= count-1))
	{
		swap(iOptions.iStickyWindowInfo[item], iOptions.iStickyWindowInfo[item + aDelta]);
		iStickyListView->DeleteAllItems();
		SetData();
		iStickyListView->SetItemState(item + aDelta, LVIS_FOCUSED | LVIS_SELECTED, LVIS_FOCUSED | LVIS_SELECTED);
	}
	iStickyListView->SetFocus();
}


void CStickyWindowsDialog::CmOk()
{
	CDialog::CmOk();
}



/************************************************
 *
 * Add sticky window rule dialog
 *
 ************************************************/

CEditStickyWindowRuleDialog::CEditStickyWindowRuleDialog(TStickyWindow& aStickyWindowRule, CDeskwinEngine& aDeskwinEngine, bool aPartialEditOnly)
	: CDialog((int) IDD_ADD_STICKY_WINDOW_RULE_DIALOG), iStickyWindowRule(aStickyWindowRule), iDeskwinEngine(aDeskwinEngine), iPartialEditOnly(aPartialEditOnly)
{
	AddCmd(SWinCmd(ID_WINDOW_LIST, (CWCMDHANDLER) CEditStickyWindowRuleDialog::OnWindowList, BN_CLICKED));
	AddCmd(SWinCmd(IDC_DESKTOP_ALL_RADIO, (CWCMDHANDLER) CEditStickyWindowRuleDialog::OnHideWindowList, BN_CLICKED));
	AddCmd(SWinCmd(IDC_DESKTOP_ONLY_RADIO, (CWCMDHANDLER) CEditStickyWindowRuleDialog::OnShowWindowList, BN_CLICKED));
	AddCmd(SWinCmd(IDC_ALL_BUT_DESKTOP_RADIO, (CWCMDHANDLER) CEditStickyWindowRuleDialog::OnShowWindowList, BN_CLICKED));

	iNoOfDesktops = iDeskwinEngine.NoOfDesktops();
}


CEditStickyWindowRuleDialog::~CEditStickyWindowRuleDialog()
{
	delete iWindowListBox;
}


void CEditStickyWindowRuleDialog::SetupDialog()
{
	if (iStickyWindowRule.iRuleName.size() != 0)
	{
		if (iPartialEditOnly)
			SetWindowText("Edit system sticky window rule");
		else
			SetWindowText("Edit sticky window rule");
	}

	iWindowListBox = new CListBox(GetDlgItem(IDC_WINDOW_LIST));
	AddChild(iWindowListBox);

	char		desktop[32];
	for (int i =0 ; i<iNoOfDesktops ; i++)
	{
		sprintf(desktop, "Desktop %d", i +1);
		iWindowListBox->AddString(desktop);
	}
	if (iStickyWindowRule.iStickyWindowDesktop == EStickyWindowDesktopAll)
		//iWindowListBox->ShowWindow(SW_HIDE);
		iWindowListBox->EnableWindow(FALSE);

	SetData();

	if (iPartialEditOnly)
	{
		::EnableWindow(GetDlgItem(IDC_RULE_NAME_EDIT), FALSE);
		::EnableWindow(GetDlgItem(IDC_TITLE_EDIT), FALSE);
		::EnableWindow(GetDlgItem(IDC_CLASS_EDIT), FALSE);
		::EnableWindow(GetDlgItem(IDC_RADIO_TITLE_AND_CLASS), FALSE);
		::EnableWindow(GetDlgItem(IDC_RADIO_TITLE_OR_CLASS), FALSE);
		::EnableWindow(GetDlgItem(ID_WINDOW_LIST), FALSE);
	}

	PositionDialog(*this);
}

void CEditStickyWindowRuleDialog::GetData()
{
	iStickyWindowRule.iStickyWindowMatch = IsDlgButtonChecked(IDC_RADIO_TITLE_OR_CLASS) ? EStickyWindowMatchOr : EStickyWindowMatchAnd;
	iStickyWindowRule.iStickyWindowDesktop = IsDlgButtonChecked(IDC_DESKTOP_ALL_RADIO) ? EStickyWindowDesktopAll : 
		IsDlgButtonChecked(IDC_DESKTOP_ONLY_RADIO) ? EStickyWindowDesktopInclude : EStickyWindowDesktopExclude;

	char	text[1024];

	::GetWindowText(GetDlgItem(IDC_RULE_NAME_EDIT), text, 1024);
	text[1023] = 0;
	iStickyWindowRule.iRuleName.assign(text);

	::GetWindowText(GetDlgItem(IDC_TITLE_EDIT), text, 1024);
	text[1023] = 0;
	iStickyWindowRule.iTitleName.assign(text);

	::GetWindowText(GetDlgItem(IDC_CLASS_EDIT), text, 1024);
	text[1023] = 0;
	iStickyWindowRule.iClassName.assign(text);

	iStickyWindowRule.iDesktopList.clear();
	for (int i=0 ; i<iNoOfDesktops ; i++)
	{
		if (iWindowListBox->GetSel(i))
		{
			iStickyWindowRule.iDesktopList.push_back(i);
		}
	}
}

void CEditStickyWindowRuleDialog::SetData()
{
	CheckDlgButton((iStickyWindowRule.iStickyWindowMatch == EStickyWindowMatchOr) ? IDC_RADIO_TITLE_OR_CLASS : IDC_RADIO_TITLE_AND_CLASS, BST_CHECKED);
	CheckDlgButton((iStickyWindowRule.iStickyWindowDesktop == EStickyWindowDesktopAll) ? IDC_DESKTOP_ALL_RADIO :
		(iStickyWindowRule.iStickyWindowDesktop == EStickyWindowDesktopInclude) ? IDC_DESKTOP_ONLY_RADIO : IDC_ALL_BUT_DESKTOP_RADIO, BST_CHECKED);
	
	::SetWindowText(GetDlgItem(IDC_RULE_NAME_EDIT), iStickyWindowRule.iRuleName.c_str());
	::SetWindowText(GetDlgItem(IDC_TITLE_EDIT), iStickyWindowRule.iTitleName.c_str());
	::SetWindowText(GetDlgItem(IDC_CLASS_EDIT), iStickyWindowRule.iClassName.c_str());

	for (TDesktopList::iterator i=iStickyWindowRule.iDesktopList.begin() ; i!=iStickyWindowRule.iDesktopList.end(); i++)
	{
		int		desktop = *i;
		iWindowListBox->SetSel(desktop, TRUE);
	}
}

void CEditStickyWindowRuleDialog::ImportWindow(TWindow aWindow)
{
	CDeskwinEngine::GetWindowName(aWindow, iStickyWindowRule.iRuleName);
	CDeskwinEngine::GetWindowName(aWindow, iStickyWindowRule.iTitleName);
	CDeskwinEngine::GetClassName(aWindow, iStickyWindowRule.iClassName);
	
	SetData();
}




void CEditStickyWindowRuleDialog::OnWindowList()
{
	CWindowListDialog dialog(iDeskwinEngine);
	AddChild(&dialog);

	if (dialog.Execute() == IDOK)
	{
		ImportWindow(dialog.GetSelectedWindow());
	}
}


void CEditStickyWindowRuleDialog::OnHideWindowList()
{
//	iWindowListBox->ShowWindow(SW_HIDE);
	iWindowListBox->EnableWindow(FALSE);
}

void CEditStickyWindowRuleDialog::OnShowWindowList()
{
//	iWindowListBox->ShowWindow(SW_SHOW);
	iWindowListBox->EnableWindow(TRUE);
}

void CEditStickyWindowRuleDialog::CmOk()
{
	GetData();

	if (iStickyWindowRule.iRuleName.length() == 0)
	{
		MessageBox("Please specify Rule name", "Add sticky window rule", MB_ICONSTOP);
		::SetFocus(GetDlgItem(IDC_RULE_NAME_EDIT));
	}
	else if ((iStickyWindowRule.iTitleName.length() == 0) && (iStickyWindowRule.iClassName.length() == 0))
	{
		MessageBox("Please specify a match string for either the Title or the Class", "Add sticky window rule", MB_ICONSTOP);
		
		::SetFocus((iStickyWindowRule.iTitleName.length() == 0) ? 
					GetDlgItem(IDC_TITLE_EDIT) : GetDlgItem(IDC_CLASS_EDIT));
	}
	else
	{
		CDialog::CmOk();
	}
}



/************************************************
 *
 * Window list dialog
 *
 ************************************************/

CWindowListDialog::CWindowListDialog(CDeskwinEngine& aDeskwinEngine)
	: CDialog((int) IDD_WINDOW_LIST_DIALOG), iDeskwinEngine(aDeskwinEngine), iFullList(FALSE), iImageListSmall(NULL)
{
	AddCmd(SWinCmd(IDC_CHECK_FULL_LIST, (CWCMDHANDLER) CWindowListDialog::OnToggleFullList, BN_CLICKED));
	AddCmd(SWinCmd(ID_FIND_WINDOW, (CWCMDHANDLER) CWindowListDialog::OnFindWindow, BN_CLICKED));
	AddMsg(SWinMsg(WM_USER_ITEM_DBLCLICKED , (CWMSGHANDLER)CWindowListDialog::OnWmUserItemDblClicked));
}


CWindowListDialog::~CWindowListDialog()
{
	delete iWindowListView;
}


HICON GetAppIcon1(TWindow aWindow, const char* aClassName)
{
	HICON icon = (HICON) ::GetClassLong(aWindow, GCL_HICON);
	if (!icon)
	{
		icon = (HICON) ::GetClassLong(aWindow, GCL_HICONSM);
	}
	return icon;
}

HICON GetAppIcon(TWindow aWindow, const char* aClassName, BOOL aCheckParent)
{
	HICON		icon;
	
	do
	{
		icon = GetAppIcon1(aWindow, aClassName);
		if (icon)
			break;
	} while(aCheckParent && (aWindow = ::GetParent(aWindow)));

	return icon;
}

TWindow CWindowListDialog::GetSelectedWindow()
{
	return iSelectedWindow;
}


void CWindowListDialog::OnUserItemDblClicked(int aItem)
{
	iSelectedWindow = (TWindow) iWindowListView->GetItemData(aItem);
	Close(IDOK);
}

void CWindowListDialog::SetupDialog()
{
	iWindowListView = new CListView(GetDlgItem(IDC_WINDOWLIST));
	AddChild(iWindowListView);

	iWindowListView->InsertTextColumn("Title", 0, 250);
	iWindowListView->InsertTextColumn("Class", 1, 250);
	iWindowListView->InsertTextColumn("Sticky Rule", 2, 200);
	iWindowListView->InsertTextColumn("Desktop", 3, 100);

	SetData();

	PositionDialog(*this);
}


void CWindowListDialog::SetupImageList()
{
	HICON		iconItem;

	if (iImageListSmall)
	{
		ImageList_Destroy(iImageListSmall);
		iImageListSmall = NULL;
	}

	iImageListSmall = ImageList_Create(GetSystemMetrics(SM_CXSMICON), 
		GetSystemMetrics(SM_CYSMICON), TRUE, 1, 1); 

	// Add an icon to each image list. 
	iconItem = LoadIcon(NULL, IDI_APPLICATION);
	ImageList_AddIcon(iImageListSmall, iconItem); 
	DeleteObject(iconItem);

	iconItem = LoadIcon(NULL, IDI_WINLOGO);
	ImageList_AddIcon(iImageListSmall, iconItem); 
	DeleteObject(iconItem); 

	iconItem = ::LoadIcon(GetAppInstance(), MAKEINTRESOURCE(IDI_DESKTOP_ICON));
	ImageList_AddIcon(iImageListSmall, iconItem); 
	DeleteObject(iconItem); 
	
	iWindowListView->SetImageList(iImageListSmall, LVSIL_SMALL); 
}

void CWindowListDialog::OnToggleFullList()
{
	iFullList = (IsDlgButtonChecked(IDC_CHECK_FULL_LIST) == BST_CHECKED);
	iWindowListView->DeleteAllItems();
	SetData();
}

void CWindowListDialog::OnFindWindow()
{
	int		item;

	item = iWindowListView->GetFirstSelectedItem();
	if (item >= 0)
	{
		TWindow window = (TWindow) iWindowListView->GetItemData(item);

		BOOL isIconic = ::IsIconic(window);
		BOOL isHidden = ::IsWindowVisible(window) ? FALSE : TRUE;

		if (isHidden)
			::ShowWindow(window, SW_SHOW);
		if (isIconic)
			::ShowWindow(window, SW_RESTORE);

		::SetForegroundWindow(window);
		for (int i=0; i<6 ; i++)
		{
			::Sleep(250);
			::ShowWindow(window, SW_HIDE);
			::Sleep(250);
			::ShowWindow(window, SW_SHOW);
		}

		if (isIconic)
			::ShowWindow(window, SW_MINIMIZE);
		if (isHidden)
			::ShowWindow(window, SW_HIDE);
	}
	else
	{
		MessageBox("Select a window first", "Find selected window", MB_ICONEXCLAMATION | MB_OK);
	}
}


void CWindowListDialog::SetData()
{
	int			noOfDesktops = iDeskwinEngine.NoOfDesktops();
	int			count = 0;
	TWindow		window = NULL;
	string		windowTitle;
	string		windowClass;
	char		desktopNum[32];
	HICON		iconItem;

	SetupImageList();

	int iconIndex = 2;
	TWindow		explorerWindow = iDeskwinEngine.GetExplorerWindow();
	TWindow		taskbarWindow = iDeskwinEngine.GetTaskbarWindow();

	TWindowList		windowList;
	iDeskwinEngine.GetAllWindows(windowList);

	for (TWindowList::iterator i=windowList.begin() ; i!=windowList.end(); i++)
	{
		window = *i;
		CDeskwinEngine::GetWindowName(window, windowTitle);
		CDeskwinEngine::GetClassName(window, windowClass);

		bool	ruleMatch;
		bool	isSticky;
		int		ruleNo;
		
		iDeskwinEngine.IsStickyWindow1(-1, window, ruleMatch, isSticky, ruleNo);

		if ((window == explorerWindow) || (window == taskbarWindow))
		{
			iWindowListView->InsertTextItem((char *) windowTitle.c_str(), count);
			iWindowListView->SetItemText(count, 1, (TCHAR*) windowClass.c_str());
			iWindowListView->SetImage(count, (window == explorerWindow) ? 2 : 1);
			iWindowListView->SetItemData(count, (DWORD)window);

			if (ruleMatch)
			{
				string	desktopList;
				iWindowListView->SetItemText(count, 2, (TCHAR *) iDeskwinEngine.StickyWindowInfo().at(ruleNo).iRuleName.c_str());
				CDeskwinEngine::FormatStickyDesktopList(iDeskwinEngine.StickyWindowInfo()[ruleNo], desktopList);
				iWindowListView->SetItemText(count, 3, (TCHAR *) desktopList.c_str());
			}
			else
			{
				string	desktopList;
				iWindowListView->SetItemText(count, 2, "<none>");
				iWindowListView->SetItemText(count, 3, "");
			}

			count++;
		}
		else
		{
			iconItem = GetAppIcon(window, windowClass.c_str(), iFullList);
			if (iconItem || iFullList)
			{
				iWindowListView->InsertTextItem((char *) windowTitle.c_str(), count);
				iWindowListView->SetItemText(count, 1, (TCHAR*) windowClass.c_str());
				iWindowListView->SetItemText(count, 2, desktopNum);

				if (ruleMatch)
				{
					string	desktopList;
					iWindowListView->SetItemText(count, 2, (TCHAR *) iDeskwinEngine.StickyWindowInfo()[ruleNo].iRuleName.c_str());
					CDeskwinEngine::FormatStickyDesktopList(iDeskwinEngine.StickyWindowInfo()[ruleNo], desktopList);
					iWindowListView->SetItemText(count, 3, (TCHAR *) desktopList.c_str());
				}
				else
				{
					string	desktopList;
					iWindowListView->SetItemText(count, 2, "<none>");
					iWindowListView->SetItemText(count, 3, "");
				}


				if (iconItem)
				{
					ImageList_AddIcon(iImageListSmall, iconItem);
					DeleteObject(iconItem);
					iconIndex++;
				}
				iWindowListView->SetImage(count, iconItem ? iconIndex : 0);
				iWindowListView->SetItemData(count, (DWORD)window);
				count++;
			}
		}
	}
}


#if 0
void CWindowListDialog::SetData()
{
	int			noOfDesktops = iDeskwinEngine.NoOfDesktops();
	int			count = 0;
	TWindow		window = NULL;
	string		windowTitle;
	string		windowClass;
	char		desktopNum[32];
	HICON		iconItem;

	SetupImageList();

	int iconIndex = 2;
	TWindow		explorerWindow = iDeskwinEngine.GetExplorerWindow();
	TWindow		taskbarWindow = iDeskwinEngine.GetTaskbarWindow();

	for (int d=0; d<noOfDesktops; d++)
	{
		sprintf(desktopNum, "%d", d + 1);

		TWindowList		windowList;
		iDeskwinEngine.GetWindowList(d, windowList);

		for (TWindowList::iterator i=windowList.begin() ; i!=windowList.end(); i++)
		{
			window = *i;
			CDeskwinEngine::GetWindowName(window, windowTitle);
			CDeskwinEngine::GetClassName(window, windowClass);

			if ((window == explorerWindow) || (window == taskbarWindow))
			{
				iWindowListView->InsertTextItem((char *) windowTitle.c_str(), count);
				iWindowListView->SetItemText(count, 1, (TCHAR*) windowClass.c_str());
				iWindowListView->SetItemText(count, 2, desktopNum);
				iWindowListView->SetImage(count, (window == explorerWindow) ? 2 : 1);
				iWindowListView->SetItemData(count, (DWORD)window);
				count++;
			}
			else
			{
				iconItem = GetAppIcon(window, windowClass.c_str(), iFullList);
				if (iconItem || iFullList)
				{
					iWindowListView->InsertTextItem((char *) windowTitle.c_str(), count);
					iWindowListView->SetItemText(count, 1, (TCHAR*) windowClass.c_str());
					iWindowListView->SetItemText(count, 2, desktopNum);

					if (iconItem)
					{
						ImageList_AddIcon(iImageListSmall, iconItem);
						DeleteObject(iconItem);
						iconIndex++;
					}
					iWindowListView->SetImage(count, iconItem ? iconIndex : 0);
					iWindowListView->SetItemData(count, (DWORD)window);
					count++;
				}
			}
		}
	}
}
#endif

void CWindowListDialog::CmOk()
{
	CDialog::CmOk();
}




/************************************************
 *
 * About dialog
 *
 ************************************************/

CAboutDialog::CAboutDialog() : CDialog((int) IDD_DIALOG_ABOUT), iHyperHomepage(NULL), iHyperMailto(NULL)
{
	
}

CAboutDialog::~CAboutDialog()
{
	if (iHyperHomepage)
		delete iHyperHomepage;
	if (iHyperMailto)
		delete iHyperMailto;
}


void CAboutDialog::SetupDialog()
{
	char	aboutText[MAX_ABOUT_TEXT];
	char	aboutText1[MAX_ABOUT_TEXT];
	char	version[32];

	aboutText[0] = 0;
	LoadString(GetAppInstance(), IDS_STRING_ABOUT, aboutText, MAX_ABOUT_TEXT);
	aboutText[MAX_ABOUT_TEXT - 1] = 0;

	if (PATCH_VERSION == 0)
		sprintf(version, "%i.%i", MAJOR_VERSION, MINOR_VERSION);
	else
		sprintf(version, "%i.%i.%i", MAJOR_VERSION, MINOR_VERSION, PATCH_VERSION);
#ifdef PRE_RELEASE_VERSION
	strcat(version, PRE_RELEASE_VERSION);
#endif
	sprintf(aboutText1, aboutText, version);
	
	HWND aboutWindow = GetDlgItem(IDC_STATIC_ABOUT);

	::SendMessage(aboutWindow, WM_SETTEXT, 0, (LPARAM) aboutText1);

	CHyperLinkText* iHyperHomepage = new CHyperLinkText(GetDlgItem(IDC_HYPER_HOMEPAGE));
	AddChild(iHyperHomepage);

	CHyperLinkText* iHyperMailto = new CHyperLinkText(GetDlgItem(IDC_HYPER_MAILTO));
	AddChild(iHyperMailto);

	PositionDialog(*this);
}


/************************************************
 *
 * Appearance dialog
 *
 ************************************************/

CAppearanceOptionsDialog::CAppearanceOptionsDialog(TDeskwinOptions& aOptions)
	:  CDialog((int) IDD_DIALOG_APPEARANCE), iOptions(aOptions), iAnimateIconSet(-1)
{
}

CAppearanceOptionsDialog::~CAppearanceOptionsDialog()
{
	KillTimer(0);

	DestroyIcon(iIconButtons[0]->SetIcon(NULL));
	DestroyIcon(iIconButtons[1]->SetIcon(NULL));
	DestroyIcon(iIconButtons[2]->SetIcon(NULL));
	DestroyIcon(iIconButtons[3]->SetIcon(NULL));
}

void CAppearanceOptionsDialog::SetupDialog(void)
{
	AddMsg(SWinMsg(WM_TIMER, (CWMSGHANDLER)CAppearanceOptionsDialog::OnWmTimer));

	AddCmd(SWinCmd(IDC_CHECK_MINI_WINDOW_ALWAYS_ON_TOP, (CWCMDHANDLER) CAppearanceOptionsDialog::OnToggleAlwaysOnTop, BN_CLICKED));

	AddCmd(SWinCmd(IDC_RADIO_ICON_NONE, (CWCMDHANDLER) CAppearanceOptionsDialog::OnSelectIconNone, BN_CLICKED));
	AddCmd(SWinCmd(IDC_RADIO_ICON1, (CWCMDHANDLER) CAppearanceOptionsDialog::OnSelectIcon1, BN_CLICKED));
	AddCmd(SWinCmd(IDC_RADIO_ICON2, (CWCMDHANDLER) CAppearanceOptionsDialog::OnSelectIcon2, BN_CLICKED));
	AddCmd(SWinCmd(IDC_RADIO_ICON3, (CWCMDHANDLER) CAppearanceOptionsDialog::OnSelectIcon3, BN_CLICKED));
	AddCmd(SWinCmd(IDC_RADIO_ICON4, (CWCMDHANDLER) CAppearanceOptionsDialog::OnSelectIcon4, BN_CLICKED));

	CheckDlgButton(IDC_CHECK_MINI_WINDOW_ALWAYS_ON_TOP, iOptions.iAlwaysOnTop ? BST_CHECKED : BST_UNCHECKED);

	// always make it transparent otherwise it will flicker
	iOptions.iSysTrayIconOptions.iTransparentBackground = true;

	HICON icon;
	for (int i=0 ; i<4 ; i++)
	{
		iIconButtons[i] = new CButton(GetDlgItem(IDC_RADIO_ICON1 + i));
		AddChild(iIconButtons[i]);
		icon = CSysTrayIconUtil::Icon(i + 1, 0, iOptions.iSysTrayIconOptions);
		iIconButtons[i]->SetIcon(icon);
	}

	CheckRadioButton(IDC_RADIO_ICON_NONE, IDC_RADIO_ICON4, IDC_RADIO_ICON_NONE + iOptions.iSysTrayIconOptions.iIconSet);
	OnSelectIcon(iOptions.iSysTrayIconOptions.iIconSet);

	PositionDialog(*this);
}
	
void CAppearanceOptionsDialog::OnToggleAlwaysOnTop(void)
{
	bool isChecked = (IsDlgButtonChecked(IDC_CHECK_MINI_WINDOW_ALWAYS_ON_TOP) == BST_CHECKED);
	iOptions.iAlwaysOnTop = isChecked;
}


void CAppearanceOptionsDialog::OnSelectIcon(int aIcon)
{
	StopAnimating();
	iOptions.iSysTrayIconOptions.iIconSet = aIcon;
	if (iOptions.iSysTrayIconOptions.iIconSet > 1)
		StartAnimating();
}

void CAppearanceOptionsDialog::StopAnimating()
{
	if (iAnimateIconSet != -1)
	{
		KillTimer(0);
		HICON icon = CSysTrayIconUtil::Icon(iAnimateIconSet, 0, iOptions.iSysTrayIconOptions);
		icon = iIconButtons[iAnimateIconSet - 1]->SetIcon(icon);
		DestroyIcon(icon);
		iAnimateIconSet = -1;
	}
}

void CAppearanceOptionsDialog::Animate()
{
	if (iAnimateDesktop == 9)
	{
		StopAnimating();
	}
	else
	{
		HICON icon = CSysTrayIconUtil::Icon(iAnimateIconSet, iAnimateDesktop++, iOptions.iSysTrayIconOptions);
		icon = iIconButtons[iAnimateIconSet - 1]->SetIcon(icon);
		DestroyIcon(icon);
		SetTimer(0, 125);
	}
}

void CAppearanceOptionsDialog::StartAnimating()
{
	SetTimer(0, 125);
	iAnimateIconSet = iOptions.iSysTrayIconOptions.iIconSet;
	iAnimateDesktop = 0;
}

bool CAppearanceOptionsDialog::IsOkToExit()
{
	bool okToExit = true;
	if (!iOptions.iAlwaysOnTop && (iOptions.iSysTrayIconOptions.iIconSet == 0) 
			&& (iOptions.iHotkeyShowDeskwin == 0))
	{

		char	text[MAX_TEXT];

		text[0] = 0;
		LoadString(GetAppInstance(), IDS_STRING_APPEARANCE_WARNING, text, MAX_TEXT); 

		okToExit = MessageBox(text,"Appearance options", MB_ICONSTOP | MB_YESNO) == IDYES;
	}
	return okToExit;
}


void CAppearanceOptionsDialog::CmOk()
{
	if (!IsOkToExit())
		return;

	CDialog::CmOk();
}


/*******************************************************************
 *
 * $Log: /Deskwin/Src/COptionDialogs.cpp $
 * 
 * 4     10/01/00 9:17 Pese
 * 
 * 3     5/01/00 20:26 Pese
 * 
 * 2     23/12/99 10:16 Pese
 *
 *******************************************************************/
