/*******************************************************************
 *
 *	File:		COptionsDialog.h
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 1999, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#ifndef __C_OPTION_DIALOGS_H
#define __C_OPTION_DIALOGS_H

#include "TDeskwinOptions.h"

class CDeskwinEngine;

class CPreviewDeskwinWindow;

class CDockOptionsDialog : public CDialog
{
public:
	CDockOptionsDialog(TDeskwinOptions& aOptions);

protected:
	virtual void SetupDialog(void);
	virtual long OnVScroll(WPARAM wParam, LPARAM lParam);

	void OnToggleEnableDocking();
	void SetDockingPercentage();

	TDeskwinOptions&	iOptions;
};


class CAutoHideOptionsDialog : public CDialog
{
public:
	CAutoHideOptionsDialog(TDeskwinOptions& aOptions);

protected:
	long OnVScroll(WPARAM wParam, LPARAM lParam);
	void OnToggleEnableAutoHide();
	void SetupDialog();

	void CmOk();

	void SetInactivityTimeout();
	void SetVisiblePercentage();

	TDeskwinOptions& iOptions;
};

class CHotkeyOptionsDialog : public CDialog
{
public:
	CHotkeyOptionsDialog(TDeskwinOptions& aOptions);

protected:
	virtual void SetupDialog();
	virtual void CmOk();
	void OnSelectNewDesktop();

	void GetDesktopHotkey(int aDesktop);
	void SetDesktopHotkey(int aDesktop);

	void OnDoIndividual();
	void OnDoGroup();


	bool IsOkToExit();

	TDeskwinOptions&		iOptions;
	int						iCurDesktop;
};



#define MAX_COLOR_INFOS		6
#define MAX_COLOR_NAME		256

typedef struct _TCOLOR_INFO
{
	char			Name[MAX_COLOR_NAME];
	COLORREF*		iColor;
	int				iStringId;
} TCOLOR_INFO;

class CColorOptionsDialog : public CDialog
{
public:
	CColorOptionsDialog(TDeskwinOptions& aOptions);

protected:
	virtual void SetupDialog();
	virtual void CmOk();

	void SetupPreviewPane();
	void SetupColorInfo();
	void SetupColorList();
	void ToggleChangeColorButton(bool aEnable);
	void SetSysTrayMiniIcon();

	void StartBlinkColor(COLORREF* aColor);
	void BlinkColor();
	void StopBlinkColor();

	void OnSelectNewColor();
	void OnChangeColor();
	void OnChangeBgIconColor();
	void OnChangeFgIconColor();
	void OnDefaultColors();
	void OnToggleCustomBackground();
	void OnToggleTransparentBackground();

	long OnWmTimer(WPARAM aWParam, LPARAM aLParam){OnTimer(); return 1;}
	void OnTimer();
	
	HBRUSH OnCtlColorBtn(HDC aDc, HWND aButton);
	bool CColorOptionsDialog::GetColor(COLORREF& aColor);
	void GetChildRect(TWindow aChild, RECT& aRect);

	CPreviewDeskwinWindow*		iDeskwinWindow;
	TDeskwinOptions&			iOptions;
	COLORREF*					iBlinkColor;
	COLORREF					iBlinkColors[2];
	int							iBlinkCount;

	TCOLOR_INFO					iColorInfo[MAX_COLOR_INFOS];
};


class CStickyWindowsDialog : public CDialog
{
public:
	CStickyWindowsDialog(TDeskwinOptions& aOptions, CDeskwinEngine& aDeskwinEngine);
	~CStickyWindowsDialog();

protected:
	long OnWmUserItemDblClicked(WPARAM aWParam, LPARAM aLParam){OnUserItemDblClicked(aWParam); return 1;}
	void OnUserItemDblClicked(int aItem);

	virtual void SetupDialog();
	virtual void CmOk();
	void AddWindowRule(TWindow aWindow);
	void OnWindowDefaults();
	void OnAddStickyWindowRule();
	void OnDeleteStickyWindowRule();
	void OnDownStickyWindowRule();
	void OnUpStickyWindowRule();
	void EditStickyWindowRule(int aItem);

	void OnMoveStickyWindowRule(int aDelta);
	void SetData();

	bool IsSystemRule(const TStickyWindow& aStickyWindow);


	CListView*				iStickyListView;
	CDeskwinEngine&			iDeskwinEngine;

	TDeskwinOptions&		iOptions;
};


class CEditStickyWindowRuleDialog : public CDialog
{
public:
	CEditStickyWindowRuleDialog(TStickyWindow& aStickyWindowRule, CDeskwinEngine& aDeskwinEngine, bool aPartialEditOnly);
	~CEditStickyWindowRuleDialog();

	const TStickyWindow& GetRule(){return iStickyWindowRule;}

protected:
	virtual void SetupDialog();
	virtual void CmOk();
	void OnWindowList();
	void OnHideWindowList();
	void OnShowWindowList();

	void ImportWindow(TWindow aWindow);

	void	GetData();
	void	SetData();

	CDeskwinEngine& iDeskwinEngine;
	TStickyWindow	iStickyWindowRule;
	int				iNoOfDesktops;
	CListBox*		iWindowListBox;
	bool			iPartialEditOnly;
};


class CWindowListDialog : public CDialog
{
public:
	CWindowListDialog(CDeskwinEngine& aDeskwinEngine);
	~CWindowListDialog();

	TWindow			GetSelectedWindow();

protected:
	virtual void	SetupDialog();
	virtual void	CmOk();
	long OnWmUserItemDblClicked(WPARAM aWParam, LPARAM aLParam){OnUserItemDblClicked(aWParam); return 1;}
	void OnUserItemDblClicked(int aItem);
	void			SetupImageList();
	void			SetData();
	void			OnToggleFullList();
	void			OnFindWindow();

	CListView*				iWindowListView;
	CDeskwinEngine&			iDeskwinEngine;
	HIMAGELIST				iImageListSmall;
	bool					iFullList;
	TWindow					iSelectedWindow;
};



class CHyperLinkText;

class CAboutDialog : public CDialog
{
public:
	CAboutDialog();
	~CAboutDialog();

protected:
	void SetupDialog();
	CHyperLinkText*		iHyperHomepage;
	CHyperLinkText*		iHyperMailto;
};


class CAppearanceOptionsDialog : public CDialog
{
public:
	CAppearanceOptionsDialog(TDeskwinOptions& aOptions);
	~CAppearanceOptionsDialog();

protected:
	virtual void SetupDialog(void);
	
	void OnToggleAlwaysOnTop(void);

	void OnSelectIconNone(){OnSelectIcon(0);}
	void OnSelectIcon1(){OnSelectIcon(1);}
	void OnSelectIcon2(){OnSelectIcon(2);}
	void OnSelectIcon3(){OnSelectIcon(3);}
	void OnSelectIcon4(){OnSelectIcon(4);}
	void OnSelectIcon(int aIcon);

	void Animate();
	void StartAnimating();
	void StopAnimating();

	bool IsOkToExit();
	void CmOk();

	int		iAnimateIconSet;
	int		iAnimateDesktop;

	long OnWmTimer(WPARAM aWParam, LPARAM aLParam){Animate(); return 1;}

	TDeskwinOptions&	iOptions;
	CButton*			iIconButtons[3];
};


/*******************************************************************
 *
 * $Log: /Deskwin/Src/COptionDialogs.h $
 * 
 * 4     10/01/00 9:17 Pese
 * 
 * 3     5/01/00 20:26 Pese
 * 
 * 2     23/12/99 10:16 Pese
 *
 *******************************************************************/


#endif			/* __C_OPTION_DIALOGS_H */