/*******************************************************************
 *
 *	File:		CSmoothWindowMover.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 1999, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#include "Deskwin.h"
#include "CSmoothWindowMover.h"

/*
 * SmootWindowMover will move a window (HWND) to a new position. The client
 * specifies the window, the delta X and Y coordinates, total animation time 
 * and a timeinterval for individual movements.
 */

void CSmoothWindowMover::AnimationTimer(HWND /*aWindow*/, UINT /*aMsg*/, UINT aEventId, DWORD /*aTime*/)
{
	CSmoothWindowMover*	aWindowMover = (CSmoothWindowMover*) aEventId;

	aWindowMover->DoAnimationStep();
}


CSmoothWindowMover::CSmoothWindowMover() : iAnimationStep(-1)
{
}

CSmoothWindowMover::~CSmoothWindowMover()
{
}

void CSmoothWindowMover::StartSmoothMoving(CWindow* aWindow, int aDeltaX, int aDeltaY, int aAnimationTime, int aAnimationStepTime, MSmoothWindowMoverObserver* aObserver)
{
	RECT	rect;
	
	iWindow = aWindow;
	iObserver = aObserver;
	iAnimationTime = aAnimationTime;
	iAnimationStepTime = aAnimationStepTime;

	iNoOfAnimationSteps = iAnimationTime / iAnimationStepTime;
	if (iNoOfAnimationSteps == 0)
		iNoOfAnimationSteps = 1;
	iAnimationStepMove.x = aDeltaX / iNoOfAnimationSteps;
	iAnimationStepMove.y = aDeltaY / iNoOfAnimationSteps;

	aWindow->GetWindowRect(&rect);
	iWindowSize.x = rect.right - rect.left;
	iWindowSize.y = rect.bottom - rect.top;

//	DPRINT((HERE "CDeskwinWindow::StartSmoothMoving, step.x=%i, step.y=%i, nrsteps=%i", iAnimationStepMove.x, iAnimationStepMove.y, iNoOfAnimationSteps));

	iAnimationStep = 0;
	::SetTimer(iWindow->HWindow, (UINT) this, iAnimationStepTime, (TIMERPROC) &CSmoothWindowMover::AnimationTimer);
	DoAnimationStep();
}


void CSmoothWindowMover::StopSmoothMoving()
{
	if (iWindow)
	{
		::KillTimer(iWindow->HWindow, (UINT) this);
		iAnimationStep = -1;
	}
}


void CSmoothWindowMover::DoAnimationStep()
{
	RECT	rect;
	
//	DPRINT((HERE "CDeskwinWindow::DoAnimationStep, step=%i, nrsteps=%i", iAnimationStep, iNoOfAnimationSteps));

	iWindow->GetWindowRect(&rect);
	iWindow->MoveWindow(		rect.left + iAnimationStepMove.x, 
								rect.top +  iAnimationStepMove.y,
								iWindowSize.x,
								iWindowSize.y
								);
	if (++iAnimationStep >= iNoOfAnimationSteps)
	{
		StopSmoothMoving();		
		iObserver->DoneSmoothWindowMoving(iWindow);		
	}
}	


/*******************************************************************
 *
 * $Log: /Deskwin/Src/CSmoothWindowMover.cpp $
 * 
 * 4     10/01/00 9:17 Pese
 * 
 * 3     5/01/00 20:26 Pese
 * 
 * 2     23/12/99 10:16 Pese
 *
 *******************************************************************/
