/*******************************************************************
 *
 *	File:		CSysTrayIconController.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2002, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#include "Deskwin.h"
#include "resource.h"
#include "CSysTrayIconController.h"
#include "CSysTrayIconUtil.h"

CSysTrayIconController::CSysTrayIconController(HWND aSysTrayEventHandler, UINT aSysTrayEvent) 
	: iSysTrayEventHandler(aSysTrayEventHandler), iSysTrayEvent(aSysTrayEvent), iCurrentDesktop(0), iIconAdded(FALSE)
{
}


CSysTrayIconController::~CSysTrayIconController()
{
	RemoveIcon();
}

void CSysTrayIconController::Construct()
{
	GetApplication()->AddWindow(this);
	GetApplication()->RegisterClass("CDeskwinSysTrayIconController", NULL, NULL, -1, 0, CS_HREDRAW | CS_VREDRAW | CS_OWNDC);
	bool result = Create("CDeskwinSysTrayIconController", 0, 0, 0, 0, 0, WS_POPUP, NULL, WS_EX_TOPMOST | WS_EX_TOOLWINDOW);
	iNotifyIconData.cbSize = sizeof(NOTIFYICONDATAA); 
	iNotifyIconData.hWnd = iSysTrayEventHandler; 
	iNotifyIconData.uID = 0; 
	iNotifyIconData.uFlags = NIF_ICON | NIF_TIP | NIF_MESSAGE; 
	iNotifyIconData.uCallbackMessage = iSysTrayEvent;
}

void CSysTrayIconController::OptionsChanged(const TSysTrayIconOptions& aSysTrayIconOptions)
{
	iSysTrayIconOptions = aSysTrayIconOptions;
	if (iSysTrayIconOptions.iIconSet == 0)
		RemoveIcon();
	else
		SetIcon();
}


void CSysTrayIconController::SetIcon()
{
	HICON	icon = CSysTrayIconUtil::Icon(iSysTrayIconOptions.iIconSet, iCurrentDesktop, iSysTrayIconOptions);

	iNotifyIconData.hIcon = icon; 
	sprintf(iNotifyIconData.szTip, "Desktop %d", iCurrentDesktop + 1);

	if (::Shell_NotifyIcon(iIconAdded ? NIM_MODIFY: NIM_ADD, &iNotifyIconData))
		iIconAdded = true;

		/*
		 * Very sublte, but we need to hide this window. It makes no difference in the appearance of the
		 * systray icon. If it remains in the visible state, deskwin will try to hide it when switching desktop.
		 * The latter is done asynchronously (using ShowWindowAsync), after which deskwin will wait a short while (up
		 * to a second) to give all relevant windows a change to hide themselves. If deskwin's CSysTrayIconController 
		 * is visible, then we always have a deadlock (for the full duration of the 1 second time-out), because whilst
		 * waiting & checking for windows to become invisible, deskwin cannot respond to the WM_SHOW message of its 
		 * CSysTrayIconController window.
		 */
	ShowWindow(SW_HIDE);
}


void CSysTrayIconController::RemoveIcon()
{
	if (iIconAdded)
	{
		::Shell_NotifyIcon(NIM_DELETE, &iNotifyIconData);
		iIconAdded = false;
	}
}

void CSysTrayIconController::HandleDesktopChanged(int aOldDesktop, int aNewDesktop)
{
	iCurrentDesktop = aNewDesktop;
	if (iSysTrayIconOptions.iIconSet != 0)
		SetIcon();
}

