/*******************************************************************
 *
 *	File:		CSysTrayIconUtil.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 1999, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#include "deskwin.h"
#include "CSysTrayIconUtil.h"
#include "resource.h"

#if 0

/************************************************
 *
 * CC_HSLtoRGB and CC_RGBtoHSL "borrowed" from
 * the WINE project. ;-)
 *
 ************************************************/

//#define min(p1,p2) (((p1)<(p2))? (p1) : (p2))
//#define max(p1,p2) (((p1)>(p2))? (p1) : (p2))

static int CC_HSLtoRGB(char c, int hue, int sat, int lum)
{
 int res = 0, maxrgb;

 /* hue */
 switch(c)
 {
  case 'R': if (hue > 80)  hue -= 80; else hue += 160; break;
  case 'G': if (hue > 160) hue -= 160; else hue += 80; break;
  case 'B': break;
 }

 /* l below 120 */
 maxrgb = (256 * min(120,lum)) / 120;  /* 0 .. 256 */
 if (hue < 80)
  res = 0;
 else
  if (hue < 120)
  {
   res = (hue - 80) * maxrgb;           /* 0...10240 */
   res /= 40;                        /* 0...256 */
  }
  else
   if (hue < 200)
    res = maxrgb;
   else
    {
     res= (240 - hue) * maxrgb;
     res /= 40;
    }
 res = res - maxrgb / 2;                 /* -128...128 */

 /* saturation */
 res = maxrgb / 2 + (sat * res) / 240;    /* 0..256 */

 /* lum above 120 */
 if (lum > 120 && res < 256)
  res += ((lum - 120) * (256 - res)) / 120;

 return min(res, 255);
}

static int CC_RGBtoHSL(char c, int r, int g, int b)
{
 int maxi, mini, mmsum, mmdif, result = 0;
 int iresult = 0;

 maxi = max(r, b);
 maxi = max(maxi, g);
 mini = min(r, b);
 mini = min(mini, g);

 mmsum = maxi + mini;
 mmdif = maxi - mini;

 switch(c)
 {
  /* lum */
  case 'L': mmsum *= 120;              /* 0...61200=(255+255)*120 */
	   result = mmsum / 255;        /* 0...240 */
	   break;
  /* saturation */
  case 'S': if (!mmsum)
	    result = 0;
	   else
	    if (!mini || maxi == 255)
	     result = 240;
	   else
	   {
	    result = mmdif * 240;       /* 0...61200=255*240 */
	    result /= (mmsum > 255 ? mmsum = 510 - mmsum : mmsum); /* 0..255 */
	   }
	   break;
  /* hue */
  case 'H': if (!mmdif)
	    result = 160;
	   else
	   {
	    if (maxi == r)
	    {
	     iresult = 40 * (g - b);       /* -10200 ... 10200 */
	     iresult /= (int) mmdif;    /* -40 .. 40 */
	     if (iresult < 0)
	      iresult += 240;          /* 0..40 and 200..240 */
	    }
	    else
	     if (maxi == g)
	     {
	      iresult = 40 * (b - r);
	      iresult /= (int) mmdif;
	      iresult += 80;           /* 40 .. 120 */
	     }
	     else
	      if (maxi == b)
	      {
	       iresult = 40 * (r - g);
	       iresult /= (int) mmdif;
	       iresult += 160;         /* 120 .. 200 */
	      }
	    result = iresult;
	   }
	   break;
 }
 return result;    /* is this integer arithmetic precise enough ? */
}


static HICON RecolorIcon(HICON aIcon)
{
	ICONINFO	srcIconInfo;
	ICONINFO	dstIconInfo;
	GetIconInfo(aIcon, &srcIconInfo);

	CMemoryDC	dstMemDc(GetDC(GetDesktopWindow())), srcMemDc, dstMaskMemDc, srcMaskMemDc;
//	CBitmap		bitmap(32, 32, 16, NULL);
	CBitmap		bitmap(dstMemDc.CreateCompatibleBitmap(32, 32));
	CBitmap		bitmapMask(32, 32, NULL);

	srcMemDc.SelectBitmap(srcIconInfo.hbmColor);
	srcMaskMemDc.SelectBitmap(srcIconInfo.hbmMask);
	dstMemDc.SelectBitmap(bitmap);
	dstMaskMemDc.SelectBitmap(bitmapMask);
	for (int i=0 ; i<32 ; i++)
	{
		for (int j=0 ; j<32 ; j++)
		{
			COLORREF	color = srcMemDc.GetPixel(i, j);
			if (color == ct_Black)
			{
				dstMemDc.SetPixel(i, j, ct_DarkRed);
			}
			color = srcMaskMemDc.GetPixel(i, j);
			dstMaskMemDc.SetPixel(i, j, color);
		}
	}
	dstIconInfo.fIcon = true;
	dstIconInfo.hbmColor = bitmap;
	dstIconInfo.hbmMask = srcIconInfo.hbmMask;
	dstIconInfo.xHotspot = 16;
	dstIconInfo.xHotspot = 16;
	return CreateIconIndirect(&dstIconInfo);
}


static HBITMAP RecolorBitmap(HICON aIcon, COLORREF aColor, COLORREF aBgColor)
{
	ICONINFO	srcIconInfo;
	GetIconInfo(aIcon, &srcIconInfo);

	HBITMAP hBitmap = ::CreateCompatibleBitmap(GetDC(GetDesktopWindow()), 32, 32);
	CBitmap	bitmap(hBitmap);

	CMemoryDC	memDc;
	memDc.SelectBitmap(srcIconInfo.hbmMask);
	CMemoryDC	memDc1;
	memDc1.SelectBitmap(bitmap);

	COLORREF	color = ct_DarkRed;
	int	r = 255;
	int g = 0;
	int b = 0;

	int lum = CC_RGBtoHSL('L', r ,g, b);
	int sat = CC_RGBtoHSL('S', r ,g, b);
	int hue = CC_RGBtoHSL('H', r ,g, b);

	int satDiff = (sat<128) ? 0: -0;
	int hueDiff = (hue<128) ? 0: -0;
	int lumDiff = (lum<160) ? 1: -1;

	for (int i=0 ; i<32 ; i++)
	{
		for (int j=0 ; j<32 ; j++)
		{
			COLORREF	color = memDc.GetPixel(i, j);
			if (color == ct_Black)
			{
				r = CC_HSLtoRGB('R', hue + ((i* hueDiff) >> 2), sat + ((j * satDiff) >> 2), lum + (i * lumDiff) + (j * lumDiff));
				g = CC_HSLtoRGB('G', hue + ((i* hueDiff) >> 2), sat + ((j * satDiff) >> 2), lum + (i * lumDiff) + (j * lumDiff));
				b = CC_HSLtoRGB('B', hue + ((i* hueDiff) >> 2), sat + ((j * satDiff) >> 2), lum + (i * lumDiff) + (j * lumDiff));
				memDc1.SetPixel(i, j, RGB(r, g, b));
			}
			else
				memDc1.SetPixel(i, j, aBgColor);
		}
	}
	return hBitmap;
}

HBITMAP CSysTrayIconUtil::Bitmap(int aIconSet, int aDesktop, COLORREF aColor, COLORREF aBgColor)
{

	HICON icon = CSysTrayIconUtil::Icon(aIconSet, aDesktop, aColor, false);
	HBITMAP bitmap = RecolorBitmap(icon, aColor, aBgColor);
	DestroyIcon(icon);
	return bitmap;

}


static HBITMAP RecolorBitmap(HICON aIcon, COLORREF aColor, COLORREF aBgColor)
{
	ICONINFO	srcIconInfo;
	GetIconInfo(aIcon, &srcIconInfo);

	HBITMAP hBitmap = ::CreateCompatibleBitmap(GetDC(GetDesktopWindow()), 32, 32);
	CBitmap	bitmap(hBitmap);

	CMemoryDC	memDc;
	memDc.SelectBitmap(srcIconInfo.hbmMask);
	CMemoryDC	memDc1;
	memDc1.SelectBitmap(bitmap);

	for (int i=0 ; i<32 ; i++)
	{
		for (int j=0 ; j<32 ; j++)
		{
			COLORREF	color = memDc.GetPixel(i, j);
			if (color == ct_Black)
				memDc1.SetPixel(i, j, aColor);
			else
				memDc1.SetPixel(i, j, aBgColor);
		}
	}
	return hBitmap;
}
static HICON RecolorIcon(HICON aIcon, COLORREF aColor)
{
	ICONINFO	srcIconInfo;
	GetIconInfo(aIcon, &srcIconInfo);

	CMemoryDC	memDc;
	memDc.SelectBitmap(srcIconInfo.hbmColor);

	for (int i=0 ; i<32 ; i++)
	{
		for (int j=0 ; j<32 ; j++)
		{
			COLORREF	color = memDc.GetPixel(i, j);
			if (color == ct_Black)
			{
				memDc.SetPixel(i, j, aColor);
			}
		}
	}
	return ::CreateIconIndirect(&srcIconInfo);
}
#endif

/********************************************
 *
 *
 ********************************************/


bool IsMonoPixelIconMaskSet(unsigned* aBits, int aX, int aY)
{
	return ((aBits[aY] & (1 << aX)) != 0);
}

void SetMonoPixelIconMaskSet(unsigned* aBits, int aX, int aY)
{
	unsigned char* byte = ((unsigned char*) aBits) + (aY * 4) + (aX / 8);
	aX &= 7;
	*byte |= (1 << (7 - aX));
}

void ClearMonoPixelIconMaskSet(unsigned* aBits, int aX, int aY)
{
	unsigned char* byte = ((unsigned char*) aBits) + (aY * 4) + (aX / 8);
	aX &= 7;
	*byte &= ~(1 << (7 - aX));
}


static HICON RecolorIcon(HICON aIcon, TSysTrayIconOptions& aSysTrayIconOptions)
{
	ICONINFO	srcIconInfo;
	GetIconInfo(aIcon, &srcIconInfo);

	CMemoryDC	memDc;
	memDc.SelectBitmap(srcIconInfo.hbmColor);

	unsigned int	Bits[32*32];
	LONG count = GetBitmapBits(srcIconInfo.hbmMask, sizeof(Bits), Bits);

	for (int i=31 ; i>=0 ; i--)
	{
		for (int j=31 ; j>=0 ; j--)
		{
			COLORREF	color = memDc.GetPixel(i, j);
			if (color == ct_White)
			{
				memDc.SetPixel(i, j, aSysTrayIconOptions.iFgColor);
				ClearMonoPixelIconMaskSet(Bits, i, j);
			}
			else
			{
				if (memDc.GetPixel(i-2, j-2) == ct_White)
				{
					ClearMonoPixelIconMaskSet(Bits, i, j);
				}
				else if (!aSysTrayIconOptions.iTransparentBackground)
				{
					memDc.SetPixel(i, j, aSysTrayIconOptions.iBgColor);
					ClearMonoPixelIconMaskSet(Bits, i, j);
				}
				else
				{
				}
			}
		}
	}
	count = SetBitmapBits(srcIconInfo.hbmMask, count, Bits);

	return ::CreateIconIndirect(&srcIconInfo);
}


HICON CSysTrayIconUtil::Icon(int aIconSet, int aDesktop, TSysTrayIconOptions& aSysTrayIconOptions, bool aReColor)
{
	if ((aIconSet == 0) || (aIconSet == 1))
	{
		return ::LoadIcon(GetAppInstance(), MAKEINTRESOURCE(IDI_MAIN_ICON));
	}
	else if ((aDesktop>=0) && (aDesktop<=8))
	{
		HICON icon = ::LoadIcon(GetAppInstance(), MAKEINTRESOURCE(IDI_ICON_SET1_1 + ((aIconSet-2) * 9) + aDesktop));
		if (aReColor)
		{
			HICON icon1 = icon;
			icon = RecolorIcon(icon, aSysTrayIconOptions);
			DestroyIcon(icon1);
		}
		return icon;
	}

	return ::LoadIcon(GetAppInstance(), MAKEINTRESOURCE(IDI_MAIN_ICON));
}

