//---------------------------
// CW framework <cwindow.cpp>
//
// (Released under the terms
// of the GNU General Public
// License.)
//
// author:  Michael Hansen
// initiated: February 1998
//---------------------------

#include "deskwin.h"
#include "CInfoWindow.h"

const int ct_nIdleWait = 10; // msec to wait after calling idle handlers

// constant strings
const TCHAR ct_szNotAdded[] = _TEXT("Window must be added to parent or application object before creation.");
const TCHAR ct_szNotAddedTitle[] = _TEXT("Cannot Create Window");

CApplication *_g_pApplication;
int _g_nCWAppClassID = 0;


void CDialog::OnHelp(LPHELPINFO aHelpInfo)
{
#if 0
	if (aHelpInfo->hItemHandle)
	{
		SToolTipInfo* toolTipInfo = FindToolTip(aHelpInfo->iCtrlId);
		if (toolTipInfo)
			iInfoWindow->SetInfoText(toolTipInfo->iHelpTextId);
		else
			iInfoWindow->SetInfoText("Stukkie text");

		iInfoWindow->Popup(aHelpInfo->MousePos.x, aHelpInfo->MousePos.y);
	}
#endif
}


void CDialog::AddToolTipInfo(SToolTipInfo& aToolTipInfo)
{
	if (!FindToolTip(aToolTipInfo.iControlId))
	{
		iToolTipInfo.Add(aToolTipInfo);
		EnableToolTip();
	}
}


SToolTipInfo* CDialog::FindToolTip(unsigned int aControlId)
{
	int	count = iToolTipInfo.GetCount();

	for (int i=0; i<count ; i++)
	{
		if (iToolTipInfo[i].iControlId == aControlId)
		{
			return &iToolTipInfo[i];
		}
	}
	return NULL;
}


void CDialog::EnableToolTip()
{
	if (!iToolTipEnabled)
	{
		iToolTipEnabled = TRUE;
		AddNewCW(iInfoWindow, GetApplication());
		iInfoWindow->Construct(this->HWindow);
	}
}

void CDialog::SpyMessageHook(HWND &HWnd, UINT &nMsg, WPARAM &wParam, LPARAM &lParam)
{
	if (iToolTipEnabled && iInfoWindow)
	{
		if (IsChild(HWnd))
		{
			switch (nMsg) 
			{
				case WM_MOUSEMOVE:
				case WM_LBUTTONDOWN:
				case WM_LBUTTONUP:
				case WM_RBUTTONDOWN:
				case WM_RBUTTONUP:
					if (iInfoWindow != NULL) 
					{
						MSG		msg; 

						msg.lParam	= lParam; 
						msg.wParam	= wParam; 
						msg.message	= nMsg; 
						msg.hwnd	= HWnd; 
						::SendMessage(iInfoWindow->HWindow, TTM_RELAYEVENT, 0, (LPARAM) (LPMSG) &msg);
					} 
					break; 

				default: 
					break; 
			}
		}
	}
}

void CDialog::OnNotify(HWND hwndFrom, UINT idFrom, UINT code, WPARAM wParam, LPARAM lParam)
{
	if (code == TTN_NEEDTEXT) 
	{
		LPTOOLTIPTEXT	lpttt = (LPTOOLTIPTEXT) lParam;

		sprintf(lpttt->lpszText, "Stukkie helpie text");

		//switch (idCtrl)
	} 
	return; 
}


HINSTANCE GetAppInstance()
{
	return _g_pApplication->HInstance;
}

CApplication *GetApplication()
{
	return _g_pApplication;
}

BOOL CALLBACK CDialogProc(HWND HWnd, UINT nMsg, WPARAM wParam, LPARAM lParam)
{
	return (BOOL)_g_pApplication->Dispatch(HWnd, nMsg, wParam, lParam);
}

LRESULT CALLBACK CWindowProc(HWND HWnd, UINT nMsg, WPARAM wParam, LPARAM lParam)
{
	return _g_pApplication->Dispatch(HWnd, nMsg, wParam, lParam);
}

int WINAPI WinMain(HINSTANCE hInst, HINSTANCE hPrevInst, LPTSTR lpszCmdLine,
		int nCmdShow)
{
	return _g_pApplication->main(hInst, hPrevInst, lpszCmdLine, nCmdShow);
}

BOOL WINAPI DllMain(HINSTANCE hInstDLL, DWORD dwReason, LPVOID)
{
    switch (dwReason)
	{
		case DLL_PROCESS_ATTACH:
			_g_pApplication->dllmain(hInstDLL);
			break;
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			break;
    }
    return TRUE;
}



//////////////////////////////////////////////////////////////
//
// CMenu
//
CMenu::CMenu(int nID) : oExternal(false)
{
	m_pnRefCnt = new int(0);
	Handle = _g_pApplication->LoadMenu(nID);
}

CMenu::CMenu(const TCHAR *pszMenu) : oExternal(false)
{
	m_pnRefCnt = new int(0);
	Handle = _g_pApplication->LoadMenu(pszMenu);
}
//
//
//
//////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////
//
// CPalette
//
CPalette::CPalette(int nEntryCnt, RGBQUAD *pEntries) : oExternal(false)
{
	m_pnRefCnt = new int(0);
	
	char *pBuffer = new char[sizeof(WORD)*2 + sizeof(PALETTEENTRY)*nEntryCnt];
	LOGPALETTE *pLogPal = (LOGPALETTE *)pBuffer;

	pLogPal->palVersion = 0x300;
	pLogPal->palNumEntries = nEntryCnt;
    for(int i=0;i<nEntryCnt;i++)
    {
		pLogPal->palPalEntry[i].peRed = pEntries[i].rgbRed;
		pLogPal->palPalEntry[i].peGreen = pEntries[i].rgbGreen;
		pLogPal->palPalEntry[i].peBlue = pEntries[i].rgbBlue;
		pLogPal->palPalEntry[i].peFlags = 0;
	}
	
	Handle = CreatePalette(pLogPal);
	
	delete [] pBuffer;
}
//
//
//
//////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////
//
// CDIB
//
void CDIB::GetInfoFromMemory(void)
{
	if(pInfo)
	{
		nPaletteCount = pInfo->bmiHeader.biClrUsed;
		if(pInfo->bmiHeader.biClrUsed == 0)
		{
			switch(pInfo->bmiHeader.biBitCount)
			{
				case 1: nPaletteCount = 2; break;
				case 4: nPaletteCount = 16; break;
				case 8: nPaletteCount = 256; break;
				default: nPaletteCount = 0; break;
			}
		}
		pbBits = (void *)((long)pInfo->bmiColors + nPaletteCount*sizeof(RGBQUAD));
	}
	else
	{
		nPaletteCount = 0;
		pbBits = 0;
	}
}
//
//
//
//////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////
//
// CDC
//
void CDC::RestoreObjects()
{
	// restore objects in reverse order
	// (note->it is OK that already selected object is reselected)
	int i = m_Objects.GetCount() - 1;
	while(i >= 0)
	{
		::SelectObject(Handle, m_Objects[i--]);
	}

	m_Objects.RemoveAll();
}

void CDC::SelectObject(CGDIObject &Object)
{
	HGDIOBJ hObject = ::SelectObject(Handle, Object);

	// if old object not in list, it does not belong to this CDC
	// and must add to list so it will be restored
	int i;
	for(i=0;i<m_Objects.GetCount();i++)
	{
		if((HGDIOBJ)m_Objects[i] == hObject)
			break;
	}
	if(i == m_Objects.GetCount())
		m_Objects.Add(CGDIObject(hObject, false));

	m_Objects.Add(Object);
}

void CDC::DrawDisableBitmap(HBITMAP hBitmap, int nX, int nY, COLORREF DisableColor,
		COLORREF DisableHilite)
{
	BITMAP bm;
	COLORREF OldColor;
	POINT ptSize;
	
	CMemoryDC dcTemp(Handle);
	dcTemp.SelectBitmap(hBitmap); // Select the bitmap
	
	GetObject(hBitmap, sizeof(BITMAP), &bm);
	ptSize.x = bm.bmWidth;            // Get width of bitmap
	ptSize.y = bm.bmHeight;           // Get height of bitmap
	DPtoLP(&ptSize, 1);      // Convert from device to logical points

	// Create some DCs to hold temporary data.
	CMemoryDC dcBack(Handle);
	CMemoryDC dcObject(Handle);
	CMemoryDC dcMem(Handle);
	CMemoryDC dcSave(Handle);
	
	// Create a bitmap for each DC. DCs are required for a number of
	// GDI functions.

	// Monochrome DC
	CBitmap bitmapAndBack(ptSize.x, ptSize.y, 0);
	CBitmap bitmapAndObject(ptSize.x, ptSize.y, 0);
	
	CBitmap bitmapAndMem(CreateCompatibleBitmap(ptSize.x, ptSize.y), true);
	CBitmap bitmapSave(CreateCompatibleBitmap(ptSize.x, ptSize.y), true);
	
	// Each DC must select a bitmap object to store pixel data.
	dcBack.SelectBitmap(bitmapAndBack);
	dcObject.SelectBitmap(bitmapAndObject);
	dcMem.SelectBitmap(bitmapAndMem);
	dcSave.SelectBitmap(bitmapSave);

	// Set proper mapping mode.
	dcTemp.SetMapMode(GetMapMode());
	
	// Save the bitmap sent here, because it will be overwritten.
	dcSave.BitBlt(0, 0, ptSize.x, ptSize.y, dcTemp, 0, 0, SRCCOPY);
	// get transparent color from first pixel
	COLORREF TransparentColor = dcSave.GetPixel(0, 0);
	
	// Set the background color of the source DC to the color.
	// contained in the parts of the bitmap that should be transparent
	OldColor = dcTemp.SetBkColor(TransparentColor);
	
	// Create the object mask for the bitmap by performing a BitBlt
	// from the source bitmap to a monochrome bitmap.
	// (Transparent is white, object is black)
	dcObject.BitBlt(0, 0, ptSize.x, ptSize.y, dcTemp, 0, 0, SRCCOPY);
	
	// Set the background color of the source DC back to the original color.
	dcTemp.SetBkColor(OldColor);

	// Create the inverse of the object mask.
	// (Transparent is black, object is white)
	dcBack.BitBlt(0, 0, ptSize.x, ptSize.y, dcObject, 0, 0, NOTSRCCOPY);

	// First, create a rectangle in dc of the color white
	CPen pen(DisableHilite);
	CBrush brush(DisableHilite);
	dcTemp.SelectObject(pen);
	dcTemp.SelectObject(brush);
	dcTemp.Rectangle(0, 0, ptSize.x, ptSize.y);
	dcTemp.RestoreObjects();
	
	// Copy the background of the main DC to the destination.
	dcMem.BitBlt(0, 0, ptSize.x, ptSize.y, *this, nX, nY, SRCCOPY);
	// (new image now has original background image)

	// Mask out the places where the bitmap will be placed.
	dcMem.BitBlt(1, 1, ptSize.x, ptSize.y, dcObject, 0, 0, SRCAND);
	// (new image now has original background image and is black
	// where the new object goes)

	// Mask out the transparent colored pixels on the bitmap.
	dcTemp.BitBlt(1, 1, ptSize.x, ptSize.y, dcBack, 0, 0, SRCAND);
	// (need to fill in offset)
	CPen blackpen(RGB(0,0,0));
	dcTemp.SelectObject(blackpen);
	dcTemp.MoveTo(0, ptSize.y);
	dcTemp.LineTo(0, 0);
	dcTemp.LineTo(ptSize.x, 0);
	dcTemp.RestoreObjects();
	// (a white object now has black for its transparent background)

	// XOR the bitmap with the background on the destination DC.
	dcMem.BitBlt(0, 0, ptSize.x, ptSize.y, dcTemp, 0, 0, SRCPAINT);
	// (new image received the white object where it was black)

	// Copy the destination to the screen.
	BitBlt(nX, nY, ptSize.x, ptSize.y, dcMem, 0, 0, SRCCOPY);

	// Create a rectangle in dc of disabled color
	CPen pen2(DisableColor);
	CBrush brush2(DisableColor);
	dcTemp.SelectObject(pen2);
	dcTemp.SelectObject(brush2);
	dcTemp.Rectangle(0, 0, ptSize.x, ptSize.y);
	dcTemp.RestoreObjects();
	
	// Copy the background of the main DC to the destination.
	dcMem.BitBlt(0, 0, ptSize.x, ptSize.y, *this, nX, nY, SRCCOPY);
	// (new image now has original background image)

	// Mask out the places where the bitmap will be placed.
	dcMem.BitBlt(0, 0, ptSize.x, ptSize.y, dcObject, 0, 0, SRCAND);
	// (new image now has original background image and is black
	// where the new object goes)

	// Mask out the transparent colored pixels on the bitmap.
	dcTemp.BitBlt(0, 0, ptSize.x, ptSize.y, dcBack, 0, 0, SRCAND);
	// (a disabled object now has black for its transparent background)

	// XOR the bitmap with the background on the destination DC.
	dcMem.BitBlt(0, 0, ptSize.x, ptSize.y, dcTemp, 0, 0, SRCPAINT);
	// (new image received the disabled object where it was black)

	// Copy the destination to the screen.
	BitBlt(nX, nY, ptSize.x, ptSize.y, dcMem, 0, 0, SRCCOPY);

	// Place the original bitmap back into the bitmap sent here.
	dcTemp.BitBlt(0, 0, ptSize.x, ptSize.y, dcSave, 0, 0, SRCCOPY);

	// restore (unselect) bitmaps before they are deleted
	dcTemp.RestoreBitmap();
	dcBack.RestoreBitmap();
	dcObject.RestoreBitmap();
	dcMem.RestoreBitmap();
	dcSave.RestoreBitmap();
}

void CDC::DrawTransparentBitmap(HBITMAP hBitmap, int nX, int nY)
{
	BITMAP bm;
	COLORREF OldColor;
	POINT ptSize;
	
	CMemoryDC dcTemp(Handle);
	dcTemp.SelectBitmap(hBitmap); // Select the bitmap
	
	GetObject(hBitmap, sizeof(BITMAP), &bm);
	ptSize.x = bm.bmWidth;            // Get width of bitmap
	ptSize.y = bm.bmHeight;           // Get height of bitmap
	DPtoLP(&ptSize, 1);      // Convert from device to logical points

	// Create some DCs to hold temporary data.
	CMemoryDC dcBack(Handle);
	CMemoryDC dcObject(Handle);
	CMemoryDC dcMem(Handle);
	CMemoryDC dcSave(Handle);
	
	// Create a bitmap for each DC. DCs are required for a number of
	// GDI functions.

	// Monochrome DC
	CBitmap bitmapAndBack(ptSize.x, ptSize.y, 0);
	CBitmap bitmapAndObject(ptSize.x, ptSize.y, 0);
	
	CBitmap bitmapAndMem(CreateCompatibleBitmap(ptSize.x, ptSize.y), true);
	CBitmap bitmapSave(CreateCompatibleBitmap(ptSize.x, ptSize.y), true);
	
	// Each DC must select a bitmap object to store pixel data.
	dcBack.SelectBitmap(bitmapAndBack);
	dcObject.SelectBitmap(bitmapAndObject);
	dcMem.SelectBitmap(bitmapAndMem);
	dcSave.SelectBitmap(bitmapSave);

	// Set proper mapping mode.
	dcTemp.SetMapMode(GetMapMode());
	
	// Save the bitmap sent here, because it will be overwritten.
	dcSave.BitBlt(0, 0, ptSize.x, ptSize.y, dcTemp, 0, 0, SRCCOPY);
	// get transparent color from first pixel
	COLORREF TransparentColor = dcSave.GetPixel(0, 0);
	
	// Set the background color of the source DC to the color.
	// contained in the parts of the bitmap that should be transparent
	OldColor = dcTemp.SetBkColor(TransparentColor);
	
	// Create the object mask for the bitmap by performing a BitBlt
	// from the source bitmap to a monochrome bitmap.
	dcObject.BitBlt(0, 0, ptSize.x, ptSize.y, dcTemp, 0, 0, SRCCOPY);
	
	// Set the background color of the source DC back to the original color.
	dcTemp.SetBkColor(OldColor);

	// Create the inverse of the object mask.
	dcBack.BitBlt(0, 0, ptSize.x, ptSize.y, dcObject, 0, 0, NOTSRCCOPY);
	
	// Copy the background of the main DC to the destination.
	dcMem.BitBlt(0, 0, ptSize.x, ptSize.y, *this, nX, nY, SRCCOPY);
	// (new image now has original background image)

	// Mask out the places where the bitmap will be placed.
	dcMem.BitBlt(0, 0, ptSize.x, ptSize.y, dcObject, 0, 0, SRCAND);
	// (new image now has original background image and is black
	// where the new object goes)

	// Mask out the transparent colored pixels on the bitmap.
	dcTemp.BitBlt(0, 0, ptSize.x, ptSize.y, dcBack, 0, 0, SRCAND);
	// (object in the bitmap now has black for its transparent background)

	// XOR the bitmap with the background on the destination DC.
	dcMem.BitBlt(0, 0, ptSize.x, ptSize.y, dcTemp, 0, 0, SRCPAINT);
	// (new image received the object in the bitmap where it was black)

	// Copy the destination to the screen.
	BitBlt(nX, nY, ptSize.x, ptSize.y, dcMem, 0, 0, SRCCOPY);

	// Place the original bitmap back into the bitmap sent here.
	dcTemp.BitBlt(0, 0, ptSize.x, ptSize.y, dcSave, 0, 0, SRCCOPY);

	// restore (unselect) bitmaps before they are deleted
	dcTemp.RestoreBitmap();
	dcBack.RestoreBitmap();
	dcObject.RestoreBitmap();
	dcMem.RestoreBitmap();
	dcSave.RestoreBitmap();
}
//
//
//
//////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////
//
// CWindow
//
CWindow::CWindow(HWND HWnd) : HWindow(HWnd), oCreating(false), m_oRemove(false),
		oAdded(false), pParent(0), oDialogTranslate(false), oClient(false), oDefDblClks(false),
		Font(0), m_oAutoDestroy(false), HAccel(0)
{
	// add destroy message processing
	AddMsg(SWinMsg(WM_NCDESTROY, (CWMSGHANDLER)CWindow::OnNCDestroy));
	AddMsg(SWinMsg(WM_HELP, (CWMSGHANDLER)CWindow::OnWmHelp));
	AddMsg(SWinMsg(WM_NOTIFY, (CWMSGHANDLER)CWindow::OnWmNotify));

	// get window text
	int nTextSize = GetWindowTextLength()+1;
	m_pszWindowText = new TCHAR[nTextSize];
	GetWindowText(m_pszWindowText, nTextSize);

	// should not compare directly since m_pDefaultProc may not equal CWindowProc
	m_pDefaultProc = (WNDPROC)GetWindowLong(GWL_WNDPROC);
	TCHAR szClassName[64];
	GetClassName(szClassName, sizeof(szClassName)/sizeof(TCHAR));
	// did application registered the class, if not subclass
	if(!_g_pApplication->IsRegisteredClass(szClassName))
		SetWindowLong(GWL_WNDPROC, (long)CWindowProc);
	else
		m_pDefaultProc = 0;

}


CWindow::~CWindow()
{
	delete [] m_pszWindowText;

	// non-allocated windows must be removed from lists
	if(!m_oAutoDestroy)
	{
		if(!pParent) // remove from application, if no parent window
			_g_pApplication->RemoveWindow(this);
		else // otherwise, remove from parent
			pParent->RemoveChild(this);
	}

	for(int i=0;i<m_Windows.GetCount();i++)
	{
		// delete and detach from remaining windows
		if(m_Windows[i]->IsAutoDestroy())
			delete m_Windows[i];
		else
			m_Windows[i]->pParent = 0;
	}
}

void CWindow::SizeToClient(SIZE &Size)
{
	for(int i=0;i<m_Windows.GetCount();i++)
	{
		if(m_Windows[i]->oClient)
			m_Windows[i]->MoveWindow(0, 0, Size.cx, Size.cy);
	}
}

void CWindow::RemoveChild(CWindow *pWindow)
{
	for(int i=0;i<m_Windows.GetCount();i++)
	{
		if(pWindow == m_Windows[i])
		{
			m_Windows.Remove(i);
			if(pWindow->IsWindow())
				pWindow->RestoreDefProc();
			break;
		}
	}
}

void CWindow::AddCmd(SWinCmd &CmdHandler)
{
	for(int i=0;i<m_CmdHandlers.GetCount();i++)
	{
		if(m_CmdHandlers[i].nId == CmdHandler.nId &&
				m_CmdHandlers[i].nNotification == CmdHandler.nNotification)
		{
			m_CmdHandlers[i].pHandler = CmdHandler.pHandler;
			return;
		}
	}

	m_CmdHandlers.Add(CmdHandler);
}

void CWindow::RemoveCmd(unsigned int nId, unsigned int nNotification)
{
	for(int i=0;i<m_CmdHandlers.GetCount();i++)
	{
		if(m_CmdHandlers[i].nId == nId &&
				m_CmdHandlers[i].nNotification == nNotification)
		{
			m_CmdHandlers.Remove(i);
			break;
		}
	}
}

void CWindow::AddMsg(SWinMsg &MsgHandler)
{
	for(int i=0;i<m_MsgHandlers.GetCount();i++)
	{
		if(m_MsgHandlers[i].nMessage == MsgHandler.nMessage)
		{
			m_MsgHandlers[i].pHandler = MsgHandler.pHandler;
			return;
		}
	}

	m_MsgHandlers.Add(MsgHandler);
}

void CWindow::RemoveMsg(unsigned int nMessage)
{
	for(int i=0;i<m_MsgHandlers.GetCount();i++)
	{
		if(m_MsgHandlers[i].nMessage == nMessage)
		{
			m_MsgHandlers.Remove(i);
			break;
		}
	}
}

void CWindow::AddMsgForward(unsigned int nMessage)
{
	for(int i=0;i<m_MsgParentForwards.GetCount();i++)
	{
		if(m_MsgParentForwards[i] == nMessage)
			return;
	}

	m_MsgParentForwards.Add(nMessage);
}

void CWindow::RemoveMsgForward(unsigned int nMessage)
{
	for(int i=0;i<m_MsgParentForwards.GetCount();i++)
	{
		if(m_MsgParentForwards[i] == nMessage)
		{
			m_MsgParentForwards.Remove(i);
			break;
		}
	}
}

bool CWindow::FindNextDialog(SFindDialog &Find)
{
	bool oRetVal = false;

	if(oDialogTranslate && !Find.nIndex)
	{
		Find.hwnd = HWindow;
		Find.nLevel++;
		Find.nIndex = Find.nLevel;
		oRetVal = true;
	}
	else if(oDialogTranslate)
	{
		Find.nIndex--;
	}

	if(!oRetVal)
	{
		for(int i=0;i<m_Windows.GetCount();i++)
		{
			// if gone, remove window and continue
			if(m_Windows[i]->IsDestroyed())
			{
				if(m_Windows[i]->IsAutoDestroy())
					delete m_Windows[i];
				else
					m_Windows[i]->oAdded = false;
				
				m_Windows.Remove(i--);
			}
			else if(m_Windows[i]->FindNextDialog(Find))
			{
				oRetVal = true;
				break;
			}
		}
	}

	return oRetVal;
}

bool CWindow::Create(const TCHAR *pszClass, int x, int y, int nWidth,
		int nHeight, int nID, DWORD dwStyle, LPVOID lpParam, DWORD dwExStyle)
{
	const TCHAR *pszClassName = (pszClass!=0)?pszClass:
			((oDefDblClks)?_TEXT("CWDblClkWin"):_TEXT("CWAppWin"));

	if(!oAdded)
	{
		::MessageBox(0, ct_szNotAdded, ct_szNotAddedTitle, MB_OK);
		return false;
	}
	oCreating = true;
	HWND HParent = (pParent)?pParent->HWindow:0;
//	HWND HParent = (pParent)?pParent->HWindow: ::GetDesktopWindow();
//	HWindow = ::CreateWindow(pszClassName, m_pszWindowText, m_dwDefStyle | dwStyle, x, y,
//			nWidth, nHeight, HParent, (HMENU)nID, _g_pApplication->HInstance, lpParam);
	HWindow = ::CreateWindowEx(dwExStyle, pszClassName, m_pszWindowText, m_dwDefStyle | dwStyle, x, y,
			nWidth, nHeight, HParent, (HMENU)nID, _g_pApplication->HInstance, lpParam);
	oCreating = false;

	if(pszClass && HWindow)
	{
		// should not compare directly since m_pDefaultProc may not equal CWindowProc
		m_pDefaultProc = (WNDPROC)GetWindowLong(GWL_WNDPROC);
		TCHAR szClassName[64];
		GetClassName(szClassName, sizeof(szClassName)/sizeof(TCHAR));
		// did application registered the class, if not subclass
		if(!_g_pApplication->IsRegisteredClass(szClassName))
			SetWindowLong(GWL_WNDPROC, (long)CWindowProc);
		else
			m_pDefaultProc = 0;
	}

	if((HFONT)Font != 0 && HWindow)
		SendMessage(WM_SETFONT, (WPARAM)(HFONT)Font);

	return (HWindow == 0)?false:true;
}

LRESULT CWindow::OnNCDestroy(WPARAM wParam, LPARAM lParam)
{
	m_oRemove = true;
	return Default(WM_NCDESTROY, wParam, lParam);
}

bool CWindow::Close()
{
	bool oRetVal = CanClose();

	for(int i=0;i<m_Windows.GetCount() && oRetVal;i++)
	{
		// if gone, remove window and continue
		if(m_Windows[i]->IsDestroyed())
		{
			if(m_Windows[i]->IsAutoDestroy())
				delete m_Windows[i];
			else
				m_Windows[i]->oAdded = false;
			
			m_Windows.Remove(i--);
		}
		else
		{
			oRetVal = m_Windows[i]->CanClose();
		}
	}

	if(oRetVal)
		Destroy();

	return oRetVal;
}

int CWindow::GetText(TCHAR *pszText, int nMaxCount)
{
	if(HWindow) // if created
	{
		delete [] m_pszWindowText;
		// get window text
		int nTextSize = GetWindowTextLength()+1;
		m_pszWindowText = new TCHAR[nTextSize];
		GetWindowText(m_pszWindowText, nTextSize);
	}

	if(m_pszWindowText)
	{
		_tcsncpy(pszText, m_pszWindowText, nMaxCount-1);
		return _tcslen(pszText);
	}

	return 0;
}

void CWindow::SetText(const TCHAR *pszText)
{
	delete [] m_pszWindowText;
	int nTextSize = _tcslen(pszText)+1;
	m_pszWindowText = new TCHAR[nTextSize];
	_tcscpy(m_pszWindowText, pszText);

	if(HWindow) // if created
		SetWindowText(m_pszWindowText);
}

void CWindow::PreDispatch(HWND &HWnd, UINT &nMsg, WPARAM &wParam, LPARAM &lParam)
{
	SpyMessageHook(HWnd, nMsg, wParam, lParam);

	for(int i=0;i<m_Windows.GetCount();i++)
	{
		// if gone, remove window and continue
		if(m_Windows[i]->IsDestroyed())
		{
			if(m_Windows[i]->IsAutoDestroy())
				delete m_Windows[i];
			else
				m_Windows[i]->oAdded = false;
			
			m_Windows.Remove(i--);
		}
		else
		{
			m_Windows[i]->PreDispatch(HWnd, nMsg, wParam, lParam);
		}
	}
}

LRESULT CWindow::Dispatch(unsigned int nMessage, WPARAM wParam, LPARAM lParam)
{
	int i;

	if(nMessage == WM_COMMAND)
	{
		unsigned int nNotification = HIWORD(wParam);
		unsigned int nId = LOWORD(wParam);

		// treat accelerator commands as from a menu
//		if(nNotification == 1)
//			nNotification = 0;

		for(i=0;i<m_CmdHandlers.GetCount();i++)
			if(m_CmdHandlers[i].nId == nId &&
					m_CmdHandlers[i].nNotification == nNotification)
			{
				(this->*m_CmdHandlers[i].pHandler)();
				return 0L;
			}
	}

	for(i=0;i<m_MsgHandlers.GetCount();i++)
	{
		if(m_MsgHandlers[i].nMessage == nMessage)
			return (this->*m_MsgHandlers[i].pHandler)(wParam, lParam);
	}

	// check parent message forward list if not processed yet
	for(i=0;i<m_MsgParentForwards.GetCount();i++)
	{
		if(m_MsgParentForwards[i] == nMessage)
			return pParent->SendMessage(nMessage, wParam, lParam);
	}

	return Default(nMessage, wParam, lParam);
}

CWindow *CWindow::FindWindow(HWND HWnd)
{
	CWindow *pRetVal = 0;

	if(HWnd == HWindow || oCreating)
	{
		pRetVal = this;
		if(oCreating)
		{
			oCreating = false;
			HWindow = HWnd;
		}
	}
	else
	{
		for(int i=0;i<m_Windows.GetCount();i++)
		{
			// if gone, remove window and continue
			if(m_Windows[i]->IsDestroyed())
			{
				if(m_Windows[i]->IsAutoDestroy())
					delete m_Windows[i];
				else
					m_Windows[i]->oAdded = false;
				
				m_Windows.Remove(i--);
			}
			else if((pRetVal = m_Windows[i]->FindWindow(HWnd)) != 0)
				break;
		}
	}

	return pRetVal;
}

LRESULT CWindow::Default(unsigned int nMessage, WPARAM wParam, LPARAM lParam)
{
	if(m_pDefaultProc)
	{
#ifdef STRICT
		return CallWindowProc(m_pDefaultProc, HWindow, nMessage, wParam, lParam);
#else
		return CallWindowProc((FARPROC)m_pDefaultProc, HWindow, nMessage, wParam,
				lParam);
#endif
	}

	return DefWindowProc(nMessage, wParam, lParam);
}





//
//
//
//////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////
//
// CWindowEx
//
void CWindowEx::InitMsgHandlers()
{
	// initialize message handlers
	AddMsg(SWinMsg(WM_COMPAREITEM, (CWMSGHANDLER)CWindowEx::OnCompareItem));
	AddMsg(SWinMsg(WM_DELETEITEM, (CWMSGHANDLER)CWindowEx::OnDeleteItem));
	AddMsg(SWinMsg(WM_DRAWITEM, (CWMSGHANDLER)CWindowEx::OnDrawItem));
	AddMsg(SWinMsg(WM_MEASUREITEM, (CWMSGHANDLER)CWindowEx::OnMeasureItem));

	AddMsg(SWinMsg(WM_SIZE, (CWMSGHANDLER)CWindowEx::OnSize));
	AddMsg(SWinMsg(WM_HSCROLL, (CWMSGHANDLER)CWindowEx::OnHScroll));
	AddMsg(SWinMsg(WM_VSCROLL, (CWMSGHANDLER)CWindowEx::OnVScroll));
	AddMsg(SWinMsg(WM_PAINT, (CWMSGHANDLER)CWindowEx::OnPaint));
	AddMsg(SWinMsg(WM_CLOSE, (CWMSGHANDLER)CWindowEx::OnClose));
	AddMsg(SWinMsg(WM_QUERYENDSESSION, (CWMSGHANDLER)CWindowEx::OnWmQueryEndSession));
}

void CWindowEx::UpdateScrollInfo()
{
#ifdef __WIN16
	SetScrollRange(SB_HORZ, 0, m_sizeRange.cx);
	SetScrollRange(SB_VERT, 0, m_sizeRange.cy);
#else
	SCROLLINFO Info;

	Info.cbSize = sizeof(Info);
	Info.fMask = SIF_PAGE | SIF_RANGE;
	Info.nMin = 0;
	Info.nPos = 0;
	Info.nTrackPos = 0;
	
	Info.nPage = m_sizePage.cx;
	Info.nMax = m_sizeRange.cx;
	SetScrollInfo(SB_HORZ, &Info);

	Info.nPage = m_sizePage.cy;
	Info.nMax = m_sizeRange.cy;
	SetScrollInfo(SB_VERT, &Info);
#endif
}

long CWindowEx::OnSize(WPARAM, LPARAM lSize)
{
	SIZE Size;
	Size.cx = LOWORD(lSize);
	Size.cy = HIWORD(lSize);

	SizeToClient(Size);

	if(m_sizeDisplayPage.cx || m_sizeDisplayPage.cy)
	{
		RECT rcClient = GetClientRect();
		int nXOffset = 0;
		int nYOffset = 0;
#ifdef __WIN16
		nXOffset = rcClient.right;
		nYOffset = rcClient.bottom;
#else
		m_sizePage.cx = rcClient.right;
		m_sizePage.cy = rcClient.bottom;
#endif
		m_sizeRange.cx = m_sizeDisplayPage.cx - nXOffset;
		m_sizeRange.cy = m_sizeDisplayPage.cy - nYOffset;

		if(m_sizeRange.cx < m_sizePage.cx)
			m_sizeRange.cx = m_sizePage.cx = 0;
		if(m_sizeRange.cy < m_sizePage.cy)
			m_sizeRange.cy = m_sizePage.cy = 0;
			
		UpdateScrollInfo();
	}

	return 0;
}

long CWindowEx::OnCompareItem(WPARAM wParam, LPARAM lParam)
{
	if(wParam)
		return SendDlgItemMessage(wParam, WM_COMPAREITEM, 0, lParam);
	else
		return 0;
}

long CWindowEx::OnDeleteItem(WPARAM wParam, LPARAM lParam)
{
	if(wParam)
		return SendDlgItemMessage(wParam, WM_DELETEITEM, 0, lParam);
	else
		return 0;
}

long CWindowEx::OnDrawItem(WPARAM wParam, LPARAM lParam)
{
	if(wParam)
		return SendDlgItemMessage(wParam, WM_DRAWITEM, 0, lParam);
	else
		return 0;
}

long CWindowEx::OnMeasureItem(WPARAM wParam, LPARAM lParam)
{
	if(wParam)
		return SendDlgItemMessage(wParam, WM_MEASUREITEM, 0, lParam);
	else
		return 0;
}

long CWindowEx::OnHScroll(WPARAM wParam, LPARAM lParam)
{
#ifdef __WIN16
	int nScrollCode = wParam;
	int nPos = LOWORD(lParam);
#else
	int nScrollCode = LOWORD(wParam);
	int nPos = HIWORD(wParam);
#endif
	int nMin, nMax;
	int nScrollBy = 0;
	int nScrollPos = m_ptScrollPos.x;

	GetScrollRange(SB_HORZ, &nMin, &nMax);

	switch(nScrollCode)
	{
		case SB_LINEDOWN:
			// move window left dx where dx = device units/line
			nScrollBy = -m_sizeLine.cx;
			nScrollPos += m_sizeLine.cx;
			break;
		case SB_LINEUP:
			// move window right dx where dx = device units/line
			nScrollBy = m_sizeLine.cx;
			nScrollPos -= m_sizeLine.cx;
			break;
		case SB_PAGEDOWN:
			// move window left dx where dx = device units/page
			nScrollBy = -m_sizePage.cx;
			nScrollPos += m_sizePage.cx;
			break;
		case SB_PAGEUP:
			// move window right dx where dx = device units/page
			nScrollBy = m_sizePage.cx;
			nScrollPos -= m_sizePage.cx;
			break;
		case SB_THUMBPOSITION:
			// scroll to x where x = nPos
			nScrollBy = m_ptScrollPos.x - nPos;
			nScrollPos = nPos;
			break;
		case SB_THUMBTRACK:
			// scroll to x where x = nPos
			nScrollBy = m_ptScrollPos.x - nPos;
			nScrollPos = nPos;
			break;
		case SB_ENDSCROLL:
			break;
		case SB_TOP:
			// scroll to x where x = 0
			nScrollBy = m_ptScrollPos.x;
			nScrollPos = 0;
			break;
		case SB_BOTTOM:
			// scroll to x where x = range of x
			nScrollBy = m_ptScrollPos.x - (nMax-nMin);
			nScrollPos = nMax - nMin;
			break;
		default:
			break;
	}

	// go to left range limit
	if(m_ptScrollPos.x-nScrollBy < 0)
	{
		nScrollBy = m_ptScrollPos.x;
		m_ptScrollPos.x = 0;
	}
	// go to right range limit
	else if(m_ptScrollPos.x-nScrollBy > (nMax-nMin) - m_sizePage.cx)
	{
		nScrollBy = m_ptScrollPos.x - (nMax-nMin) - m_sizePage.cx;
		m_ptScrollPos.x = (nMax-nMin) - m_sizePage.cx;
	}
	else
	{
		m_ptScrollPos.x = nScrollPos;
	}
	
	if(nScrollBy)
	{
		ScrollWindow(nScrollBy, 0);
		SetScrollPos(SB_HORZ, m_ptScrollPos.x);
		UpdateWindow();
	}

	return 0;
}

long CWindowEx::OnVScroll(WPARAM wParam, LPARAM lParam)
{
#ifdef __WIN16
	int nScrollCode = wParam;
	int nPos = LOWORD(lParam);
#else
	int nScrollCode = LOWORD(wParam);
	int nPos = HIWORD(wParam);
#endif
	int nMin, nMax;
	int nScrollBy = 0;
	int nScrollPos = m_ptScrollPos.y;

	GetScrollRange(SB_VERT, &nMin, &nMax);

	switch(nScrollCode)
	{
		case SB_LINEDOWN:
			// move window up dy where dy = device units/line
			nScrollBy = -m_sizeLine.cy;
			nScrollPos += m_sizeLine.cy;
			break;
		case SB_LINEUP:
			// move window down dy where dy = device units/line
			nScrollBy = m_sizeLine.cy;
			nScrollPos -= m_sizeLine.cy;
			break;
		case SB_PAGEDOWN:
			// move window up dy where dy = device units/page
			nScrollBy = -m_sizePage.cy;
			nScrollPos += m_sizePage.cy;
			break;
		case SB_PAGEUP:
			// move window down dy where dy = device units/page
			nScrollBy = m_sizePage.cy;
			nScrollPos -= m_sizePage.cy;
			break;
		case SB_THUMBPOSITION:
			// scroll to y where y = nPos
			nScrollBy = m_ptScrollPos.y - nPos;
			nScrollPos = nPos;
			break;
		case SB_THUMBTRACK:
			// scroll to y where y = nPos
			nScrollBy = m_ptScrollPos.y - nPos;
			nScrollPos = nPos;
			break;
		case SB_ENDSCROLL:
			break;
		case SB_TOP:
			// scroll to y where y = 0
			nScrollBy = m_ptScrollPos.y;
			nScrollPos = 0;
			break;
		case SB_BOTTOM:
			// scroll to y where y = range of y
			nScrollBy = m_ptScrollPos.y - (nMax-nMin);
			nScrollPos = nMax - nMin;
			break;
		default:
			break;
	}

	// go to top range limit
	if(m_ptScrollPos.y-nScrollBy < 0)
	{
		nScrollBy = m_ptScrollPos.y;
		m_ptScrollPos.y = 0;
	}
	// go to bottom range limit
	else if(m_ptScrollPos.y-nScrollBy > (nMax-nMin) - m_sizePage.cy)
	{
		nScrollBy = m_ptScrollPos.y - (nMax-nMin) - m_sizePage.cy;
		m_ptScrollPos.y = (nMax-nMin) - m_sizePage.cy;
	}
	else
	{
		m_ptScrollPos.y = nScrollPos;
	}

	if(nScrollBy)
	{
		ScrollWindow(0, nScrollBy);
		SetScrollPos(SB_VERT, m_ptScrollPos.y);
		UpdateWindow();
	}

	return 0;
}

long CWindowEx::OnPaint(WPARAM, LPARAM)
{
	CDC dc = BeginPaint();
	int nXPos = 0;
	int nYPos = 0;
	
	if(GetStyle()&WS_HSCROLL)
		nXPos = -GetScrollPos(SB_HORZ);
	if(GetStyle()&WS_VSCROLL)
		nYPos = -GetScrollPos(SB_VERT);

	dc.SetViewportOrgEx(nXPos, nYPos);
	Paint(dc);
	
	EndPaint();
	
	return 0;
}


long CWindowEx::OnClose(WPARAM, LPARAM)
{
	Close();
	return 0;
}
//
//
//
//////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////
//
// CDialog
//
void CDialog::InitMsgHandlers()
{
	// initialize message handlers
	AddMsg(SWinMsg(WM_INITDIALOG, (CWMSGHANDLER)CDialog::OnInitDialog));
	AddMsg(SWinMsg(WM_COMPAREITEM, (CWMSGHANDLER)CDialog::OnCompareItem));
	AddMsg(SWinMsg(WM_DELETEITEM, (CWMSGHANDLER)CDialog::OnDeleteItem));
	AddMsg(SWinMsg(WM_DRAWITEM, (CWMSGHANDLER)CDialog::OnDrawItem));
	AddMsg(SWinMsg(WM_MEASUREITEM, (CWMSGHANDLER)CDialog::OnMeasureItem));



	// initialize command handlers
	AddCmd(SWinCmd(IDOK, (CWCMDHANDLER)CDialog::CmOk));
	AddCmd(SWinCmd(IDCANCEL, (CWCMDHANDLER)CDialog::CmCancel));
}

long CDialog::OnCompareItem(WPARAM wParam, LPARAM lParam)
{
	if(wParam)
		return SendDlgItemMessage(wParam, WM_COMPAREITEM, 0, lParam);
	else
		return 0;
}

long CDialog::OnDeleteItem(WPARAM wParam, LPARAM lParam)
{
	if(wParam)
		return SendDlgItemMessage(wParam, WM_DELETEITEM, 0, lParam);
	else
		return 0;
}

long CDialog::OnDrawItem(WPARAM wParam, LPARAM lParam)
{
	if(wParam)
		return SendDlgItemMessage(wParam, WM_DRAWITEM, 0, lParam);
	else
		return 0;
}

long CDialog::OnMeasureItem(WPARAM wParam, LPARAM lParam)
{
	if(wParam)
		return SendDlgItemMessage(wParam, WM_MEASUREITEM, 0, lParam);
	else
		return 0;
}

void CDialog::Destroy()
{
	if(m_oModal)
		::EndDialog(HWindow, m_nResult);
	else
		::DestroyWindow(HWindow);

	HWindow = 0;
}

bool CDialog::Close(int nResult)
{
	m_nResult = nResult;
	return CWindow::Close();
}

bool CDialog::Create()
{
	if(!oAdded)
	{
		::MessageBox(0, ct_szNotAdded, ct_szNotAddedTitle, MB_OK);
		return false;
	}
	m_oModal = false;
	oCreating = true;
	oDialogTranslate = true;
	HWND HParent = (pParent)?pParent->HWindow:GetDesktopWindow();
	HWindow = ::CreateDialog(_g_pApplication->HInstance, m_pszDlgID, HParent,
			(DLGPROC)CDialogProc);
	oCreating = false;
	
	return (HWindow == 0)?false:true;
}

int CDialog::Execute()
{
	if(!oAdded)
	{
		::MessageBox(0, ct_szNotAdded, ct_szNotAddedTitle, MB_OK);
		return false;
	}
	m_oModal = true;
	oCreating = true;
	HWND HParent = (pParent)?pParent->HWindow:0;
	return ::DialogBox(_g_pApplication->HInstance, m_pszDlgID, HParent,
			(DLGPROC)CDialogProc);
}

LRESULT CDialog::Default(unsigned int, WPARAM, LPARAM)
{
	return false;
}






//
//
//
//////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////
//
// CAppWindow
//
CAppWindow::CAppWindow(const TCHAR *pszIcon, const TCHAR *pszMenu)
{
	HICON HIcon = _g_pApplication->LoadIcon(pszIcon);
#ifdef __WINV40
	_g_pApplication->SetSmallIcon(HIcon);
#endif
	wsprintf(m_szClassName, _TEXT("CWApp_%4.4d"), _g_nCWAppClassID++);
	_g_pApplication->RegisterClass(m_szClassName, HIcon, pszMenu);

	SetDefStyle(WS_VISIBLE);
}

bool CAppWindow::Create(int x, int y, int nWidth, int nHeight, DWORD dwStyle)
{
	return CWindow::Create(m_szClassName, x, y, nWidth, nHeight, 0, dwStyle);
}
//
//
//
//////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////
//
// CApplication
//
CApplication::CApplication() : HInstance(0), HPrevInstance(0), m_pszCmdLine(_TEXT("")),
		m_nCmdShow(SW_SHOWNORMAL)
{
	_g_pApplication = this;
#ifdef __WINV40
	m_hSmallIcon = 0;
#endif
}

CApplication::~CApplication()
{
	int i;
	for(i=0;i<m_Windows.GetCount();i++)
	{
		// remove remaining windows
		if(m_Windows[i]->IsAutoDestroy())
			delete m_Windows[i];
		else
			m_Windows[i]->oAdded = false;
	}

	for(i=0;i<m_Classes.GetCount();i++)
		::UnregisterClass(m_Classes[i].szName, HInstance);
}

void CApplication::RemoveWindow(CWindow *pWindow)
{
	for(int i=0;i<m_Windows.GetCount();i++)
		if(pWindow == m_Windows[i])
		{
			m_Windows.Remove(i);
			pWindow->RestoreDefProc();
			break;
		}
}

int CApplication::main(HINSTANCE hInst, HINSTANCE hPrevInst,
		const TCHAR *pszCmdLine, int nCmdShow)
{
	HInstance = hInst;
	HPrevInstance = hPrevInst;
	m_pszCmdLine = pszCmdLine;
	m_nCmdShow = nCmdShow;

	GetModuleFileName(HInstance, m_szModuleName,
			sizeof(m_szModuleName)/sizeof(TCHAR));
	_tcscpy(m_szProfileName, m_szModuleName);
	_tcscpy(&m_szProfileName[_tcslen(m_szProfileName)-3], _TEXT("ini"));

	// register default window classes
	RegisterClass();
	RegisterClass(_TEXT("CWDblClkWin"), 0, 0, -1, 0,
			CS_DBLCLKS | CS_HREDRAW | CS_VREDRAW );

	int nRetVal = 0;
	if(Initialize())
		nRetVal = RunLoop();

	return nRetVal;
}

void CApplication::dllmain(HINSTANCE hInstDLL)
{
	HInstance = hInstDLL;
	HPrevInstance = 0;
	m_pszCmdLine = 0;
	m_nCmdShow = 0;

	GetModuleFileName(HInstance, m_szModuleName, sizeof(m_szModuleName)/sizeof(TCHAR));
	_tcscpy(m_szProfileName, m_szModuleName);
	_tcscpy(&m_szProfileName[_tcslen(m_szProfileName)-3], _TEXT("ini"));

	// register default window classes
	RegisterClass();
	RegisterClass(_TEXT("CWDblClkWin"), 0, 0, -1, 0, CS_DBLCLKS | CS_HREDRAW | CS_VREDRAW );

	Initialize();
}

LRESULT CApplication::Dispatch(HWND HWnd, UINT nMsg, WPARAM wParam,
		LPARAM lParam)
{
	long lRetVal;

	CWindow *pWindow = 0;
	for(int i=0;i<m_Windows.GetCount();i++)
	{
		// if gone, remove window and continue
		if(m_Windows[i]->IsDestroyed())
		{
			if(m_Windows[i]->IsAutoDestroy())
				delete m_Windows[i];
			else
				m_Windows[i]->oAdded = false;
			
			m_Windows.Remove(i--);
		}
		else
		{
			// allow other windows to preview message from queue
			m_Windows[i]->PreDispatch(HWnd, nMsg, wParam, lParam);

/*
			CWindow*	window = m_Windows[i];
			if (window->oCreating)
			{
				if ((nMsg == WM_INITDIALOG) || (nMsg == WM_CREATE) || (nMsg == WM_SETFONT))
				{
					window->HWindow = HWnd;
					pWindow = window;
					break;
				}
			}
*/
			if((pWindow = m_Windows[i]->FindWindow(HWnd)) != 0)
				break;
		}
	}

	// if window is found in chain, dispatch; otherwise,
	// call the default window procedure
	if(pWindow)
		lRetVal = pWindow->Dispatch(nMsg, wParam, lParam);
	else
		lRetVal = ::DefWindowProc(HWnd, nMsg, wParam, lParam);

	return lRetVal;
}

void CApplication::RegisterClass(const TCHAR *pszClassName, HICON HIcon,
		const TCHAR *pszMenu, int nBkgColor, HCURSOR HCursor, unsigned int nStyle)
{
	if(!HCursor)
		HCursor = ::LoadCursor(0, IDC_ARROW);

#ifdef __WINV40
	WNDCLASSEX WndClass;
#else
	WNDCLASS WndClass;
#endif

	memset(&WndClass, 0, sizeof(WndClass));

	WndClass.style = nStyle;
	WndClass.lpfnWndProc = (WNDPROC)CWindowProc;
	WndClass.hInstance = HInstance;
	WndClass.hIcon = HIcon;
	WndClass.hCursor = HCursor;
	WndClass.hbrBackground = (HBRUSH)(nBkgColor+1);
	WndClass.lpszMenuName = pszMenu;
	WndClass.lpszClassName = pszClassName;
	
#ifdef __WINV40
	WndClass.cbSize = sizeof(WndClass);
	WndClass.hIconSm = m_hSmallIcon;
	
	::RegisterClassEx(&WndClass);
#else
	::RegisterClass(&WndClass);
#endif

	SClassName ClassName;
	_tcscpy(ClassName.szName, pszClassName);
	m_Classes.Add(ClassName);
}

bool CApplication::IsRegisteredClass(const TCHAR *pszClassName)
{
	bool oRetVal = false;

	// check list to see if window is registered by framework
	for(int i=0;i<m_Classes.GetCount();i++)
		if(!_tcscmp(m_Classes[i].szName, pszClassName))
			oRetVal = true;

	return oRetVal;
}

bool CApplication::IsAccelerator(LPMSG pMsg)
{
	if(pMsg->message == WM_KEYDOWN || pMsg->message == WM_SYSKEYDOWN)
	{
		for(int i=0;i<m_Windows.GetCount();i++)
		{
			// make sure it is not an accelerator
			if(m_Windows[i]->HAccel)
			{
				int nEntryCnt = CopyAcceleratorTable(m_Windows[i]->HAccel, 0, 0);
				ACCEL *pEntries = new ACCEL[nEntryCnt];
				CopyAcceleratorTable(m_Windows[i]->HAccel, pEntries, nEntryCnt);

				// build key state flag
				int nFlags = 0;
				if(GetKeyState(VK_CONTROL)&0x8000)
					nFlags |= FCONTROL;
				if(GetKeyState(VK_SHIFT)&0x8000)
					nFlags |= FSHIFT;

				for(int p=0;p<nEntryCnt;p++)
				{
					if((pEntries[p].fVirt&(FSHIFT|FCONTROL)) == nFlags &&
							pEntries[p].key == pMsg->wParam)
					{
						// no need to go further, it is an accelerator
						delete [] pEntries;
						return true;
					}
				}

				delete [] pEntries;
			}
		}
	}

	return false;
}

bool CApplication::PreTranslate(LPMSG pMsg)
{
	bool oRetVal = true;
	SFindDialog Find;
	int i;

	do
	{
		for(i=0;i<m_Windows.GetCount();i++)
		{
			// make sure it is not an accelerator
			if(m_Windows[i]->HAccel && ::TranslateAccelerator(m_Windows[i]->HWindow,
					m_Windows[i]->HAccel, pMsg))
			{
				// no need to go further, it is an accelerator
				return true;
			}
			// if the window chain has the next dialog,
			// break from for(...) loop and process
			else if(m_Windows[i]->FindNextDialog(Find))
			{
				break;
			}
		}

		// if there are no more dialogs break and return
		if(i == m_Windows.GetCount())
		{
			oRetVal = false;
			break;
		}
		
	}while(!::IsDialogMessage(Find.hwnd, pMsg));

	return oRetVal;
}

int CApplication::RunLoop()
{
	MSG msg;
	while(::GetMessage(&msg, 0, 0, 0))
	{
		// if mouse is captured when ALT key or accelerator is pressed, release
		// to properly translate message
		if((msg.message == WM_SYSKEYDOWN || IsAccelerator(&msg)) && ::GetCapture())
			::ReleaseCapture();

		// pre-process accelerator and dialog messages
		if(!PreTranslate(&msg))
		{
			::TranslateMessage(&msg);
			::DispatchMessage(&msg);
		}

		// handlers for occasional processing when no messages are in queue
		if(m_IdleHandlers.GetCount())
		{
			while(!PeekMessage(&msg, 0, 0, 0, PM_NOREMOVE))
			{
				for(int i=0;i<m_IdleHandlers.GetCount();i++)
					(m_IdleHandlers[i].pWindow->*m_IdleHandlers[i].pHandler)();

				// sleep for multi-threaded environments, else just break
#ifdef __WINV40
				Sleep(ct_nIdleWait);
#else
				break;
#endif
			}
		}
	}

	return msg.wParam;
}
//
//
//
//////////////////////////////////////////////////////////////

/*******************************************************************
 *
 * $Log: /Deskwin/Src/CWindow.cpp $
 * 
 * 4     10/01/00 9:17 Pese
 * 
 * 3     5/01/00 20:26 Pese
 * 
 * 2     23/12/99 10:16 Pese
 *
 *******************************************************************/
