//---------------------------
// CW framework <cwindow.h>
//
// (Released under the terms
// of the GNU General Public
// License.)
//
// author:  Michael Hansen
// initiated: February 1998
//---------------------------

#ifndef _CWINDOW_H_
#define _CWINDOW_H_

// set up defines
#ifdef WIN32
#define __WINV40
#endif
#ifdef __WIN32__
#define __WIN32
#endif
#ifdef __WINV40
#define __WIN32
#endif

#include <windows.h>
#include <tchar.h>
#include "xvector.h"


const COLORREF ct_Black = RGB(0,0,0);
const COLORREF ct_DarkBlue = RGB(0,0,127);
const COLORREF ct_DarkGreen = RGB(0,127,0);
const COLORREF ct_DarkRed = RGB(127,0,0);
const COLORREF ct_DarkPurple = RGB(127,0,127);
const COLORREF ct_DarkAqua = RGB(0,127,127);
const COLORREF ct_DarkYellow = RGB(127,127,0);
const COLORREF ct_DarkGray = RGB(127,127,127);
const COLORREF ct_LightGray = RGB(192,192,192);
const COLORREF ct_LightBlue = RGB(0,0,255);
const COLORREF ct_LightGreen = RGB(0,255,0);
const COLORREF ct_LightRed = RGB(255,0,0);
const COLORREF ct_LightPurple = RGB(255,0,255);
const COLORREF ct_LightAqua = RGB(0,255,255);
const COLORREF ct_LightYellow = RGB(255,255,0);
const COLORREF ct_White = RGB(255,255,255);

class CApplication;

HINSTANCE GetAppInstance(void);
CApplication *GetApplication(void);

class CWindow;

class CEventInfo
{
private:
	int m_nType;
public:
	CEventInfo(int nType) : m_nType(nType){}
	int GetType(void) const {return m_nType;}
};
typedef int (CWindow::*CWEVENT)(CEventInfo *);

typedef void (CWindow::*CWIDLEHANDLER)(void);
typedef void (CWindow::*CWCMDHANDLER)(void);
typedef LRESULT (CWindow::*CWMSGHANDLER)(WPARAM, LPARAM);

struct SWinIdle
{
	CWindow *pWindow;
	CWIDLEHANDLER pHandler;
	SWinIdle(CWindow *window=0, CWCMDHANDLER handler=0)
			: pWindow(window), pHandler(handler){}
};

struct SWinCmd
{
	unsigned int nNotification;
	unsigned int nId;
	CWCMDHANDLER pHandler;
	SWinCmd(unsigned int id=0, CWCMDHANDLER handler=0, unsigned int notification=0)
			: nId(id), pHandler(handler), nNotification(notification){}
};

struct SWinMsg
{
	unsigned int nMessage;
	CWMSGHANDLER pHandler;
	SWinMsg(unsigned int message=0, CWMSGHANDLER handler=0)
			: nMessage(message), pHandler(handler){}
};

struct SToolTipInfo
{
	unsigned int iControlId;
	unsigned int iHelpTextId;
	SToolTipInfo(unsigned int aControlId=0, unsigned int aHelpTextId=0)
			: iControlId(aControlId), iHelpTextId(aHelpTextId){}
};


class CPopupMenu;

class CMenu
{
private:
	int *m_pnRefCnt;

	void Destroy(void)
	{
		if(m_pnRefCnt[0] == 0)
		{
			if(Handle && !oExternal) DestroyMenu(Handle);
			delete m_pnRefCnt;
		}
		else
			m_pnRefCnt[0]--;
	}
	void Copy(CMenu &rMenu)
	{
		rMenu.m_pnRefCnt[0]++;

		Handle = rMenu.Handle;
		oExternal = rMenu.oExternal;
		m_pnRefCnt = rMenu.m_pnRefCnt;
	}

protected:
	HMENU Handle;
	bool oExternal;

	virtual void Create(void){Handle = CreateMenu();}

public:
	// object must be created with constructor
	CMenu() : oExternal(false){m_pnRefCnt = new int(0); Create();}
	CMenu(HMENU handle, bool autodestroy=false) : Handle(handle),
			oExternal(!autodestroy){m_pnRefCnt = new int(0);}
	CMenu(int nID);
	CMenu(const TCHAR *pszMenu);
	CMenu(CMenu &rMenu){Copy(rMenu);}
	~CMenu(){Destroy();}
	operator HMENU() const {return Handle;}
	CMenu &operator=(CMenu &rMenu)
	{
		if(&rMenu != this)
		{
			Destroy();
			Copy(rMenu);
		}
		return *this;
	}
	BOOL AppendItem(int nItem, const TCHAR *pszName, int nFlags=MF_ENABLED | MF_STRING)
			{return ::AppendMenu(Handle, nFlags, nItem, pszName);}
	BOOL AppendItem(CPopupMenu &Popup, const TCHAR *pszName);
	BOOL CheckItem(int nItem, int nFlags=MF_BYCOMMAND)
			{return ::CheckMenuItem(Handle, nItem, nFlags | MF_CHECKED);}
	BOOL DeleteItem(int nItem, int nFlags=MF_BYCOMMAND)
			{return ::RemoveMenu(Handle, nItem, nFlags);}
	BOOL DisableItem(int nItem, int nFlags=MF_BYCOMMAND)
			{return ::EnableMenuItem(Handle, nItem, nFlags | MF_GRAYED);}
	BOOL EnableItem(int nItem, int nFlags=MF_BYCOMMAND)
			{return ::EnableMenuItem(Handle, nItem, nFlags | MF_ENABLED);}
	int GetItemCount(void){return ::GetMenuItemCount(Handle);}
	unsigned int GetItemID(int nPos){return ::GetMenuItemID(Handle, nPos);}
	int GetItemString(int nItem, TCHAR *pszBuffer, int nSize, int nFlags=MF_BYCOMMAND)
			{return ::GetMenuString(Handle, nItem, pszBuffer, nSize, nFlags);}
	CPopupMenu GetSubMenu(int nPos);
	BOOL InsertItem(int nAfterItem, int nItem, const TCHAR *pszName,
			int nFlags=MF_BYCOMMAND | MF_ENABLED | MF_STRING)
			{return ::InsertMenu(Handle, nAfterItem, nFlags, nItem, pszName);}
	BOOL InsertItem(int nAfterItem, CPopupMenu &Popup, const TCHAR *pszName,
			int nMethod=MF_BYCOMMAND);
	bool IsItemChecked(int nItem, int nFlags=MF_BYCOMMAND)
			{return (::GetMenuState(Handle, nItem, nFlags)&MF_CHECKED)?true:false;}
	bool IsItemEnabled(int nItem, int nFlags=MF_BYCOMMAND)
			{return (::GetMenuState(Handle, nItem, nFlags)&MF_ENABLED)?true:false;}
	BOOL ModifyItem(int nOldItem, int nItem, const TCHAR *pszName,
			int nFlags=MF_BYCOMMAND | MF_ENABLED | MF_STRING)
			{return ::ModifyMenu(Handle, nOldItem, nFlags, nItem, pszName);}
	BOOL UncheckItem(int nItem, int nFlags=MF_BYCOMMAND)
			{return ::CheckMenuItem(Handle, nItem, nFlags | MF_UNCHECKED);}
};

class CPopupMenu : public CMenu
{
protected:
	void Create(void){Handle = CreatePopupMenu();}

public:
	CPopupMenu(){}
	CPopupMenu(HMENU handle, bool autodestroy=false) : CMenu(handle, autodestroy){}
	CPopupMenu(CPopupMenu &rMenu) : CMenu(rMenu){}
	BOOL Track(int x, int y, HWND hwnd, int nFlags=TPM_LEFTALIGN | TPM_LEFTBUTTON,
			const RECT *pRect=0)
			{return ::TrackPopupMenu(Handle, nFlags, x, y, 0, hwnd, pRect);}
};

inline
BOOL CMenu::AppendItem(CPopupMenu &Popup, const TCHAR *pszName)
{
	return ::AppendMenu(Handle, MF_ENABLED | MF_POPUP, (unsigned int)Popup.Handle,
			pszName);
}

inline
BOOL CMenu::InsertItem(int nAfterItem, CPopupMenu &Popup, const TCHAR *pszName,
		int nMethod)
{
	return ::InsertMenu(Handle, nAfterItem, MF_ENABLED | MF_POPUP | nMethod,
			(unsigned int)Popup.Handle, pszName);
}

inline
CPopupMenu CMenu::GetSubMenu(int nPos)
{
	return CPopupMenu(::GetSubMenu(Handle, nPos));
}

class CPalette
{
private:
	int *m_pnRefCnt;

	void Destroy(void)
	{
		if(m_pnRefCnt[0] == 0)
		{
			if(Handle && !oExternal) DeleteObject(Handle);
			delete m_pnRefCnt;
		}
		else
			m_pnRefCnt[0]--;
	}
	void Copy(CPalette &rPalette)
	{
		rPalette.m_pnRefCnt[0]++;

		Handle = rPalette.Handle;
		oExternal = rPalette.oExternal;
		m_pnRefCnt = rPalette.m_pnRefCnt;
	}

protected:
	HPALETTE Handle;
	bool oExternal;

public:
	// palette from existing palette
	CPalette(HPALETTE handle, bool autodestroy=true) : Handle(handle),
			oExternal(!autodestroy){m_pnRefCnt = new int(0);}
	// palette from DIB
	CPalette(int nEntryCnt, RGBQUAD *pEntries);
	CPalette(CPalette &rPalette){Copy(rPalette);}
	~CPalette(){Destroy();}
	operator HPALETTE() const {return (HPALETTE)Handle;}
	CPalette &operator=(CPalette &rPalette)
	{
		if(&rPalette != this)
		{
			Destroy();
			Copy(rPalette);
		}
		return *this;
	}
};

class CGDIObject
{
private:
	int *m_pnRefCnt;

	void Destroy(void)
	{
		if(m_pnRefCnt[0] == 0)
		{
			if(Handle && !oExternal)
				DeleteObject(Handle);
			delete m_pnRefCnt;
		}
		else
			m_pnRefCnt[0]--;
	}
	void Copy(CGDIObject &rObject)
	{
		rObject.m_pnRefCnt[0]++;

		Handle = rObject.Handle;
		oExternal = rObject.oExternal;
		m_pnRefCnt = rObject.m_pnRefCnt;
	}

protected:
	HGDIOBJ Handle;
	bool oExternal;

public:
	// object must be created with constructor
	CGDIObject() : Handle(0), oExternal(true){m_pnRefCnt = new int(0);}
	CGDIObject(HGDIOBJ handle, bool autodestroy=true) : Handle(handle),
			oExternal(!autodestroy){m_pnRefCnt = new int(0);}
	CGDIObject(CGDIObject &rObject){Copy(rObject);}
	~CGDIObject(){Destroy();}
	operator HGDIOBJ() const {return (HGDIOBJ)Handle;}
	CGDIObject &operator=(CGDIObject &rObject)
	{
		if(&rObject != this)
		{
			Destroy();
			Copy(rObject);
		}
		return *this;
	}
};

class CPen : public CGDIObject
{
public:
	CPen(HPEN HPen, bool oDelete=false) : CGDIObject((HGDIOBJ)HPen, oDelete){}
	CPen(COLORREF Color, int nWidth=0, int nStyle=PS_SOLID)
		: CGDIObject((HGDIOBJ)CreatePen(nStyle, nWidth, Color)){}
	operator HPEN() const {return (HPEN)Handle;}
};

class CBrush : public CGDIObject
{
public:
	CBrush(HBRUSH HBrush, bool oDelete=false) : CGDIObject((HGDIOBJ)HBrush, oDelete){}
	CBrush(COLORREF Color) : CGDIObject((HGDIOBJ)CreateSolidBrush(Color)){}
	CBrush(int nStyle, COLORREF Color)
			: CGDIObject((HGDIOBJ)CreateHatchBrush(nStyle, Color)){}
	CBrush(HBITMAP HBitmap) : CGDIObject((HGDIOBJ)CreatePatternBrush(HBitmap)){}
	operator HBRUSH() const {return (HBRUSH)Handle;}
};

class CRegion : public CGDIObject
{
public:
	CRegion(HRGN HRegion, bool oDelete=false) : CGDIObject((HGDIOBJ)HRegion, oDelete){}
	CRegion(int left, int top, int right, int bottom)
		: CGDIObject((HGDIOBJ)CreateRectRgn(left, top, right, bottom)){}
	operator HRGN() const {return (HRGN)Handle;}
};

class CFont : public CGDIObject
{
public:
	CFont(HFONT HFont, bool oDelete=false) : CGDIObject((HGDIOBJ)HFont, oDelete){}
	CFont(int nHeight, int nWidth, int nEscapement, int nOrientation, int nWeight,
			DWORD dwItalic, DWORD dwUnderline, DWORD dwStrikeOut, DWORD dwCharSet,
			DWORD dwOutputPrecision, DWORD dwClipPrecision, DWORD dwQuality,
			DWORD dwPitchAndFamily, const TCHAR *pszFace)
			: CGDIObject((HGDIOBJ)CreateFont(nHeight, nWidth, nEscapement, nOrientation,
			nWeight, dwItalic, dwUnderline, dwStrikeOut, dwCharSet, dwOutputPrecision,
			dwClipPrecision, dwQuality, dwPitchAndFamily, pszFace)){}
	// simpler and more practical forms (uses common defaults)
	CFont(int nHeight, int nOrientation=0, int nWeight=FW_NORMAL, DWORD dwItalic=0,
			DWORD dwUnderline=0, DWORD dwStrikeOut=0, DWORD dwPitchAndFamily=
			VARIABLE_PITCH | FF_SWISS, const TCHAR *pszFace=0)
			: CGDIObject((HGDIOBJ)CreateFont(nHeight, 0, nOrientation, nOrientation,
			nWeight, dwItalic, dwUnderline, dwStrikeOut, ANSI_CHARSET, OUT_DEFAULT_PRECIS,
			CLIP_DEFAULT_PRECIS | CLIP_LH_ANGLES, DEFAULT_QUALITY, dwPitchAndFamily, pszFace)){}
	CFont(const TCHAR *pszFace, int nHeight, int nOrientation=0, int nWeight=FW_NORMAL,
			DWORD dwItalic=0, DWORD dwUnderline=0, DWORD dwStrikeOut=0,
			DWORD dwPitchAndFamily=VARIABLE_PITCH | FF_SWISS)
			: CGDIObject((HGDIOBJ)CreateFont(nHeight, 0, nOrientation, nOrientation,
			nWeight, dwItalic, dwUnderline, dwStrikeOut, ANSI_CHARSET, OUT_DEFAULT_PRECIS,
			CLIP_DEFAULT_PRECIS | CLIP_LH_ANGLES, DEFAULT_QUALITY, dwPitchAndFamily, pszFace)){}
	operator HFONT() const {return (HFONT)Handle;}
};

class CBitmap : public CGDIObject
{
protected:
	BITMAP bmInfo;

	void GetInfo(void){if(Handle) GetObject(Handle, sizeof(BITMAP), &bmInfo);}

public:
	CBitmap(HBITMAP HBitmap, bool oDelete=false) : CGDIObject((HGDIOBJ)HBitmap, oDelete)
			{GetInfo();}
	// monochrome bitmap created in memory
	CBitmap(int nWidth, int nHeight, const void *pvBits)
			: CGDIObject((HGDIOBJ)CreateBitmap(nWidth, nHeight, 1, 1, pvBits))
			{GetInfo();}
	CBitmap(int nWidth, int nHeight, int bpp, const void *pvBits)
			: CGDIObject((HGDIOBJ)CreateBitmap(nWidth, nHeight, 1, bpp, pvBits))
			{GetInfo();}
	// bitmap from resource
	CBitmap(int nID)
			: CGDIObject((HGDIOBJ)LoadBitmap(GetAppInstance(), MAKEINTRESOURCE(nID)))
			{GetInfo();}
	CBitmap(const TCHAR *pszID)
			: CGDIObject((HGDIOBJ)LoadBitmap(GetAppInstance(), pszID)){GetInfo();}
	CBitmap(CBitmap &rBitmap) : CGDIObject(rBitmap){GetInfo();}

	int GetWidth(void){return bmInfo.bmWidth;}
	int GetHeight(void){return bmInfo.bmHeight;}
	int GetColorCount(void){return bmInfo.bmPlanes*bmInfo.bmBitsPixel;}
	void *GetBits(void){return bmInfo.bmBits;}

	operator HBITMAP() const {return (HBITMAP)Handle;}
	CBitmap &operator=(CBitmap &rBitmap)
	{
		if(&rBitmap != this)
		{
			CGDIObject::operator=(rBitmap);
			GetInfo();
		}
		return *this;
	}
};

class CDIB
{
private:
	int *m_pnRefCnt;

	void Destroy(void)
	{
		if(m_pnRefCnt[0] == 0)
		{
			if(Resource)
			{
				UnlockResource(Resource);
				FreeResource(Resource);
			}
			delete m_pnRefCnt;
		}
		else
			m_pnRefCnt[0]--;
	}
	void Copy(CDIB &rDIB)
	{
		rDIB.m_pnRefCnt[0]++;

		Resource = rDIB.Resource;
		nPaletteCount = rDIB.nPaletteCount;
		pInfo = rDIB.pInfo;
		pbBits = rDIB.pbBits;
		m_pnRefCnt = rDIB.m_pnRefCnt;
	}

protected:
	HANDLE Resource;
	int nPaletteCount;

	void GetInfoFromMemory(void);

public:
	BITMAPINFO *pInfo;
	void *pbBits;
	
	// null bitmap
	CDIB() : Resource(0), nPaletteCount(0), pInfo(0), pbBits(0)
			{m_pnRefCnt = new int(0);}
	// bitmap from memory
	CDIB(void *pMemory)
	{
		m_pnRefCnt = new int(0);
		Resource = 0;
		pInfo = (BITMAPINFO *)pMemory;
		GetInfoFromMemory();
	}
	// bitmap from resource
	CDIB(int nID)
	{
		m_pnRefCnt = new int(0);
		Resource = LoadResource(GetAppInstance(),
				FindResource(GetAppInstance(), MAKEINTRESOURCE(nID), RT_BITMAP));
		pInfo = (BITMAPINFO *)LockResource(Resource);
		GetInfoFromMemory();
	}
	CDIB(const TCHAR *pszID)
	{
		m_pnRefCnt = new int(0);
		Resource = LoadResource(GetAppInstance(),
				FindResource(GetAppInstance(), pszID, RT_BITMAP));
		pInfo = (BITMAPINFO *)LockResource(Resource);
		GetInfoFromMemory();
	}
	CDIB(CDIB &rDIB){Copy(rDIB);}
	~CDIB(){Destroy();}

	int GetWidth(void){return (pInfo!=0)?pInfo->bmiHeader.biWidth:0;}
	int GetHeight(void){return (pInfo!=0)?((pInfo->bmiHeader.biHeight>0)?
			pInfo->bmiHeader.biHeight:-pInfo->bmiHeader.biHeight):0;}
	int GetColorCount(void){return (pInfo!=0)?pInfo->bmiHeader.biBitCount:0;}
	int GetPaletteCount(void){return nPaletteCount;}
	RGBQUAD *GetPalette(void){return (pInfo!=0)?pInfo->bmiColors:0;}

	CDIB &operator=(CDIB &rDIB)
	{
		if(&rDIB != this)
		{
			Destroy();
			Copy(rDIB);
		}
		return *this;
	}
};

class CDC
{
private:
	XList <CGDIObject> m_Objects;

protected:
	HDC Handle;

public:
	CDC(HDC hdc) : Handle(hdc){}
	~CDC(){RestoreObjects();}
	operator HDC() const {return Handle;}

	BOOL Arc(int nLeftRect, int nTopRect, int nRightRect, int nBottomRect,
			int nXStartArc, int nYStartArc, int nXEndArc, int nYEndArc)
			{return ::Arc(Handle, nLeftRect, nTopRect, nRightRect, nBottomRect,
			nXStartArc, nYStartArc, nXEndArc, nYEndArc);}
	BOOL BitBlt(int nX, int nY, int nWidth, int nHeight, CDC &dc,
			int nXSrc, int nYSrc, DWORD dwROP=SRCCOPY)
			{return ::BitBlt(Handle, nX, nY, nWidth, nHeight, dc, nXSrc, nYSrc,
			dwROP);}
	BOOL Chord(int nLeftRect, int nTopRect, int nRightRect, int nBottomRect,
			int nXStartLine, int nYStartLine, int nXEndLine, int nYEndLine)
			{return ::Chord(Handle, nLeftRect, nTopRect, nRightRect, nBottomRect,
			nXStartLine, nYStartLine, nXEndLine, nYEndLine);}
	HBITMAP CreateCompatibleBitmap(int nWidth, int nHeight)
			{return ::CreateCompatibleBitmap(Handle, nWidth, nHeight);}
	HBITMAP CreateDIBitmap(const BITMAPINFOHEADER *pBmpInfoHdr, const void *pvBits,
			BITMAPINFO *pBmpInfo, unsigned int nColorUsage)
			{return ::CreateDIBitmap(Handle, pBmpInfoHdr, CBM_INIT, pvBits, pBmpInfo,
			nColorUsage=DIB_RGB_COLORS);}
#ifdef __WINV40
	HBITMAP CreateDIBSection(const BITMAPINFO *pBmpInfo, unsigned int nUsage,
			void **pvBits, HANDLE HSection=0, DWORD dwOffset=0)
			{return ::CreateDIBSection(Handle, pBmpInfo, nUsage, pvBits, HSection,
			dwOffset);}
#endif
	BOOL DPtoLP(POINT *pPoints, int nPointCnt)
			{return ::DPtoLP(Handle, pPoints, nPointCnt);}
#ifdef __WINV40
	BOOL DrawEdge(RECT *pRect, unsigned int nEdge, unsigned int nFlags)
			{return ::DrawEdge(Handle, pRect, nEdge, nFlags);}
#endif
	BOOL DrawFocusRect(const RECT *pRect){return ::DrawFocusRect(Handle, pRect);}
#ifdef __WINV40
	BOOL DrawFrameControl(RECT *pRect, unsigned int nType, unsigned int nState)
			{return ::DrawFrameControl(Handle, pRect, nType, nState);}
#endif
	BOOL DrawIcon(int nX, int nY, HICON HIcon)
			{return ::DrawIcon(Handle, nX, nY, HIcon);}
#ifdef __WINV40
	BOOL DrawIconEx(int nX, int nY, HICON HIcon, int nWidth, int nHeight,
			unsigned int nFlags)
			{return ::DrawIconEx(Handle, nX, nY, HIcon, nWidth, nHeight, 0, 0,
			nFlags);}
#endif
	int DrawText(const TCHAR *pszString, int nCount, RECT *pRect,
			unsigned int nFormat)
			{return ::DrawText(Handle, pszString, nCount, pRect, nFormat);}
	void DrawDisableBitmap(HBITMAP hBitmap, int nX, int nY,
			COLORREF DisableColor=GetSysColor(COLOR_BTNSHADOW),
			COLORREF DisableHilite=GetSysColor(COLOR_BTNHIGHLIGHT));
	void DrawTransparentBitmap(HBITMAP hBitmap, int nX, int nY);
	BOOL Ellipse(int left, int top, int right, int bottom)
			{return ::Ellipse(Handle, left, top, right, bottom);}
	int ExcludeClipRect(int left, int top, int right, int bottom)
			{return ::ExcludeClipRect(Handle, left, top, right, bottom);}
	BOOL ExtTextOut(int nX, int nY, unsigned int nOptions, const RECT *pRect,
			const TCHAR *pszString, unsigned int nCount, const int *pnDx=0)
			{return ::ExtTextOut(Handle, nX, nY, nOptions, pRect, pszString,
			nCount, pnDx);}
	int FillRect(const RECT *pRect, HBRUSH Brush)
			{return ::FillRect(Handle, pRect, Brush);}
	BOOL FillRgn(HRGN HRegion, HBRUSH Brush)
			{return ::FillRgn(Handle, HRegion, Brush);}
	int FrameRect(const RECT *pRect, HBRUSH Brush)
			{return ::FrameRect(Handle, pRect, Brush);}
	BOOL FrameRgn(HRGN HRegion, HBRUSH Brush, int nWidth, int nHeight)
			{return ::FrameRgn(Handle, HRegion, Brush, nWidth, nHeight);}
	BOOL GetAspectRatioFilterEx(SIZE *pSize)
			{return ::GetAspectRatioFilterEx(Handle, pSize);}
	COLORREF GetBkColor(void){return ::GetBkColor(Handle);}
	int GetBkMode(void){return ::GetBkMode(Handle);}
	unsigned int GetBoundsRect(RECT *pRect, unsigned int nFlags)
			{return ::GetBoundsRect(Handle, pRect, nFlags);}
	BOOL GetBrushOrgEx(POINT *pPoint){return ::GetBrushOrgEx(Handle, pPoint);}
	BOOL GetCharABCWidths(unsigned int nFirstChar, unsigned int nLastChar,
			ABC *pABCList)
			{return ::GetCharABCWidths(Handle, nFirstChar, nLastChar, pABCList);}
	BOOL GetCharWidth(unsigned int nFirstChar, unsigned int nLastChar, int *pWidths)
			{return ::GetCharWidth(Handle, nFirstChar, nLastChar, pWidths);}
	int GetClipBox(RECT *pRect){return ::GetClipBox(Handle, pRect);}
#ifdef __WINV40
	int GetClipRgn(HRGN HRegion){return ::GetClipRgn(Handle, HRegion);}
#endif
	HGDIOBJ GetCurrentObject(unsigned int nType)
			{return ::GetCurrentObject(Handle, nType);}
	BOOL GetCurrentPositionEx(POINT *pPoint)
			{return ::GetCurrentPositionEx(Handle, pPoint);}
#ifdef __WIN16
	DWORD GetDCOrg(void){return ::GetDCOrg(Handle);}
#else
	BOOL GetDCOrgEx(POINT *pPoint){return ::GetDCOrgEx(Handle, pPoint);}
#endif
	int GetDeviceCaps(int nIndex){return ::GetDeviceCaps(Handle, nIndex);}
	int GetDIBits(HBITMAP hBitmap, unsigned int nStartScan, unsigned int nScanLines,
			void *pvBits, BITMAPINFO *pBmpInfo, unsigned int nColorUsage)
			{return ::GetDIBits(Handle, hBitmap, nStartScan, nScanLines, pvBits,
			pBmpInfo, nColorUsage);}
	int GetMapMode(void){return ::GetMapMode(Handle);}
	COLORREF GetNearestColor(COLORREF Color)
			{return ::GetNearestColor(Handle, Color);}
	COLORREF GetPixel(int nXPos, int nYPos)
			{return ::GetPixel(Handle, nXPos, nYPos);}
	int GetPolyFillMode(void){return ::GetPolyFillMode(Handle);}
	int GetROP2(void){return ::GetROP2(Handle);}
	int GetStretchBltMode(void){return ::GetStretchBltMode(Handle);}
	unsigned int GetSystemPaletteEntries(unsigned int nStart,
			unsigned int nEntries, PALETTEENTRY *pEntries)
			{return ::GetSystemPaletteEntries(Handle, nStart, nEntries, pEntries);}
	unsigned int GetTextAlign(void){return ::GetTextAlign(Handle);}
	int GetTextCharacterExtra(void){return ::GetTextCharacterExtra(Handle);}
	COLORREF GetTextColor(void){return ::GetTextColor(Handle);}
#ifdef __WIN32
	BOOL GetTextExtentExPoint(const TCHAR *pszString, int nCount, int nMaxExtent,
			int *pnFit, int *pnDXs, SIZE *pSize)
			{return ::GetTextExtentExPoint(Handle, pszString, nCount, nMaxExtent,
			pnFit, pnDXs, pSize);}
#endif
	BOOL GetTextExtentPoint(const TCHAR *pszString, int nCount, SIZE *pSize)
			{return ::GetTextExtentPoint(Handle, pszString, nCount, pSize);}
	int GetTextFace(int nCount, TCHAR *pszFaceName)
			{return ::GetTextFace(Handle, nCount, pszFaceName);}
	BOOL GetTextMetrics(TEXTMETRIC *pTM){return ::GetTextMetrics(Handle, pTM);}
	BOOL GetViewportExtEx(SIZE *pSize){return ::GetViewportExtEx(Handle, pSize);}
	BOOL GetViewportOrgEx(POINT *pPoint){return ::GetViewportOrgEx(Handle, pPoint);}
	BOOL GetWindowExtEx(SIZE *pSize){return ::GetWindowExtEx(Handle, pSize);}
	BOOL GetWindowOrgEx(POINT *pPoint){return ::GetWindowOrgEx(Handle, pPoint);}
	BOOL GrayString(HBRUSH Brush, const TCHAR *pszString, int nStrLen, int nX,
			int nY, int nWidth, int nHeight)
			{return ::GrayString(Handle, Brush, 0, (long)pszString, nStrLen, nX,
			nY, nWidth, nHeight);}
	int IntersectClipRect(int left, int top, int right, int bottom)
			{return ::IntersectClipRect(Handle, left, top, right, bottom);}
	BOOL InvertRgn(HRGN HRegion){return ::InvertRgn(Handle, HRegion);}
	void InvertRect(const RECT *pRect){::InvertRect(Handle, pRect);}
	BOOL LineTo(int nX, int nY){return ::LineTo(Handle, nX, nY);}
	BOOL LPtoDP(POINT *pPoints, int nPointCnt)
			{return ::LPtoDP(Handle, pPoints, nPointCnt);}
	BOOL MoveTo(int nX, int nY)
			{POINT pt; return ::MoveToEx(Handle, nX, nY, &pt);}
	BOOL MoveToEx(int nX, int nY, POINT *pPoint)
			{return ::MoveToEx(Handle, nX, nY, pPoint);}
#ifdef __WINNT
	BOOL MaskBlt(int nXDest, int nYDest, int nWidth, int nHeight, CDC &dc,
			int nXSrc, int nYSrc, HBITMAP hbmMask, int nXMask, int nYMask, DWORD
			dwROP){return ::MaskBlt(Handle, nXDest, nYDest, nWidth, nHeight,
			dc, nXSrc, nYSrc, hbmMask, nXMask, nYMask, dwROP);}
#endif
	int OffsetClipRgn(int nXOffset, int nYOffset)
			{return ::OffsetClipRgn(Handle, nXOffset, nYOffset);}
	BOOL OffsetViewportOrgEx(int nX, int nY, POINT *pPoint)
			{return ::OffsetViewportOrgEx(Handle, nX, nY, pPoint);}
	BOOL OffsetWindowOrgEx(int nX, int nY, POINT *pPoint)
			{return ::OffsetWindowOrgEx(Handle, nX, nY, pPoint);}
	BOOL PaintRgn(HRGN HRegion){return ::PaintRgn(Handle, HRegion);}
	BOOL PatBlt(int nX, int nY, int nWidth, int nHeight, DWORD dwROP)
			{return ::PatBlt(Handle, nX, nY, nWidth, nHeight, dwROP);}
	BOOL Pie(int nLeftRect, int nTopRect, int nRightRect, int nBottomRect,
			int nXStartArc, int nYStartArc, int nXEndArc, int nYEndArc)
			{return ::Pie(Handle, nLeftRect, nTopRect, nRightRect, nBottomRect,
			nXStartArc, nYStartArc, nXEndArc, nYEndArc);}
#ifdef __WINV40
	BOOL PolyBezier(const POINT *pPoints, int nCount)
			{return ::PolyBezier(Handle, pPoints, nCount);}
	BOOL PolyBezierTo(const POINT *pPoints, int nCount)
			{return ::PolyBezierTo(Handle, pPoints, nCount);}
#endif
#ifdef __WINNT
	BOOL PolyDraw(const POINT *pPoints, const BYTE *pTypes, int nCount)
			{return ::PolyDraw(Handle, pPoints, pTypes, nCount);}
#endif
	BOOL Polygon(const POINT *pPoints, int nCount)
			{return ::Polygon(Handle, pPoints, nCount);}
	BOOL Polyline(const POINT *pPoints, int nCount)
			{return ::Polyline(Handle, pPoints, nCount);}
#ifdef __WINNT
	BOOL PolylineTo(const POINT *pPoints, int nCount)
			{return ::PolylineTo(Handle, pPoints, nCount);}
#endif
	BOOL PtVisible(int nXPos, int nYPos){return ::PtVisible(Handle, nXPos, nYPos);}
	BOOL Rectangle(int nLeft, int nTop, int nRight, int nBottom)
			{return ::Rectangle(Handle, nLeft, nTop, nRight, nBottom);}
	BOOL RectVisible(const RECT *pRect){return ::RectVisible(Handle, pRect);}
	unsigned int RealizePalette(void){return ::RealizePalette(Handle);}
	BOOL RestoreDC(int nSaved){return ::RestoreDC(Handle, nSaved);}
	void RestoreObjects(void);
	BOOL RoundRect(int nLeft, int nTop, int nRight, int nBottom, int nW, int nH)
			{return ::RoundRect(Handle, nLeft, nTop, nRight, nBottom, nW, nH);}
	int SaveDC(void){return ::SaveDC(Handle);}
	BOOL ScaleViewportExtEx(int nXNum, int nXDenom, int nYNum, int nYDenom,
			SIZE *pSize)
			{return ::ScaleViewportExtEx(Handle, nXNum, nXDenom, nYNum, nYDenom,
			pSize);}
	BOOL ScaleWindowExtEx(int nXNum, int nXDenom, int nYNum, int nYDenom,
			SIZE *pSize)
			{return ::ScaleWindowExtEx(Handle, nXNum, nXDenom, nYNum, nYDenom,
			pSize);}
	BOOL ScrollDC(int nDX, int nDY, const RECT *prcScroll, const RECT *prcClip,
			HRGN hrgnUpdate, RECT *prcUpdate){return ::ScrollDC(Handle, nDX, nDY,
			prcScroll, prcClip, hrgnUpdate, prcUpdate);}
	int SelectClipRgn(HRGN HRegion){return ::SelectClipRgn(Handle, HRegion);}
	void SelectObject(CGDIObject &Object);
	HPALETTE SelectPalette(HPALETTE HPalette, BOOL bForceBkgnd=FALSE)
			{return ::SelectPalette(Handle, HPalette, bForceBkgnd);}
	COLORREF SetBkColor(COLORREF Color){return ::SetBkColor(Handle, Color);}
	int SetBkMode(int nMode){return ::SetBkMode(Handle, nMode);}
#ifdef __WIN16
	DWORD SetBrushOrg(int nXOrg, int nYOrg)
			{return ::SetBrushOrg(Handle, nXOrg, nYOrg);}
#else
	BOOL SetBrushOrgEx(int nXOrg, int nYOrg, POINT *pPoint)
			{return ::SetBrushOrgEx(Handle, nXOrg, nYOrg, pPoint);}
#endif
	int SetDIBits(HBITMAP hBitmap, unsigned int nStartScan, unsigned int nScanLines,
			void *pvBits, BITMAPINFO *pBmpInfo, unsigned int nColorUsage)
			{return ::SetDIBits(Handle, hBitmap, nStartScan, nScanLines, pvBits,
			pBmpInfo, nColorUsage);}
	int SetDIBitsToDevice(int nX, int nY, int nWidth, int nHeight, int nXSrc,
			int nYSrc, unsigned int nStartScan, unsigned int nScanLines,
			void *pvBits, BITMAPINFO *pBmpInfo, unsigned int nColorUsage)
			{return ::SetDIBitsToDevice(Handle, nX, nY, nWidth, nHeight, nXSrc,
			nYSrc, nStartScan, nScanLines, pvBits, pBmpInfo, nColorUsage);}
	int SetMapMode(int nMode){return ::SetMapMode(Handle, nMode);}
	COLORREF SetPixel(int nX, int nY, COLORREF Color)
			{return ::SetPixel(Handle, nX, nY, Color);}
	int SetPolyFillMode(int nMode){return ::SetPolyFillMode(Handle, nMode);}
	int SetROP2(int nMode){return ::SetROP2(Handle, nMode);}
	int SetStretchBltMode(int nMode){return ::SetStretchBltMode(Handle, nMode);}
	unsigned int SetTextAlign(unsigned int nAlign)
			{return ::SetTextAlign(Handle, nAlign);}
	int SetTextCharacterExtra(int nExtraSpace)
			{return ::SetTextCharacterExtra(Handle, nExtraSpace);}
	COLORREF SetTextColor(COLORREF Color){return ::SetTextColor(Handle, Color);}
	int SetTextJustification(int nExtraSpace, int nBreakCharCnt)
			{return ::SetTextJustification(Handle, nExtraSpace, nBreakCharCnt);}
	BOOL SetViewportExtEx(int nX, int nY, SIZE *pSize=0)
			{return ::SetViewportExtEx(Handle, nX, nY, pSize);}
	BOOL SetViewportOrgEx(int nXOrg, int nYOrg, POINT *pPoint=0)
			{return ::SetViewportOrgEx(Handle, nXOrg, nYOrg, pPoint);}
	BOOL SetWindowExtEx(int nX, int nY, SIZE *pSize=0)
			{return ::SetWindowExtEx(Handle, nX, nY, pSize);}
	BOOL SetWindowOrgEx(int nXOrg, int nYOrg, POINT *pPoint=0)
			{return ::SetWindowOrgEx(Handle, nXOrg, nYOrg, pPoint);}
	BOOL StretchBlt(int nX, int nY, int nWidth, int nHeight, CDC &dc,
			int nXSrc, int nYSrc, int nWSrc, int nHSrc, DWORD dwROP=SRCCOPY)
			{return ::StretchBlt(Handle, nX, nY, nWidth, nHeight, dc,
			nXSrc, nYSrc, nWSrc, nHSrc, dwROP);}
	int StretchDIBits(int nX, int nY, int nWidth, int nHeight, int nXSrc,
			int nYSrc, int nWidthSrc, int nHeightSrc, void *pvBits,
			BITMAPINFO *pBmpInfo, unsigned int nColorUsage=DIB_RGB_COLORS,
			DWORD dwROP=SRCCOPY)
			{return ::StretchDIBits(Handle, nX, nY, nWidth, nHeight, nXSrc,
			nYSrc, nWidthSrc, nHeightSrc, pvBits, pBmpInfo, nColorUsage, dwROP);}
	BOOL TextOut(int nXStart, int nYStart, const TCHAR *pszString, int nStrLen)
			{return ::TextOut(Handle, nXStart, nYStart, pszString, nStrLen);}
	int UpdateColors(void){return ::UpdateColors(Handle);}
};

class CMemoryDC : public CDC
{
private:
	HBITMAP m_hOrigBitmap;

public:
	CMemoryDC() : CDC(::CreateCompatibleDC(0)), m_hOrigBitmap(0){}
	CMemoryDC(HDC hdc) : CDC(::CreateCompatibleDC(hdc)), m_hOrigBitmap(0){}
	~CMemoryDC(){RestoreBitmap(); ::DeleteDC(Handle);}
	void RestoreBitmap(void)
			{if(m_hOrigBitmap) SelectBitmap(m_hOrigBitmap); m_hOrigBitmap = 0;}
	void SelectBitmap(HBITMAP hBitmap)
	{
		HBITMAP hOrigBitmap = (HBITMAP)::SelectObject(Handle, (HGDIOBJ)hBitmap);
		if(!m_hOrigBitmap) m_hOrigBitmap = hOrigBitmap;
	}
};

struct SFindDialog
{
	HWND hwnd;
	int nIndex;
	int nLevel;
	SFindDialog() : nIndex(0), nLevel(0){}
};

class CWindow
{
private:
	XList <CWindow *> m_Windows;
	XList <SWinCmd> m_CmdHandlers;
	XList <SWinMsg> m_MsgHandlers;
	XList <unsigned int> m_MsgParentForwards;

	bool m_oRemove;
	TCHAR *m_pszWindowText;
	WNDPROC m_pDefaultProc;
	DWORD m_dwDefStyle;
	bool m_oAutoDestroy;

	void PreDispatch(HWND &HWnd, UINT &nMsg, WPARAM &wParam, LPARAM &lParam);

protected:
	CFont Font;
	bool oCreating;
	bool oDialogTranslate;
	bool oAdded;
	PAINTSTRUCT m_PaintStruct;

	void SizeToClient(SIZE &Size);
	virtual void SpyMessageHook(HWND &HWnd, UINT &nMsg, WPARAM &wParam, LPARAM &lParam){}

	// remove window from parent list
	LRESULT OnNCDestroy(WPARAM wParam, LPARAM lParam);
	LRESULT OnWmHelp(WPARAM wParam, LPARAM lParam){OnHelp((LPHELPINFO)lParam);return TRUE;}
	virtual void OnHelp(LPHELPINFO aHelpInfo){};
	LRESULT OnWmNotify(WPARAM wParam, LPARAM lParam){NMHDR* nmhdr = (NMHDR*)lParam; OnNotify(nmhdr->hwndFrom, nmhdr->idFrom, nmhdr->code, wParam, lParam);return TRUE;}
	virtual void OnNotify(HWND hwndFrom, UINT idFrom, UINT code, WPARAM wParam, LPARAM lParam){};

public:
	bool oClient;
	bool oDefDblClks;
	CWindow *pParent;
	HWND HWindow;
	HACCEL HAccel;

	// create unattached and empty window object
	CWindow() : HWindow(0), HAccel(0), oClient(false), oDefDblClks(false), oAdded(false), pParent(0),
			Font(0), oCreating(false), oDialogTranslate(false), m_oRemove(false), m_pszWindowText(0),
			m_pDefaultProc(0), m_dwDefStyle(WS_CHILD | WS_VISIBLE), m_oAutoDestroy(false)
			{
				AddMsg(SWinMsg(WM_HELP, (CWMSGHANDLER)CWindow::OnWmHelp));
			}
	// attach to existing window and hook WindowProc()
	CWindow(HWND HWnd);
	virtual ~CWindow();

	void AddChild(CWindow *pWindow)
			{m_Windows.Add(pWindow); pWindow->oAdded = true; pWindow->pParent = this;}
	void RemoveChild(CWindow *pWindow);
	void AddCmd(SWinCmd &CmdHandler);
	void RemoveCmd(unsigned int nId, unsigned int nNotification=0);
	void AddMsg(SWinMsg &MsgHandler);
	void RemoveMsg(unsigned int nMessage);
	void AddMsgForward(unsigned int nMessage);
	void RemoveMsgForward(unsigned int nMessage);
	bool FindNextDialog(SFindDialog &Find);
	bool IsAutoDestroy(void){return m_oAutoDestroy;}
	void SetAutoDestroy(bool oAuto=true){m_oAutoDestroy = oAuto;}
	void RestoreDefProc(void)
			{if(m_pDefaultProc) SetWindowLong(GWL_WNDPROC, (long)m_pDefaultProc);}
	
	bool Create(const TCHAR *pszClass, int x, int y, int nWidth, int nHeight,
			int nID=0, DWORD dwStyle=0, LPVOID lpParam=0, DWORD dwExStyle=0);
	virtual void Destroy(void){::DestroyWindow(HWindow); HWindow = 0;}
	virtual bool CanClose(void){return true;}
	bool Close(void);
	virtual void OnQueryEndSession(){}
	int GetText(TCHAR *pszText, int nMaxCount);
	int GetTextLength(void)
			{return SendMessage(WM_GETTEXTLENGTH);}
	void SetText(const TCHAR *pszText);
	void SetDefStyle(DWORD dwDefStyle){m_dwDefStyle = dwDefStyle;}
	DWORD GetStyle(void){return GetWindowLong(GWL_STYLE);}

	// API calls
	CDC BeginPaint(){return CDC(::BeginPaint(HWindow, &m_PaintStruct));}
	BOOL BringWindowToTop(void){return ::BringWindowToTop(HWindow);}
	BOOL CheckDlgButton(int nID, unsigned int nCheck)
			{return ::CheckDlgButton(HWindow, nID, nCheck);}
	BOOL CheckRadioButton(int nIDFirst, int nIDLast, int nID)
			{return ::CheckRadioButton(HWindow, nIDFirst, nIDLast, nID);}
	HWND ChildWindowFromPoint(POINT pt)
			{return ::ChildWindowFromPoint(HWindow, pt);}
	BOOL ClientToScreen(POINT *pt){return ::ClientToScreen(HWindow, pt);}
	BOOL CreateCaret(HBITMAP HBitmap, int nWidth, int nHeight)
			{return ::CreateCaret(HWindow, HBitmap, nWidth, nHeight);}
	LRESULT DefWindowProc(UINT nMsg, WPARAM wParam, LPARAM lParam)
			{return ::DefWindowProc(HWindow, nMsg, wParam, lParam);}
	BOOL DrawMenuBar(void){return ::DrawMenuBar(HWindow);}
	BOOL EnableWindow(bool oEnable){return ::EnableWindow(HWindow, oEnable);}
	BOOL EndPaint(void){return ::EndPaint(HWindow, &m_PaintStruct);}
	DWORD GetClassLong(int nIndex){return ::GetClassLong(HWindow, nIndex);}
	int GetClassName(TCHAR *pszClassName, int nMaxCount)
			{return ::GetClassName(HWindow, pszClassName, nMaxCount);}
	BOOL GetClientRect(RECT *pRect){return ::GetClientRect(HWindow, pRect);}
	RECT GetClientRect(void){RECT rc; ::GetClientRect(HWindow, &rc); return rc;}
	HDC GetDC(void){return ::GetDC(HWindow);}
	int GetDlgCtrlID(void){return ::GetDlgCtrlID(HWindow);}
	HWND GetDlgItem(int nID){return ::GetDlgItem(HWindow, nID);}
	UINT GetDlgItemInt(int nID, int *pnTranslated=0, bool oSigned=true)
			{return ::GetDlgItemInt(HWindow, nID, pnTranslated, oSigned);}
	UINT GetDlgItemText(int nID, TCHAR *pszString, int nMaxCount)
			{return ::GetDlgItemText(HWindow, nID, pszString, nMaxCount);}
	HMENU GetMenu(void){return ::GetMenu(HWindow);}
	HWND GetNextDlgGroupItem(int nID, bool oPrevious=false)
			{return ::GetNextDlgGroupItem(HWindow, GetDlgItem(nID), oPrevious);}
	HWND GetNextDlgTabItem(int nID, bool oPrevious=false)
			{return ::GetNextDlgTabItem(HWindow, GetDlgItem(nID), oPrevious);}
	HWND GetParent(void){return ::GetParent(HWindow);}
	BOOL GetScrollInfo(int nBar, SCROLLINFO *pScrollInfo)
			{return ::GetScrollInfo(HWindow, nBar, pScrollInfo);}
	int GetScrollPos(int nFlag){return ::GetScrollPos(HWindow, nFlag);}
	void GetScrollRange(int nFlag, int *pnMin, int *pnMax)
			{::GetScrollRange(HWindow, nFlag, pnMin, pnMax);}
	HMENU GetSystemMenu(bool oRevert){return ::GetSystemMenu(HWindow, oRevert);}
	BOOL GetUpdateRect(RECT *pRect, bool oErase=true)
			{return ::GetUpdateRect(HWindow, pRect, oErase);}
	BOOL GetUpdateRgn(HRGN HRegion, bool oErase=true)
			{return ::GetUpdateRgn(HWindow, HRegion, oErase);}
	HWND GetWindow(unsigned int nCmd){return ::GetWindow(HWindow, nCmd);}
	HDC GetWindowDC(void){return ::GetWindowDC(HWindow);}
	long GetWindowLong(int nIndex){return ::GetWindowLong(HWindow, nIndex);}
	BOOL GetWindowRect(RECT *pRect){return ::GetWindowRect(HWindow, pRect);}
	RECT GetWindowRect(void){RECT rc; ::GetWindowRect(HWindow, &rc); return rc;}
	int GetWindowText(TCHAR *pszText, int nMaxCount)
			{return ::GetWindowText(HWindow, pszText, nMaxCount);}
	int GetWindowTextLength(void){return ::GetWindowTextLength(HWindow);}
	BOOL HideCaret(void){return ::HideCaret(HWindow);}
	BOOL InvalidateRect(RECT *pRect=0, bool oErase=true)
			{return ::InvalidateRect(HWindow, pRect, oErase);}
	BOOL InvalidateRgn(HRGN HRegion=0, bool oErase=true)
			{return ::InvalidateRgn(HWindow, HRegion, oErase);}
	BOOL IsChild(HWND HChild){return ::IsChild(HWindow, HChild);}
	int IsDlgButtonChecked(int nID){return ::IsDlgButtonChecked(HWindow, nID);}
	BOOL IsWindow(void){return ::IsWindow(HWindow);}
	BOOL IsWindowEnabled(void){return ::IsWindowEnabled(HWindow);}
	BOOL IsWindowVisible(void){return ::IsWindowVisible(HWindow);}
	BOOL KillTimer(unsigned int nEventID){return ::KillTimer(HWindow, nEventID);}
	int MessageBox(const TCHAR *pszMsg, const TCHAR *pszTitle, unsigned int nStyle)
			{return ::MessageBox(HWindow, pszMsg, pszTitle, nStyle);}
	BOOL MoveWindow(int X, int Y, int nWidth, int nHeight, bool oRepaint=true)
			{return ::MoveWindow(HWindow, X, Y, nWidth, nHeight, oRepaint);}
	BOOL OpenClipboard(void){return ::OpenClipboard(HWindow);}
	BOOL PostMessage(unsigned int nMsg, WPARAM wParam=0, LPARAM lParam=0)
			{return ::PostMessage(HWindow, nMsg, wParam, lParam);}
#ifdef __WINV40
	BOOL RegisterHotKey(int nID, unsigned int nModFlags, unsigned int nVKey)
			{return ::RegisterHotKey(HWindow, nID, nModFlags, nVKey);}
#endif
	int ReleaseDC(HDC hdc){return ::ReleaseDC(HWindow, hdc);}
	BOOL ScreenToClient(POINT *pPoint){return ::ScreenToClient(HWindow, pPoint);}
	void ScreenToClient(RECT *pRect)
	{
		POINT ptLT = {pRect->left, pRect->top};
		::ScreenToClient(HWindow, &ptLT);
		pRect->left = ptLT.x;
		pRect->top = ptLT.y;
		POINT ptRB = {pRect->right, pRect->bottom};
		::ScreenToClient(HWindow, &ptRB);
		pRect->right = ptRB.x;
		pRect->bottom = ptRB.y;
	}
	void ScrollWindow(int dx, int dy, const RECT *prcScroll=0,
			const RECT *prcClip=0)
			{::ScrollWindow(HWindow, dx, dy, prcScroll, prcClip);}
	int ScrollWindowEx(int dx, int dy, const RECT *prcScroll=0,
			const RECT *prcClip=0, HRGN HRegion=0, RECT *pRect=0,
			unsigned int nFlags=SW_ERASE | SW_INVALIDATE)
			{return ::ScrollWindowEx(HWindow, dx, dy, prcScroll, prcClip, HRegion,
			pRect, nFlags);}
	long SendDlgItemMessage(int nID, unsigned int nMsg, WPARAM wParam=0,
			LPARAM lParam=0)
			{return ::SendDlgItemMessage(HWindow, nID, nMsg, wParam, lParam);}
	// may mess up macros
	long SendMessage(unsigned int nMsg, WPARAM wParam=0, LPARAM lParam=0)
			{return ::SendMessage(HWindow, nMsg, wParam, lParam);}
	HWND SetActiveWindow(void){return ::SetActiveWindow(HWindow);}
	HWND SetCapture(void){return ::SetCapture(HWindow);}
	DWORD SetClassLong(int nIndex, long lValue)
			{return ::SetClassLong(HWindow, nIndex, lValue);}
	BOOL SetDlgItemInt(int nID, unsigned int nValue, bool oSigned=true)
			{return ::SetDlgItemInt(HWindow, nID, nValue, oSigned);}
	BOOL SetDlgItemText(int nID, const TCHAR *pszText)
			{return ::SetDlgItemText(HWindow, nID, pszText);}
	HWND SetFocus(void){return ::SetFocus(HWindow);}
	void SetFont(CFont &font, bool oRepaint=false)
	{
		Font = font;
		if(IsWindow())
			SendMessage(WM_SETFONT, (WPARAM)(HFONT)Font, oRepaint);
	}
#ifdef __WIN32
	BOOL SetForegroundWindow(void){return ::SetForegroundWindow(HWindow);}
#endif
	BOOL SetMenu(HMENU HMenu){return ::SetMenu(HWindow, HMenu);}
	HWND SetParent(HWND HParent){return ::SetParent(HWindow, HParent);}
	void SetRedraw(bool oRedraw=true){SendMessage(WM_SETREDRAW, oRedraw);}
#ifdef __WINV40
	int SetScrollInfo(int nBar, SCROLLINFO *pScrollInfo, bool oRedraw=true)
			{return ::SetScrollInfo(HWindow, nBar, pScrollInfo, oRedraw);}
#endif
	int SetScrollPos(int nBarFlag, int nPos, bool oRedraw=true)
			{return ::SetScrollPos(HWindow, nBarFlag, nPos, oRedraw);}
	int SetScrollRange(int nBarFlag, int nMinPos, int nMaxPos, bool oRedraw=true)
			{return ::SetScrollRange(HWindow, nBarFlag, nMinPos, nMaxPos, oRedraw);}
	unsigned int SetTimer(unsigned int nID, unsigned int nInterval)
			{return ::SetTimer(HWindow, nID, nInterval, 0);}
	long SetWindowLong(int nIndex, long lValue)
			{return ::SetWindowLong(HWindow, nIndex, lValue);}
	BOOL SetWindowPos(HWND HWndPrior, int x, int y, int nWidth, int nHeight,
			unsigned int nFlags){return ::SetWindowPos(HWindow, HWndPrior, x, y,
			nWidth, nHeight, nFlags);}
	BOOL SetWindowText(TCHAR *pszText){return ::SetWindowText(HWindow, pszText);}
	BOOL ShowCaret(void){return ::ShowCaret(HWindow);}
	void ShowScrollBar(int nFlag, BOOL oShow)
			{::ShowScrollBar(HWindow, nFlag, oShow);}
	BOOL ShowWindow(int nCmdShow){return ::ShowWindow(HWindow, nCmdShow);}
#ifdef __WINV40
	BOOL UnregisterHotKey(int nID){return ::UnregisterHotKey(HWindow, nID);}
#endif
	BOOL UpdateWindow(void){return ::UpdateWindow(HWindow);}
	BOOL ValidateRect(const RECT *pRect=0){return ::ValidateRect(HWindow, pRect);}

	virtual LRESULT Dispatch(unsigned int nMessage, WPARAM wParam, LPARAM lParam);
	CWindow *FindWindow(HWND HWnd);
	bool IsDestroyed(void){return m_oRemove;}
	void LoadAccelerators(int nID)
			{HAccel = ::LoadAccelerators(GetAppInstance(), MAKEINTRESOURCE(nID));}
	void LoadAccelerators(const TCHAR *pszTable)
			{HAccel = ::LoadAccelerators(GetAppInstance(), pszTable);}
	// this function is to call DefWindowProc (cannot subclass common controls)
	virtual LRESULT Default(unsigned int nMessage, WPARAM wParam, LPARAM lParam);

	// CApplication needs access to PreDispatch
	friend class CApplication;
};

class CWindowEx : public CWindow
{
private:
	SIZE m_sizeLine;
	SIZE m_sizePage;
	SIZE m_sizeRange;
	SIZE m_sizeDisplayPage;
	POINT m_ptScrollPos;

	void InitMsgHandlers(void);
	void InitScroll(void)
	{
		m_sizeLine.cx = m_sizeLine.cy = 1;
		m_sizePage.cx = m_sizePage.cy = m_sizeRange.cx = m_sizeRange.cy = 0;
		m_sizeDisplayPage.cx = m_sizeDisplayPage.cy = 0;
		m_ptScrollPos.x = m_ptScrollPos.y = 0;
	}

protected:
	long OnCompareItem(WPARAM wParam, LPARAM lParam);
	long OnDeleteItem(WPARAM wParam, LPARAM lParam);
	long OnDrawItem(WPARAM wParam, LPARAM lParam);
	long OnMeasureItem(WPARAM wParam, LPARAM lParam);

	long OnSize(WPARAM, LPARAM lSize);
	long OnHScroll(WPARAM wParam, LPARAM lParam);
	long OnVScroll(WPARAM wParam, LPARAM lParam);
	long OnPaint(WPARAM, LPARAM);
	long OnClose(WPARAM, LPARAM);
	long OnWmQueryEndSession(WPARAM, LPARAM){OnQueryEndSession();return 1;}

	virtual void Paint(CDC& dc){}

public:
	CWindowEx(){InitMsgHandlers(); InitScroll();}
	CWindowEx(HWND HWnd) : CWindow(HWnd){InitMsgHandlers(); InitScroll();}

	void SetScrollLine(SIZE sizeLine){m_sizeLine = sizeLine;}
	void SetScrollPage(SIZE sizePage){m_sizePage = sizePage;}
	void SetScrollRange(SIZE sizeRange){m_sizeRange = sizeRange;}
	SIZE GetScrollLine(void){return m_sizeLine;}
	SIZE GetScrollPage(void){return m_sizePage;}
	void UpdateScrollInfo(void);
	void SetDisplayPage(SIZE sizePage){m_sizeDisplayPage = sizePage;}
	SIZE GetDisplayPage(void){return m_sizeDisplayPage;}
};

class CInfoWindow;

class CDialog : public CWindowEx
{
private:
	const TCHAR *m_pszDlgID;
	bool m_oModal;
	int m_nResult;

	void InitMsgHandlers(void);

protected:
	long OnInitDialog(WPARAM, LPARAM){SetupDialog(); return true;}
	long OnCompareItem(WPARAM wParam, LPARAM lParam);
	long OnDeleteItem(WPARAM wParam, LPARAM lParam);
	long OnDrawItem(WPARAM wParam, LPARAM lParam);
	long OnMeasureItem(WPARAM wParam, LPARAM lParam);

	virtual void SetupDialog(void){}
	virtual void CmOk(void){Close(IDOK);}
	virtual void CmCancel(void){Close(IDCANCEL);}

public:
	CDialog(int nDlgID) : iToolTipEnabled(FALSE), iInfoWindow(NULL), m_pszDlgID(MAKEINTRESOURCE(nDlgID)){InitMsgHandlers();}
	CDialog(const TCHAR *pszDlgID) : iToolTipEnabled(FALSE), iInfoWindow(NULL), m_pszDlgID(pszDlgID){InitMsgHandlers();}

	bool Create(void);
	int Execute(void);
	bool Close(int nResult=IDCANCEL);
	void Destroy(void);
	// override default processing to return false
	LRESULT Default(unsigned int, WPARAM, LPARAM);

	BOOL MapDialogRect(RECT *pRect){return ::MapDialogRect(HWindow, pRect);}

protected:	// tooltip data members
	XList<SToolTipInfo>		iToolTipInfo;
	BOOL					iToolTipEnabled;
	CInfoWindow*			iInfoWindow;

protected:	// tooltip methods
	void SpyMessageHook(HWND &HWnd, UINT &nMsg, WPARAM &wParam, LPARAM &lParam);
	virtual void OnNotify(HWND hwndFrom, UINT idFrom, UINT code, WPARAM wParam, LPARAM lParam);
	virtual void OnHelp(LPHELPINFO aHelpInfo);

	void AddToolTipInfo(SToolTipInfo& aToolTipInfo);
	SToolTipInfo* FindToolTip(unsigned int aControlId);

private:
	void EnableToolTip();
};


class CAppWindow : public CWindowEx
{
private:
	TCHAR m_szClassName[32];

public:
	CAppWindow(const TCHAR *pszIcon=0, const TCHAR *pszMenu=0);
	bool Create(int x=CW_USEDEFAULT, int y=CW_USEDEFAULT, int nWidth=CW_USEDEFAULT,
			int nHeight=CW_USEDEFAULT, DWORD dwStyle=WS_OVERLAPPEDWINDOW);
	void Destroy(void){CWindow::Destroy(); PostQuitMessage(0);}
};

struct SClassName
{
	TCHAR szName[64];
};

class CApplication
{
private:
	XList <CWindow *> m_Windows;
	XList <SClassName> m_Classes;
	XList <SWinIdle> m_IdleHandlers;
	const TCHAR *m_pszCmdLine;
	int m_nCmdShow;
#ifdef __WINV40
	HICON m_hSmallIcon;
#endif
	TCHAR m_szModuleName[255];
	TCHAR m_szProfileName[255];

protected:
	bool IsAccelerator(LPMSG pMsg);
	bool PreTranslate(LPMSG pMsg);
	virtual bool Initialize(void){return true;}
	virtual int RunLoop(void);

	virtual int main(HINSTANCE hInst, HINSTANCE hPrevInst, const TCHAR *pszCmdLine,
			int nCmdShow);
	virtual void dllmain(HINSTANCE hInstDLL);

public:
	HINSTANCE HInstance;
	HINSTANCE HPrevInstance;

	CApplication(); // initializes global pointer used by WinMain and WindowProcs
	~CApplication();

	void AddWindow(CWindow *pWindow)
			{m_Windows.Add(pWindow); pWindow->oAdded = true;}
	void RemoveWindow(CWindow *pWindow);
	void AddIdle(SWinIdle &IdleHandler){m_IdleHandlers.Add(IdleHandler);}

	LRESULT Dispatch(HWND HWnd, UINT nMsg, WPARAM wParam, LPARAM lParam);

	HBITMAP LoadBitmap(const TCHAR *pszBitmap)
		{return ::LoadBitmap(HInstance, pszBitmap);}
	HBITMAP LoadBitmap(int nID)
		{return ::LoadBitmap(HInstance, MAKEINTRESOURCE(nID));}
	HCURSOR LoadCursor(const TCHAR *pszCursor)
		{return ::LoadCursor(HInstance, pszCursor);}
	HCURSOR LoadCursor(int nID)
		{return ::LoadCursor(HInstance, MAKEINTRESOURCE(nID));}
	HICON LoadIcon(const TCHAR *pszIcon)
		{return ::LoadIcon(HInstance, pszIcon);}
	HICON LoadIcon(int nID)
		{return ::LoadIcon(HInstance, MAKEINTRESOURCE(nID));}
	HMENU LoadMenu(const TCHAR *pszMenu)
		{return ::LoadMenu(HInstance, pszMenu);}
	HMENU LoadMenu(int nID)
		{return ::LoadMenu(HInstance, MAKEINTRESOURCE(nID));}
	int LoadString(int nID, TCHAR *pszString, int nSize)
		{return ::LoadString(HInstance, nID, pszString, nSize);}

	const TCHAR *GetCmdLine(void){return m_pszCmdLine;}
	int GetShowCommand(void){return m_nCmdShow;}
	const TCHAR *GetModuleName(void){return m_szModuleName;}
	CWindow *GetWindow(int nIndex=0)
	{
		if(nIndex >= 0 && nIndex < m_Windows.GetCount())
			return m_Windows[nIndex];
		else
			return 0;
	}
	// Application profile storage
	void GetProfileString(const TCHAR *pszSection, const TCHAR *pszKey,
			const TCHAR *pszDefault, TCHAR *pszString, int nSize)
	{
		GetPrivateProfileString(pszSection, pszKey, pszDefault, pszString,
				nSize, m_szProfileName);
	}
	int GetProfileInt(const TCHAR *pszSection, const TCHAR *pszKey, int nDefault)
	{
		return GetPrivateProfileInt(pszSection, pszKey, nDefault,
				m_szProfileName);
	}
	void SetProfileString(const TCHAR *pszSection, const TCHAR *pszKey,
			TCHAR *pszString)
	{
		WritePrivateProfileString(pszSection, pszKey, pszString, m_szProfileName);
	}
	void SetProfileInt(const TCHAR *pszSection, const TCHAR *pszKey, int nValue)
	{
		TCHAR szValue[16];
		wsprintf(szValue, _TEXT("%d"), nValue);
		WritePrivateProfileString(pszSection, pszKey, szValue, m_szProfileName);
	}

#ifdef __WINV40
	void SetSmallIcon(HICON HSmallIcon){m_hSmallIcon = HSmallIcon;}
#endif
	void RegisterClass(const TCHAR *pszClassName=_TEXT("CWAppWin"), HICON HIcon=0,
			const TCHAR *pszMenu=0, int nBkgColor=-1, HCURSOR HCursor=0,
			unsigned int nStyle=CS_HREDRAW | CS_VREDRAW);

	bool IsRegisteredClass(const TCHAR *pszClassName);

friend int WINAPI WinMain(HINSTANCE, HINSTANCE, LPTSTR, int);
friend BOOL WINAPI DllMain(HINSTANCE, DWORD, LPVOID);
};

// template function to make new windows
//////////////////////
// Application Windows
template <class T> void AddNewCW(T *&pWindow, CApplication *pParent)
{
	pWindow = new T();
	if(pWindow)
	{
		pWindow->SetAutoDestroy();
		pParent->AddWindow(pWindow);
	}
}

template <class T> void AddNewCW(T *&pWindow, CApplication *pParent, HWND HWindow)
{
	pWindow = new T(HWindow);
	if(pWindow)
	{
		pWindow->SetAutoDestroy();
		pParent->AddWindow(pWindow);
	}
}

template <class T> void AddNewCW(T *&pWindow, CApplication *pParent,
		const TCHAR *pszIcon, const TCHAR *pszMenu)
{
	pWindow = new T(pszIcon, pszMenu);
	if(pWindow)
	{
		pWindow->SetAutoDestroy();
		pParent->AddWindow(pWindow);
	}
}

template <class T> void AddCW(T &Window, CApplication *pParent)
{
	pParent->AddWindow(&Window);
}
//////////////////////
// Owner/Child Windows
template <class T> void AddNewCW(T *&pWindow, CWindow *pParent)
{
	pWindow = new T();
	if(pWindow)
	{
		pWindow->SetAutoDestroy();
		pParent->AddChild(pWindow);
	}
}

template <class T> void AddNewCW(T *&pWindow, CWindow *pParent, HWND HWindow)
{
	pWindow = new T(HWindow);
	if(pWindow)
	{
		pWindow->SetAutoDestroy();
		pParent->AddChild(pWindow);
	}
}

template <class T> void AddNewCW(T *&pWindow, CWindow *pParent, int nDlgID)
{
	pWindow = new T(nDlgID);
	if(pWindow)
	{
		pWindow->SetAutoDestroy();
		pParent->AddChild(pWindow);
	}
}

template <class T> void AddNewCW(T *&pWindow, CWindow *pParent, const TCHAR *pszDlgID)
{
	pWindow = new T(pszDlgID);
	if(pWindow)
	{
		pWindow->SetAutoDestroy();
		pParent->AddChild(pWindow);
	}
}

template <class T> void AddCW(T &Window, CWindow *pParent)
{
	pParent->AddChild(&Window);
}

/*******************************************************************
 *
 * $Log: /Deskwin/Src/CWindow.h $
 * 
 * 4     10/01/00 9:17 Pese
 * 
 * 3     5/01/00 20:26 Pese
 * 
 * 2     23/12/99 10:16 Pese
 *
 *******************************************************************/

#endif

