/*******************************************************************
 *
 *	File:		TDeskwinOptions.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 1999, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#include "Deskwin.h"
#include <COMMCTRL.H>			// for hotkey modifiers
#include "TDeskwinOptions.h"
#include "regutil.h"

long SetRegKeyBoolValue(
	IN	HKEY				RootKey,
	IN	const _TCHAR		*KeyName,
	IN	const _TCHAR		*ValueName,
	IN	bool				BoolValue
	)
{
	int		IntegerValue = BoolValue ? 1 : 0;
	return SetRegKeyIntegerValue(
				RootKey,
				KeyName,
				ValueName,
				IntegerValue
				);
}

long GetRegKeyBoolValue(
	IN	HKEY				RootKey,
	IN	const _TCHAR		*KeyName,
	IN	const _TCHAR		*ValueName,
	OUT	bool				*BoolValue
	)
{
	int	dummy;
	long error = GetRegKeyIntegerValue(RootKey, KeyName, ValueName, &dummy);
	if (!error)
		*BoolValue = (dummy == 0) ? false : true;
	return error;
}



void SaveOptions(TDeskwinOptions& aOptions)
{
	CreateRegKey(HKEY_CURRENT_USER, _T(""), REG_OPTION_ROOT);
	CreateRegKey(HKEY_CURRENT_USER, _T(""), OPTION_COLSIZE_ROOT);
	CreateRegKey(HKEY_CURRENT_USER, _T(""), OPTION_DOCKING_ROOT);
	CreateRegKey(HKEY_CURRENT_USER, _T(""), OPTION_AUTOHIDE_ROOT);
	CreateRegKey(HKEY_CURRENT_USER, _T(""), OPTION_WINPOS_TRACKING_ROOT);
	CreateRegKey(HKEY_CURRENT_USER, _T(""), OPTION_HOTKEY_ROOT);
	CreateRegKey(HKEY_CURRENT_USER, _T(""), OPTION_STATE_ROOT);
	CreateRegKey(HKEY_CURRENT_USER, _T(""), OPTION_APPEARANCE_ROOT);

	DestroyRegKey(HKEY_CURRENT_USER, _T(""), OPTION_STICKY_ROOT);
	CreateRegKey(HKEY_CURRENT_USER, _T(""), OPTION_STICKY_ROOT);

	SetRegKeyIntegerValue(HKEY_CURRENT_USER, REG_OPTION_ROOT, OPTION_NO_OF_DESKTOPS_HOR, aOptions.iNoOfDesktopsHor);
	SetRegKeyIntegerValue(HKEY_CURRENT_USER, REG_OPTION_ROOT, OPTION_NO_OF_DESKTOPS_VERT, aOptions.iNoOfDesktopsVert);

	SetRegKeyIntegerValue(HKEY_CURRENT_USER, OPTION_COLSIZE_ROOT, OPTION_DESKTOP_PIXEL_WIDTH, aOptions.iDesktopPixelWidth);
	SetRegKeyIntegerValue(HKEY_CURRENT_USER, OPTION_COLSIZE_ROOT, OPTION_DESKTOP_PIXEL_HEIGHT, aOptions.iDesktopPixelHeight);
	SetRegKeyIntegerValue(HKEY_CURRENT_USER, OPTION_COLSIZE_ROOT, OPTION_DESKTOP_OUT_FRAME_PIXEL_SIZE, aOptions.iDesktopOutFramePixelSize);
	SetRegKeyIntegerValue(HKEY_CURRENT_USER, OPTION_COLSIZE_ROOT, OPTION_DESKTOP_IN_FRAME_PIXEL_SIZE, aOptions.iDesktopInFramePixelSize);
	SetRegKeyUintValue(HKEY_CURRENT_USER, OPTION_COLSIZE_ROOT, OPTION_OUTER_FRAME_COLOR, aOptions.iOuterFrameColor);
	SetRegKeyUintValue(HKEY_CURRENT_USER, OPTION_COLSIZE_ROOT, OPTION_INNER_FRAME_COLOR, aOptions.iInnerFrameColor);
	SetRegKeyUintValue(HKEY_CURRENT_USER, OPTION_COLSIZE_ROOT, OPTION_BACK_GROUND_COLOR, aOptions.iBackgroundColor);
	SetRegKeyUintValue(HKEY_CURRENT_USER, OPTION_COLSIZE_ROOT, OPTION_ACTIVE_DESKTOP_COLOR, aOptions.iActiveDesktopColor);
	SetRegKeyUintValue(HKEY_CURRENT_USER, OPTION_COLSIZE_ROOT, OPTION_WINDOW_FRAME_COLOR, aOptions.iWindowFrameColor);
	SetRegKeyUintValue(HKEY_CURRENT_USER, OPTION_COLSIZE_ROOT, OPTION_WINDOW_COLOR, aOptions.iWindowColor);

	SetRegKeyBoolValue(HKEY_CURRENT_USER, OPTION_DOCKING_ROOT, OPTION_WANT_DOCKING, aOptions.iWantsDocking);
	SetRegKeyIntegerValue(HKEY_CURRENT_USER, OPTION_DOCKING_ROOT, OPTION_DOCK_PERCENT_FROM_EDGE, aOptions.iDockPercentFromEdge);
	SetRegKeyIntegerValue(HKEY_CURRENT_USER, OPTION_DOCKING_ROOT, OPTION_PERCENT_VISIBLE, aOptions.iPercentVisible);

	SetRegKeyBoolValue(HKEY_CURRENT_USER, OPTION_AUTOHIDE_ROOT, OPTION_AUTO_HIDE, aOptions.iWantsAutoHide);
	SetRegKeyBoolValue(HKEY_CURRENT_USER, OPTION_AUTOHIDE_ROOT, OPTION_SHOW_BY_MOUSE_CLICK, aOptions.iWantsShowByMouseClick);
	SetRegKeyIntegerValue(HKEY_CURRENT_USER, OPTION_AUTOHIDE_ROOT, OPTION_AUTO_HIDE_TIMEOUT, aOptions.iAutoHideTimeOut);
	SetRegKeyIntegerValue(HKEY_CURRENT_USER, OPTION_AUTOHIDE_ROOT, OPTION_ANIMATION_TIME, aOptions.iAnimationTime);
	SetRegKeyIntegerValue(HKEY_CURRENT_USER, OPTION_AUTOHIDE_ROOT, OPTION_ANIMATION_STEP_TIME, aOptions.iAnimationStepTime);

	SetRegKeyIntegerValue(HKEY_CURRENT_USER, OPTION_WINPOS_TRACKING_ROOT, OPTION_ACTIVE_POSITION_TRACKING_TIME, aOptions.iActivePositionTrackingTime);
	SetRegKeyIntegerValue(HKEY_CURRENT_USER, OPTION_WINPOS_TRACKING_ROOT, OPTION_PASSIVE_POSITION_TRACKING_TIME, aOptions.iBackgroundPositionTrackingTime);

	SetRegKeyIntegerValue(HKEY_CURRENT_USER, OPTION_HOTKEY_ROOT, OPTION_HOTKEY_SHOW_DESKWIN, aOptions.iHotkeyShowDeskwin);
	SetRegKeyIntegerValue(HKEY_CURRENT_USER, OPTION_HOTKEY_ROOT, OPTION_HOTKEY_NEXT, aOptions.iHotkeyNextDesktop);
	SetRegKeyIntegerValue(HKEY_CURRENT_USER, OPTION_HOTKEY_ROOT, OPTION_HOTKEY_PREVIOUS, aOptions.iHotkeyPreviousDesktop);
	SetRegKeyIntegerValue(HKEY_CURRENT_USER, OPTION_HOTKEY_ROOT, OPTION_HOTKEY_UP, aOptions.iHotkeyDesktopUp);
	SetRegKeyIntegerValue(HKEY_CURRENT_USER, OPTION_HOTKEY_ROOT, OPTION_HOTKEY_DOWN, aOptions.iHotkeyDesktopDown);
	SetRegKeyIntegerValue(HKEY_CURRENT_USER, OPTION_HOTKEY_ROOT, OPTION_HOTKEY_TOGGLE_EXPLORER, aOptions.iHotkeyToggleExplorer);
	SetRegKeyIntegerValue(HKEY_CURRENT_USER, OPTION_HOTKEY_ROOT, OPTION_HOTKEY_MINIMISE_ALL, aOptions.iHotkeyMinimiseAll);
	SetRegKeyIntegerValue(HKEY_CURRENT_USER, OPTION_HOTKEY_ROOT, OPTION_HOTKEY_VOLUME_TOGGLE_MUTE, aOptions.iHotkeyVolumeToggleMute);
	SetRegKeyIntegerValue(HKEY_CURRENT_USER, OPTION_HOTKEY_ROOT, OPTION_HOTKEY_VOLUME_UP, aOptions.iHotkeyVolumeUp);
	SetRegKeyIntegerValue(HKEY_CURRENT_USER, OPTION_HOTKEY_ROOT, OPTION_HOTKEY_VOLUME_DOWN, aOptions.iHotkeyVolumeDown);

	int d=0;
	for (THotkeyList::iterator i=aOptions.iHotkeyList.begin() ; i!=aOptions.iHotkeyList.end() ; i++, d++)
	{
		THotkey hotkey = *i;
		if (hotkey)
		{
			TCHAR		Text[128];
			sprintf(Text, _T("%s%i"), OPTION_HOTKEY_DESKTOP_BASE, d);
			SetRegKeyIntegerValue(HKEY_CURRENT_USER, OPTION_HOTKEY_ROOT, Text, hotkey);
		}
	}

	for (TStickyWindowInfo::iterator j=aOptions.iStickyWindowInfo.begin() ; j!=aOptions.iStickyWindowInfo.end() ; j++)
	{
		TStickyWindow&	stickyWindow = *j;
		TCHAR		Text[1024];
		sprintf(Text, _T("%s\\%s"), OPTION_STICKY_ROOT, stickyWindow.iRuleName.c_str());
		CreateRegKey(HKEY_CURRENT_USER, OPTION_STICKY_ROOT, stickyWindow.iRuleName.c_str());

		SetRegKeyStringValue(HKEY_CURRENT_USER, Text, STICKY_TITLE_NAME, stickyWindow.iTitleName.c_str());
		SetRegKeyStringValue(HKEY_CURRENT_USER, Text, STICKY_CLASS_NAME, stickyWindow.iClassName.c_str());
		
		SetRegKeyIntegerValue(HKEY_CURRENT_USER, Text, STICKY_DESKTOP_STATE, (int) stickyWindow.iStickyWindowDesktop);

		if (stickyWindow.iStickyWindowMatch == EStickyWindowMatchAnd)
			SetRegKeyStringValue(HKEY_CURRENT_USER, Text, STICKY_RULE_MATCH, _T("And"));
		else
			SetRegKeyStringValue(HKEY_CURRENT_USER, Text, STICKY_RULE_MATCH, _T("Or"));

		int			count = 0;
		TCHAR		desktopList[1024];
		TCHAR		num[32];
		desktopList[0] = 0;
		for (TDesktopList::iterator d=stickyWindow.iDesktopList.begin() ; d!=stickyWindow.iDesktopList.end() ; d++)
		{
			int		val = *d;
			sprintf(num, "%i", val);
			if (count != 0)
				strcat(desktopList, ", ");
			strcat(desktopList, num);
			count++;
		}
		SetRegKeyStringValue(HKEY_CURRENT_USER, Text, STICKY_DESKTOP_LIST, desktopList);
	}


	SetRegKeyBoolValue(HKEY_CURRENT_USER, OPTION_APPEARANCE_ROOT, OPTION_APPEARANCE_ALWAYS_ON_TOP, aOptions.iAlwaysOnTop);
	SetRegKeyBoolValue(HKEY_CURRENT_USER, OPTION_APPEARANCE_ROOT, OPTION_APPEARANCE_ICON_TRANSPARENT, aOptions.iSysTrayIconOptions.iTransparentBackground);
	SetRegKeyIntegerValue(HKEY_CURRENT_USER, OPTION_APPEARANCE_ROOT, OPTION_APPEARANCE_ICONSET, aOptions.iSysTrayIconOptions.iIconSet);
	SetRegKeyUintValue(HKEY_CURRENT_USER, OPTION_APPEARANCE_ROOT, OPTION_APPEARANCE_ICON_FG_COLOR, aOptions.iSysTrayIconOptions.iFgColor);
	SetRegKeyUintValue(HKEY_CURRENT_USER, OPTION_APPEARANCE_ROOT, OPTION_APPEARANCE_ICON_BG_COLOR, aOptions.iSysTrayIconOptions.iBgColor);
}


void LoadOptions(TDeskwinOptions& aOptions)
{
	GetRegKeyIntegerValue(HKEY_CURRENT_USER, REG_OPTION_ROOT, OPTION_NO_OF_DESKTOPS_HOR, &aOptions.iNoOfDesktopsHor);
	GetRegKeyIntegerValue(HKEY_CURRENT_USER, REG_OPTION_ROOT, OPTION_NO_OF_DESKTOPS_VERT, &aOptions.iNoOfDesktopsVert);

	GetRegKeyIntegerValue(HKEY_CURRENT_USER, OPTION_COLSIZE_ROOT, OPTION_DESKTOP_PIXEL_WIDTH, &aOptions.iDesktopPixelWidth);
	GetRegKeyIntegerValue(HKEY_CURRENT_USER, OPTION_COLSIZE_ROOT, OPTION_DESKTOP_PIXEL_HEIGHT, &aOptions.iDesktopPixelHeight);

	GetRegKeyIntegerValue(HKEY_CURRENT_USER, OPTION_COLSIZE_ROOT, OPTION_DESKTOP_OUT_FRAME_PIXEL_SIZE, &aOptions.iDesktopOutFramePixelSize);
	GetRegKeyIntegerValue(HKEY_CURRENT_USER, REG_OPTION_ROOT, OPTION_DESKTOP_IN_FRAME_PIXEL_SIZE, &aOptions.iDesktopInFramePixelSize);

	GetRegKeyUintValue(HKEY_CURRENT_USER, OPTION_COLSIZE_ROOT, OPTION_OUTER_FRAME_COLOR, (unsigned int*) &aOptions.iOuterFrameColor);
	GetRegKeyUintValue(HKEY_CURRENT_USER, OPTION_COLSIZE_ROOT, OPTION_INNER_FRAME_COLOR, (unsigned int*) &aOptions.iInnerFrameColor);
	GetRegKeyUintValue(HKEY_CURRENT_USER, OPTION_COLSIZE_ROOT, OPTION_BACK_GROUND_COLOR, (unsigned int*) &aOptions.iBackgroundColor);
	GetRegKeyUintValue(HKEY_CURRENT_USER, OPTION_COLSIZE_ROOT, OPTION_ACTIVE_DESKTOP_COLOR, (unsigned int*) &aOptions.iActiveDesktopColor);
	GetRegKeyUintValue(HKEY_CURRENT_USER, OPTION_COLSIZE_ROOT, OPTION_WINDOW_FRAME_COLOR, (unsigned int*) &aOptions.iWindowFrameColor);
	GetRegKeyUintValue(HKEY_CURRENT_USER, OPTION_COLSIZE_ROOT, OPTION_WINDOW_COLOR, (unsigned int*) &aOptions.iWindowColor);

	GetRegKeyBoolValue(HKEY_CURRENT_USER, OPTION_DOCKING_ROOT, OPTION_WANT_DOCKING, &aOptions.iWantsDocking);
	GetRegKeyIntegerValue(HKEY_CURRENT_USER, OPTION_DOCKING_ROOT, OPTION_DOCK_PERCENT_FROM_EDGE, &aOptions.iDockPercentFromEdge);
	GetRegKeyIntegerValue(HKEY_CURRENT_USER, OPTION_DOCKING_ROOT, OPTION_PERCENT_VISIBLE, &aOptions.iPercentVisible);

	GetRegKeyBoolValue(HKEY_CURRENT_USER, OPTION_AUTOHIDE_ROOT, OPTION_AUTO_HIDE, &aOptions.iWantsAutoHide);
	GetRegKeyBoolValue(HKEY_CURRENT_USER, OPTION_AUTOHIDE_ROOT, OPTION_SHOW_BY_MOUSE_CLICK, &aOptions.iWantsShowByMouseClick);
	GetRegKeyIntegerValue(HKEY_CURRENT_USER, OPTION_AUTOHIDE_ROOT, OPTION_AUTO_HIDE_TIMEOUT, &aOptions.iAutoHideTimeOut);
	GetRegKeyIntegerValue(HKEY_CURRENT_USER, OPTION_AUTOHIDE_ROOT, OPTION_ANIMATION_TIME, &aOptions.iAnimationTime);
	GetRegKeyIntegerValue(HKEY_CURRENT_USER, OPTION_AUTOHIDE_ROOT, OPTION_ANIMATION_STEP_TIME,  &aOptions.iAnimationStepTime);

	GetRegKeyIntegerValue(HKEY_CURRENT_USER, OPTION_WINPOS_TRACKING_ROOT, OPTION_ACTIVE_POSITION_TRACKING_TIME, &aOptions.iActivePositionTrackingTime);
	GetRegKeyIntegerValue(HKEY_CURRENT_USER, OPTION_WINPOS_TRACKING_ROOT, OPTION_PASSIVE_POSITION_TRACKING_TIME, &aOptions.iBackgroundPositionTrackingTime);

	GetRegKeyUintValue(HKEY_CURRENT_USER, OPTION_HOTKEY_ROOT, OPTION_HOTKEY_SHOW_DESKWIN, &aOptions.iHotkeyShowDeskwin);
	GetRegKeyUintValue(HKEY_CURRENT_USER, OPTION_HOTKEY_ROOT, OPTION_HOTKEY_NEXT, &aOptions.iHotkeyNextDesktop);
	GetRegKeyUintValue(HKEY_CURRENT_USER, OPTION_HOTKEY_ROOT, OPTION_HOTKEY_PREVIOUS, &aOptions.iHotkeyPreviousDesktop);
	GetRegKeyUintValue(HKEY_CURRENT_USER, OPTION_HOTKEY_ROOT, OPTION_HOTKEY_UP, &aOptions.iHotkeyDesktopUp);
	GetRegKeyUintValue(HKEY_CURRENT_USER, OPTION_HOTKEY_ROOT, OPTION_HOTKEY_DOWN, &aOptions.iHotkeyDesktopDown);
	GetRegKeyUintValue(HKEY_CURRENT_USER, OPTION_HOTKEY_ROOT, OPTION_HOTKEY_TOGGLE_EXPLORER, &aOptions.iHotkeyToggleExplorer);
	GetRegKeyUintValue(HKEY_CURRENT_USER, OPTION_HOTKEY_ROOT, OPTION_HOTKEY_MINIMISE_ALL, &aOptions.iHotkeyMinimiseAll);
	GetRegKeyUintValue(HKEY_CURRENT_USER, OPTION_HOTKEY_ROOT, OPTION_HOTKEY_VOLUME_TOGGLE_MUTE, &aOptions.iHotkeyVolumeToggleMute);
	GetRegKeyUintValue(HKEY_CURRENT_USER, OPTION_HOTKEY_ROOT, OPTION_HOTKEY_VOLUME_UP, &aOptions.iHotkeyVolumeUp);
	GetRegKeyUintValue(HKEY_CURRENT_USER, OPTION_HOTKEY_ROOT, OPTION_HOTKEY_VOLUME_DOWN, &aOptions.iHotkeyVolumeDown);

	for (int d=0 ; d< aOptions.iNoOfDesktopsHor * aOptions.iNoOfDesktopsVert ; d++)
	{
		TCHAR		Text[128];
		THotkey		hotkey = 0;
		sprintf(Text, _T("%s%i"), OPTION_HOTKEY_DESKTOP_BASE, d);
		GetRegKeyUintValue(HKEY_CURRENT_USER, OPTION_HOTKEY_ROOT, Text, &hotkey);
		aOptions.iHotkeyList.push_back(hotkey);
	}

	for (int i= 0 ; ; i++)
	{
		TCHAR			RuleName[1204];
		TStickyWindow	stickyWindow;
		TCHAR			Text[1024];
		TCHAR			tempString[1024];
		int				tempStringLen;
		int				tempInt;

		long status = EnumRegKey(HKEY_CURRENT_USER, OPTION_STICKY_ROOT, i, RuleName, 1023);
		if (status != 0)
			break;
		
		RuleName[1203] = 0;
		stickyWindow.iRuleName.assign(RuleName);

		sprintf(Text, _T("%s\\%s"), OPTION_STICKY_ROOT, RuleName);

		tempStringLen = 1024;
		GetRegKeyStringValue(HKEY_CURRENT_USER, Text, STICKY_TITLE_NAME, tempString, &tempStringLen);
		stickyWindow.iTitleName.assign(tempString);

		tempStringLen = 1024;
		GetRegKeyStringValue(HKEY_CURRENT_USER, Text, STICKY_CLASS_NAME, tempString, &tempStringLen);
		stickyWindow.iClassName.assign(tempString);

		GetRegKeyIntegerValue(HKEY_CURRENT_USER, Text, STICKY_DESKTOP_STATE, &tempInt);
		stickyWindow.iStickyWindowDesktop = (TStickyWindowDesktop) tempInt;

		tempStringLen = 1024;
		GetRegKeyStringValue(HKEY_CURRENT_USER, Text, STICKY_RULE_MATCH, tempString, &tempStringLen);
		if (stricmp(tempString, "and") == 0)
			stickyWindow.iStickyWindowMatch = EStickyWindowMatchAnd;
		else
			stickyWindow.iStickyWindowMatch = EStickyWindowMatchOr;
	
		TCHAR		desktopList[1024];
		int			desktopListLen = 1024;;
		TCHAR		*desktopListPtr = desktopList;
		int			num;
		desktopList[0] = 0;

		GetRegKeyStringValue(HKEY_CURRENT_USER, Text, STICKY_DESKTOP_LIST, desktopList, &desktopListLen);
		do
		{
			int rval = sscanf(desktopListPtr, "%i", &num);
			if ((rval == 0) || (rval == EOF))
				break;
			stickyWindow.iDesktopList.push_back(num);
			desktopListPtr = strchr(desktopListPtr, ',');
			if (!desktopListPtr)
				break;
			desktopListPtr += 2;		// skip comma and space
		} while (TRUE);

		aOptions.iStickyWindowInfo.push_back(stickyWindow);
	}

	GetRegKeyBoolValue(HKEY_CURRENT_USER, OPTION_APPEARANCE_ROOT, OPTION_APPEARANCE_ALWAYS_ON_TOP, &aOptions.iAlwaysOnTop);
	GetRegKeyBoolValue(HKEY_CURRENT_USER, OPTION_APPEARANCE_ROOT, OPTION_APPEARANCE_ICON_TRANSPARENT, &aOptions.iSysTrayIconOptions.iTransparentBackground);
	GetRegKeyIntegerValue(HKEY_CURRENT_USER, OPTION_APPEARANCE_ROOT, OPTION_APPEARANCE_ICONSET, &aOptions.iSysTrayIconOptions.iIconSet);
	GetRegKeyUintValue(HKEY_CURRENT_USER, OPTION_APPEARANCE_ROOT, OPTION_APPEARANCE_ICON_FG_COLOR, (unsigned int*) &aOptions.iSysTrayIconOptions.iFgColor);
	GetRegKeyUintValue(HKEY_CURRENT_USER, OPTION_APPEARANCE_ROOT, OPTION_APPEARANCE_ICON_BG_COLOR, (unsigned int*) &aOptions.iSysTrayIconOptions.iBgColor);
}


void MakeWParmHotkeyMessage(THotkey aHotkey, WORD& aWParm)
{
	int vk, mod;
	int c_mod;

	c_mod = 0;
	MakeRegisterHotkeyParam(aHotkey, vk, mod);
	if (mod & MOD_ALT)
		c_mod |= HOTKEYF_ALT;
	if (mod & MOD_CONTROL)
		c_mod |= HOTKEYF_CONTROL;
	if (mod & MOD_SHIFT)
		c_mod |= HOTKEYF_SHIFT;

	aWParm = MAKEWORD(vk, c_mod); 
}

void MakeHotkeyFromWParamMessage(THotkey& aHotkey, WORD aWParm)
{
	int vk, mod;
	int c_mod;

	mod = 0;
	vk = aWParm & 0xff;
	c_mod = aWParm >> 8;
	if (c_mod & HOTKEYF_ALT)
		mod |= MOD_ALT;
	if (c_mod & HOTKEYF_CONTROL)
		mod |= MOD_CONTROL;
	if (c_mod & HOTKEYF_SHIFT)
		mod |= MOD_SHIFT;

	aHotkey = (vk << 16) | (mod & 0xffff);
}


void MakeRegisterHotkeyParam(THotkey aHotkey, int& aVk, int& aMod)
{
	aVk = aHotkey >> 16;
	aMod = aHotkey & 0xffff;
}

void CopyOptions(const TDeskwinOptions& aSrcOptions, TDeskwinOptions& aDstOptions)
{
	aDstOptions = aSrcOptions;
	aDstOptions.iHotkeyList.assign(aSrcOptions.iHotkeyList.begin(), aSrcOptions.iHotkeyList.end());
}

/*******************************************************************
 *
 * $Log: /Deskwin/Src/TDeskwinOptions.cpp $
 * 
 * 4     10/01/00 9:17 Pese
 * 
 * 3     5/01/00 20:26 Pese
 * 
 * 2     23/12/99 10:16 Pese
 *
 *******************************************************************/
