//----------------------------
// Array templates <xvector.h>
//
// (Released under the terms
// of the GNU General Public
// License.)
//
// author:  Michael Hansen
// initiated: October 1995
//----------------------------

#ifndef _XVECTOR_H_
#define _XVECTOR_H_

// included for XSortedList::Add() memmove()
#include <string.h>

//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
// LOCAL OBJECT LIST CLASS
// This class manages a list of objects allocated on the heap.  Since new and
// delete operators are used, this class may only be used in a single module
// or application.  If intermodule communication requires passing a copy of
// this or a derived class, DO NOT USE THIS CLASS!!!
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

const int ct_nGrowSize = 10;

/////////////////////////////////////////////////////////////////////////////
// XVector definition

template <class T>
class XVector
{
private:
	T ***m_pPtrArray;
	int *m_pnSize;
	int *m_pnRefCnt;

	void GrowArray(int nNewSize);
	void Destroy(void);
	void Copy(XVector &rVector)
	{
		rVector.m_pnRefCnt[0]++;

		m_pPtrArray = rVector.m_pPtrArray;
		m_pnSize = rVector.m_pnSize;
		m_pnRefCnt = rVector.m_pnRefCnt;
	}

protected:
	void Empty(void)
	{
		// detach from original list
		Destroy();
		// initialize new list
		m_pPtrArray = new T **(0);
		m_pnSize = new int(0);
		m_pnRefCnt = new int(0);
	}

public:
	XVector(){m_pPtrArray = new T **(0); m_pnSize = new int(0); m_pnRefCnt = new int(0);}
	XVector(XVector &rVector){Copy(rVector);}
	virtual ~XVector(){Destroy();}

	int Size(void){return m_pnSize[0];}

	// overloaded operators
	T& operator[](int nIndex)
	{
//		if(nIndex < 0) throw;
		if(nIndex >= m_pnSize[0]) GrowArray(nIndex+ct_nGrowSize);
		return *(m_pPtrArray[0])[nIndex];
	}
	XVector& operator=(XVector &rVector)
	{
		if(&rVector != this)
		{
			// detach from original list first
			Destroy();
			Copy(rVector);
		}
		return *this;
	}
};


template <class T>
void XVector<T>::GrowArray(int nNewSize)
{
	T **pPtrTemp = new T* [nNewSize];
	int i;

	for(i=0;i<m_pnSize[0];i++)
		pPtrTemp[i] = (m_pPtrArray[0])[i];
	while(i < nNewSize)
		pPtrTemp[i++] = new T;

	m_pnSize[0] = nNewSize;

	delete [] (m_pPtrArray[0]);
	m_pPtrArray[0] = pPtrTemp;
}


template <class T>
void XVector<T>::Destroy(void)
{
	if(m_pnRefCnt[0] == 0)
	{
		for(int i=0;i<m_pnSize[0];i++) delete (m_pPtrArray[0])[i];
		delete [] (m_pPtrArray[0]);
		delete m_pPtrArray;
		delete m_pnSize;
		delete m_pnRefCnt;
	}
	else
		m_pnRefCnt[0]--;
}



/////////////////////////////////////////////////////////////////////////////
// XList definition

template <class T>
class XList : public XVector<T>
{
private:
	int m_nNumObjects;

public:
	XList() : m_nNumObjects(0){}

	void Add(T &rObject){(*this)[m_nNumObjects++] = rObject;}
	void Remove(int nIndex);
	void RemoveAll(void){Empty(); m_nNumObjects = 0;}
	int GetCount(void){return m_nNumObjects;}
	// overloaded operators
	XList& operator=(XList &rList)
	{
		if(&rList != this)
		{
			XVector<T>::operator=(rList);
			m_nNumObjects = rList.m_nNumObjects;
		}
		return *this;
	}
};

template <class T>
void XList<T>::Remove(int nIndex)
{
	if(nIndex >= 0 && nIndex < m_nNumObjects)
	{
		// in case old object needs to be destroyed by assignment operator
		// copy garbage from end of list
		for(int i=nIndex+1;i<=m_nNumObjects;i++)
			(*this)[i-1] = (*this)[i];
		m_nNumObjects--;
	}
}



/////////////////////////////////////////////////////////////////////////////
// XSortedList definition

template <class T>
class XSortedList : public XVector<T>
{
private:
	int m_nNumObjects;

public:
	XSortedList() : m_nNumObjects(0){}

	int Find(T &rObject);
	int FindExact(T &rObject)
	{
		int nIndex = Find(rObject);
		if(m_nNumObjects && nIndex < m_nNumObjects &&
				rObject == (*this)[nIndex])
			return nIndex;
		else
			return -1;
	}
	void Add(T &rObject);
	void Remove(int nIndex);
	void RemoveAll(void){Empty(); m_nNumObjects = 0;}
	int GetCount(void){return m_nNumObjects;}
	// overloaded operators
	XSortedList& operator=(XSortedList &rList)
	{
		if(&rList != this)
		{
			XVector<T>::operator=(rList);
			m_nNumObjects = rList.m_nNumObjects;
		}
		return *this;
	}
};

template <class T>
int XSortedList<T>::Find(T &rObject)
{
	int nLow = 0, nHigh = m_nNumObjects-1;
	int nMid;

	if(!m_nNumObjects)
		return 0;
	else if(rObject > (*this)[nHigh])
		return nHigh+1;
	else if(rObject == (*this)[nHigh])
		return nHigh;
	else if(rObject == (*this)[nLow])
		return nLow;

	while(nLow <= nHigh)
	{
		nMid = (nLow + nHigh)/2;
		if(rObject < (*this)[nMid])
			nHigh = nMid - 1;
		else if(rObject > (*this)[nMid])
			nLow = nMid + 1;
		else
			return nMid;
	}

	return nLow;
}

template <class T>
void XSortedList<T>::Add(T &rObject)
{
	int nIndex = Find(rObject);

	(*this)[m_nNumObjects++] = rObject;

	if(nIndex < m_nNumObjects-1)
	{
		for(int i=m_nNumObjects-1;i>nIndex;i--)
			(*this)[i] = (*this)[i-1];

		(*this)[nIndex] = rObject;
	}
}

template <class T>
void XSortedList<T>::Remove(int nIndex)
{
	if(nIndex >= 0 && nIndex < m_nNumObjects)
	{
		// in case old object needs to be destroyed by assignment operator
		// copy garbage from end of list
		for(int i=nIndex+1;i<=m_nNumObjects;i++)
			(*this)[i-1] = (*this)[i];
		m_nNumObjects--;
	}
}



/////////////////////////////////////////////////////////////////////////////
// XStack definition

template <class T>
class XStack : public XVector<T>
{
private:
	int m_nNumObjects;

public:
	XStack() : m_nNumObjects(0){}

	void Push(T &rObject){(*this)[m_nNumObjects++] = rObject;}
	T& Pop(void)
	{
		if(m_nNumObjects) m_nNumObjects--;
//		else throw;
		return (*this)[m_nNumObjects];
	}
	void RemoveAll(void){Empty(); m_nNumObjects = 0;}
	int GetCount(void){return m_nNumObjects;}
	// overloaded operators
	XStack& operator=(XStack &rStack)
	{
		if(&rStack != this)
		{
			XVector<T>::operator=(rStack);
			m_nNumObjects = rStack.m_nNumObjects;
		}
		return *this;
	}
};

#endif

/*******************************************************************
 *
 * $Log: /Deskwin/Src/XVector.h $
 * 
 * 4     10/01/00 9:17 Pese
 * 
 * 3     5/01/00 20:26 Pese
 * 
 * 2     23/12/99 10:16 Pese
 *
 *******************************************************************/
